package com.ca.dso.itpam.webservice.cli;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Vector;

import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient.*;



/** 
CA ITPAM Web Service Command Line Interface
@author George Curran, CA DSO
<br><br>
<b>General Usage:</b>
<blockquote>
<i> caitpamwsu.cmd [/? or /h or /help]</i>
<br>	
<br>Displays general usage.
<br><br>	
<i>	caitpamwsu.cmd <Action Specific Arguments></i>
<br>	
<br>Invokes web service methods silently. Commands that produce output
	will write to STDOUT.  Error messages will be written to STDERR.
	Environment variable "ERRORLEVEL" will be set to one of the
	success/error codes listed below and the success/error code will be
	returned:
<br><br>	
<br>	0	    Completed successfully
<br>	9001	Attempt to initialize CLI utilities failed
<br>	9002	Unexpected Java or web service exception
<br>	9003	IT PAM entity not found or inaccessible
<br>	9004	Method not supported for action/entities specified
<br>	9005	Required argument or parameter not specified
<br>	9006	Unsupported date format specified
<br>	9007	Host specified not found or inaccessible
<br>	9008	Insufficient user permissions
<br>	9009	Unexpected web service exception thrown by web service host
<br>	
<br>	caitpamwsu.cmd [/i] <Action Specific Arguments>
<br>	
<br>Invokes web service methods interactively (i.e., results and errors)
	are displayed on the console.  The "/i" switch must be the first
	argument specified.
</blockquote>
<b>Service Configuration and Authentication:</b>
<blockquote>

	Actions that communicate with the IT PAM web service require the
	user to specify values for the following parameters:
<br>	
<br>user=<IT PAM user name>
<br>password=<IT PAM user password>
<br>uri=<IT PAM web service URI (ex. http://<SERVER>:<PORT>/itpam/soap)>
<br>version=<IT PAM version (ex. 2.1 or 2.2)
<br>	
<br>For security reasons user name and password are not stored and must be
	specified each time the utility is invoked.
<br>
<br>The action, "setServiceConfig", has been provided to store the "uri"
	and "version" to eliminate the need to specify the values each time.
	The saved values may be superseded by specifying the arguments when a
	command is issued. The current stored values may be retrieved using the
	"getServiceConfig" command.
</blockquote>
																									*/
public class ItpamWebServiceCLI extends ItpamWebServiceCLIUtils{
	
/**	Public string constant required by CA																			*/
	public static final String CACOPYRIGHT = "(c) 2009 CA.  All rights reserved.";
	
	
/**
 * Main to support launch from command line.
 * 
 * @param args
 * 																													*/
	public static void main(String[] args)
	{
		try {
			
		//	Display usage if no arguments were specified otherwise initialize CLI utilities
			if (args.length == 0){
				
			//	Display usage
				ItpamWebServiceCLI.displayUsage();				
			}
			else {
				
			//	Initialize client utilities
				ItpamWebServiceCLI.initUtils(args);
				
			//	Record progress
				ItpamWebServiceCLI.logger.debug("Initialized web service utilities and common objects");
			
			//	Attempt to invoke method corresponding to user specified action using reflection
				try {
					
				//	Record progress
					ItpamWebServiceCLI.logger.debug("Execute method '" + strAction + "'");
					
					Class<?> classItpamCLI = Class.forName(ItpamWebServiceCLI.class.getName());
					Method method = classItpamCLI.getDeclaredMethod(ItpamWebServiceCLI.strAction);
					ItpamWebServiceCLI objMethod = new ItpamWebServiceCLI();
					method.invoke(objMethod);
				
				//	Record progress
					ItpamWebServiceCLI.logger.debug("Executed method '" + strAction + "'");
					
				//	Record progress
					ItpamWebServiceCLI.logger.info("Web service CLI request completed successfully");
				}
				catch (InvocationTargetException e) {
					
				//	Attempt to execute method failed
					if (ItpamWebServiceCLI.bolInteractiveMode) {
						
					//	Print exception message to standard error
						if (!ItpamWebServiceCLI.isNullEmpty(e.getCause().getLocalizedMessage())){

							System.err.println(e.getCause().getLocalizedMessage());
						}
						else {
							
							System.err.println(e.getCause().getMessage());
						}
						
					//	Display usage
						ItpamWebServiceCLI.displayUsage();
					}
					
				//	Set exit code
					ItpamWebServiceCLI.setExitCode(e);
					
				//	Record progress
					ItpamWebServiceCLI.logger.error("Web service CLI request failed due to InvocationTargetException");
	
				//	Set failed return code
					System.exit(ItpamWebServiceCLI.intExitCode);
				}
			}
		}
		catch (ItpamCLIParameterMissingOrInvalidException e){
			
		//	Attempt initialize CLI utilities failed; 
			if (ItpamWebServiceCLI.bolInteractiveMode){
				
			//	Print exception message to standard error
				if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){

					System.err.println(e.getLocalizedMessage());
				}
				else {
					
					System.err.println(e.getMessage());
				}
				
			//	Display usage
				ItpamWebServiceCLI.displayUsage();
			}
			
		//	Record progress
			ItpamWebServiceCLI.logger.error("Web service CLI request failed due to ItpamCLIParameterMissingOrInvalidException");
			
		//	Set failed return code
			System.exit(9001);
		}
		catch (Exception e){
			
		//	Unexpected exception occurred 
			if (ItpamWebServiceCLI.bolInteractiveMode){
				
			//	Print exception message to standard error
				if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){

					System.err.println(e.getLocalizedMessage());
				}
				else {
					
					System.err.println(e.getMessage());
				}
				
			//	Display usage
				ItpamWebServiceCLI.displayUsage();
			}
			
		//	Record progress
			ItpamWebServiceCLI.logger.error("Web service CLI request failed due to unexception exception: " + e.getMessage());
			
		//	Set failed return code
			System.exit(ItpamWebServiceCLI.intExitCode);
		}
		
	//	Set successful return code and exit
		System.exit(0);
	}
	
	
/**
 * Abort process specified by process ID.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.2 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair processId=[Process ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void abortProcess() throws Exception
	{
	//	Invoke web service utility method
		ItpamWebServiceCLI.client.abortProcess(ItpamWebServiceCLI.lProcessId);
		
	//	Record success
		logger.info(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
		
	//	Output success message
		System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
	}
	
	
/**
 * Approve pending user interaction request.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair interactionId=[Interaction ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void approvePendingInteraction() throws Exception
	{
	//	Invoke web service client method
		ItpamWebServiceCLI.client.approvePendingInteraction(ItpamWebServiceCLIUtils.lInteractionId, ItpamWebServiceCLIUtils.propertiesParameters);
		
	//	Record success
		logger.info(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
		
	//	Output success message
		System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
	}
	
	
/**
 * Delete archived instances in date range specified.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.2 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair startDate=[Start date of date range]
 * <br>
 * Name/value pair endDate=[End date of date range]
 * <br>
 * <br>
 * Supported date formats 'yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy'
 * <br>
 * Supported time formats 'hh:mm:ss', 'hh:mm'
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote> 
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void deleteArchivedInstances() throws Exception
	{
	//	Invoke web service client method
		ItpamWebServiceCLI.client.deleteArchivedInstances(ItpamWebServiceCLI.strStartDate, ItpamWebServiceCLI.strEndDate);
		
	//	Record success
		logger.info(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
		
	//	Output success message
		System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
	}
	
	
/**
 * Delete specified attachments.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * At least one name/value pair attachmentId=[Attachment ID to be deleted]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void deleteAttachments() throws Exception
	{
	//	Convert attachments ID vector to array of strings
		long[] aryAttachmentIds = null;
		if (ItpamWebServiceCLI.vAttachmentIds.size() > 0){
			
			aryAttachmentIds = new long[ItpamWebServiceCLI.vAttachmentIds.size()];
			for (int intId = 0; intId < ItpamWebServiceCLI.vAttachmentIds.size(); intId++){
				
				aryAttachmentIds[intId] = ItpamWebServiceCLI.vAttachmentIds.get(intId);
			}
		}
		
	//	Invoke web service client method
		ItpamWebServiceCLI.client.deleteAttachments(aryAttachmentIds);
	}
	
	
/**
 * Export objects from IT PAM library to file.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 3.0 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=exportObject
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair sourceLibraryPath=[Source ITPAM library path to search for content to be exported (include object name to export a specific object)]
 * <br>
 * Name/value pair targetPath=[Target file system path for resulting object export file (Note: Content may exported to existing directory on the IT PAM server or an 
				 accessible UNC path)]
 * <br>
 * Name/value pair targetFileName=[Target file name for resulting object export file]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * <br>
 * Name/value pair levelsToSearch=[Integer set levels relative to source library path to search/include for content to be exported (default is 0)]
 * <br>
 * Name/value pair absolutePath=[Flag set to true if library paths in resulting export file should be absolute or false if paths should be relative (default is false)]
 * <br>
 * Name/value pair overwriteTargetFile=[Flag set to true if any existing export file with the same name in the same location should be overwritten (default is false)]
 * <br>
 * Name/value pair(s) objectType=[String set to a valid object type applied as a filter that determines which objects in the source path to include in the exported content <i>(Note: Multiple name/value pairs may be specified)</i>]
 * <br><br>	
 * 		Supported object type filter strings:
 * <br>	- Agenda
 * <br>	- Calendar
 * <br>	- Commander
 * <br>	- CustomIcon
 * <br>	- CustomOperator
 * <br>	- CustomSensor
 * <br>	- Dataset
 * <br>	- InteractionRequestForm
 * <br>	- LogViewer
 * <br>	- Package
 * <br>	- Process
 * <br>	- ProcessWatch
 * <br>	- Resources
 * <br>	- StatePolicy
 * <br>	- System
 * <br></blockquote>
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 * 																								*/
	@SuppressWarnings("unused")
	private static void exportObject() throws Exception
	{
	//	Convert object types vector to array of strings
		String[] aryObjectTypes = null;
		if (ItpamWebServiceCLI.vObjectTypes.size() > 0){
			
			aryObjectTypes = new String[ItpamWebServiceCLI.vObjectTypes.size()];
			for (int intId = 0; intId < ItpamWebServiceCLI.vObjectTypes.size(); intId++){
				
				aryObjectTypes[intId] = ItpamWebServiceCLI.vObjectTypes.get(intId);
			}
		}
		
	//	Invoke web service client method
		ItpamWebServiceCLI.client.exportObject(ItpamWebServiceCLI.strSourceLibraryPath, 
											   ItpamWebServiceCLI.intLevelsToSearch, 
											   ItpamWebServiceCLI.bolAbsolutePath, 
											   ItpamWebServiceCLI.strTargetPath, 
											   ItpamWebServiceCLI.strTargetFileName,
											   ItpamWebServiceCLI.bolTargetPathIsLocal,
											   ItpamWebServiceCLI.bolOverwriteTargetFile, 
											   aryObjectTypes);
				                     
	//	Record success
		logger.info(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
		
	//	Output success message
		System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
	}
	
	
/**
 * Generate an event consumable by waiting IT PAM process operator.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 3.0 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=generateEvent
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * </blockquote>
 * <b>
 * The following parameter(s) must be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair eventName=[Event Name]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * <br>
 * Name/value pair eventType=[Event Type]
 * <br>
 * Name/value pair eventDestination=[Event Destination]
 * <br>
 * Name/value pair eventSource=[Event Source]
 * <br>
 * Name/value pair eventExpirationDuration=[Event Duration in Seconds]
 * <br>
 * Name/value pair eventSingleSubscriber=[True or False to flag event as "consumable" by a single listener only]
 * <br>
 * Name/value pair eventPayload=[Event Pay Load (content to be used by waiting process operator(s)]
 * <br>
 * Name/value pair eventSingleSubscriber=[True or False to flag event as "consumable" by a single listener only]
 * <br>
 * One or more name/value pairs param.[Parameter Name]=[Parameter Value]
 * <br><br>
 * <b>NOTE:</b> Event pay load will be superseded by parameters.
 * <br>
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of event data for the event generated formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void generateEvent()
	throws Exception
	{
	//	Invoke web service client method
		ItpamEventData eventData = ItpamWebServiceCLI.client.generateEvent(ItpamWebServiceCLI.strEventName,
																   		   ItpamWebServiceCLI.strEventType,
																   		   ItpamWebServiceCLI.strEventDestination,
																   		   ItpamWebServiceCLI.strEventSource,
																   		   ItpamWebServiceCLI.lEventExpirationDuration,
																   		   ItpamWebServiceCLI.bolEventSingleSubscriber,
																   		   ItpamWebServiceCLI.strEventPayload,
																   		   ItpamWebServiceCLI.propertiesParameters);

	//	Initialize output String
		String strOutput = ItpamWebServiceCLI.getEventDataString(eventData);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	

/**
 * Get information for existing attachments.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * </blockquote>
 * <b>
 * One or both value(s) for the following parameter(s) must be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair contentId=[Content ID]
 * <br>
 * Name/value pair contentType=[Content Type]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of attachment properties formatted as specified by user matching the filter criteria (Content ID and/or Content Type)
 * specified. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getAttachments() throws Exception
	{
	//	Invoke web service client method
		Vector<ItpamAttachment> vAttachments = ItpamWebServiceCLI.client.getAttachments(ItpamWebServiceCLI.strContentId, ItpamWebServiceCLI.strContentType);
		
	//	Initialize output String
		String strOutput = ItpamWebServiceCLI.getAttachmentsString(vAttachments);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	
	
/**
 * Get event data for events matching criteria specified.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 3.0 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair eventName=[Event Name]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair eventType=[Event Type]
 * <br>
 * Name/value pair eventDestination=[Event Destination]
 * <br>
 * Name/value pair eventSource=[Event Source]
 * <br>
 * Name/value pair eventExpression=[Expression that will be compared to event "payload" parameter(s) (ex. payload.paramName=="paramValue").
 * Note that entire expression must be encapsulated in double quotes and double equal signs ("==") are required.]
 * <br>
 * Name/value pair allowPartialMatch=[Specify true to return data for events that partially match the specified criteria (default is false)]
 * <br>
 * Name/value pair enablePatternMatch=[Specify true if criteria specified should be interpreted as regex-type patterns (default is false)]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of event data formatted as specified by user for events matching criteria specified. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getMatchingEvents() throws Exception
	{
	//	Invoke web service client method
		Vector<ItpamEventData> vEventData = ItpamWebServiceCLI.client.getMatchingEvents(ItpamWebServiceCLI.strEventName,
																   	   					ItpamWebServiceCLI.strEventType,
																   	   					ItpamWebServiceCLI.strEventDestination,
																   	   					ItpamWebServiceCLI.strEventSource,
																   	   					ItpamWebServiceCLI.strEventExpression,
																   	   					ItpamWebServiceCLI.bolAllowPartialMatch,
																   	   					ItpamWebServiceCLI.bolEnablePatternMatch);
	//	Initialize output string
		String strOutput = ItpamWebServiceCLI.getMatchingEventsDataString(vEventData);

	//	Output string to console
		System.out.println(strOutput);
	}
	

/**
 * Get information related to pending user interaction specified.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair interactionId=[Interaction ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of interaction properties formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getPendingInteraction() throws Exception
	{
	//	Initialize output string
		String strOutput = "";
		
	//	Invoke web service client method to get pending interaction object specified
		ItpamInteraction interaction = ItpamWebServiceCLI.client.getPendingInteraction(ItpamWebServiceCLI.lInteractionId);
		
	//	Add interaction to vector to eliminate need to add new parsing method
		Vector<ItpamInteraction> vInteractions = new Vector<ItpamInteraction>();
		vInteractions.add(interaction);
		
	//	Initialize output string
		strOutput = ItpamWebServiceCLI.getProcessPendingInteractionsString(vInteractions);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	
	
/**
 * Get pending interaction information.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of attachment properties formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getPendingInteractions() throws Exception
	{		
	//	Invoke web service client method
		Vector<ItpamInteraction> vInteractions = ItpamWebServiceCLI.client.getPendingInteractions();
	
	//	Initialize output string
		String strOutput = ItpamWebServiceCLI.getProcessPendingInteractionsString(vInteractions);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	

/**
 * Get process logs.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.2 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair processId=[Process ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * <br>
 * Name/value pair level=[Minimum severity of log entries to be return (all messages returned by default)]
 * <br>
 * One or more name/value pairs category=[Log category ("Process", "Operator", "Handler", "Response", "Other", "Custom" (all
 * messages return by default, invalid entries are ignored))
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of log entries formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getProcessLogs() throws Exception
	{
	//	Convert categories vector to array of strings
		String[] aryCategories = null;
		if (ItpamWebServiceCLI.vCategories.size() > 0){
			
			aryCategories = new String[ItpamWebServiceCLI.vCategories.size()];
			for (int intCategory = 0; intCategory < ItpamWebServiceCLI.vCategories.size(); intCategory++){
				
				aryCategories[intCategory] = ItpamWebServiceCLI.vCategories.get(intCategory);
			}
		}
		
	//	Invoke web service client method
		Vector<ItpamProcessLogEntry> vProcessLogEntries = ItpamWebServiceCLI.client.getProcessLogs(ItpamWebServiceCLI.lProcessId,
															                             ItpamWebServiceCLI.strLevel,
															                             aryCategories);
		
	//	Initialize output String
		String strOutput = ItpamWebServiceCLI.getProcessLogEntriesString(vProcessLogEntries);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	

/**
 * Get parameter name/values for process specified.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair processId=[Process ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of parameter properties formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getProcessParameters() throws Exception
	{
	//	Invoke web service client method
		ItpamProcessStatus status = ItpamWebServiceCLI.client.getProcessStatus(ItpamWebServiceCLI.lProcessId);
		
	//	Initialize output String
		String strOutput = ItpamWebServiceCLI.getProcessParametersString(status.getParameters(), null);
		
	//	Output string to console
		System.out.println(strOutput);
	}

/**
 * Get list of pending interactions related to the process ID specified.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair processId=[Process ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of interaction properties formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getProcessPendingInteractions() throws Exception
	{		
	//	Invoke web service client method
		Vector<ItpamInteraction> vInteractions = ItpamWebServiceCLI.client.getProcessPendingInteractions(ItpamWebServiceCLI.lProcessId);
		
	//	Initialize output string
		String strOutput = ItpamWebServiceCLI.getProcessPendingInteractionsString(vInteractions);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	
	
/**
 * Get current status of process specified.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair processId=[Process ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of interaction properties formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getProcessStatus() throws Exception
	{
	//	Invoke web service client method
		ItpamProcessStatus status = ItpamWebServiceCLI.client.getProcessStatus(ItpamWebServiceCLI.lProcessId, ItpamWebServiceCLI.strTagId);
		
	//	Initialize output String
		String strOutput = ItpamWebServiceCLI.getProcessStatusString(status);
		
	//	Output string to console
		System.out.println(strOutput);
	}	
	
/**
 * Control instance of a Process or Start Request Form
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 4.0 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=controlInstance
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Choice of one of the following parameters must be specified via the command line:
 * <br>
 * Name/value pair rootUuid=[Root UUID]
 * <br>
 * Name/value pair instanceId=[Instance ID]
 * <br>
 * Name/value pair tagId=[Tag ID]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Status
 * </blockquote>
 * 
 * @throws Exception
 */
@SuppressWarnings("unused")
private static void controlInstance() throws Exception 
{
	//	Invoke web service client method
	String status = ItpamWebServiceCLI.client.controlInstance(ItpamWebServiceCLI.strRootUuid, ItpamWebServiceCLI.lInstanceId, ItpamWebServiceCLI.strTagId);

	logger.info(status);
	
	//	Output success message
	System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));

}
	
/**
 * Get current status of the ITPAM web service server.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String set to server status.
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getServerStatus() throws Exception
	{
	//	Initialize return String
		String strOutput = ItpamWebServiceCLI.client.getServerStatus();
		
	//	Append uri and version
		strOutput = strOutput + " (" + ItpamWebServiceCLI.strItpamUri + " version " + ItpamWebServiceCLI.strItpamVersion + ")";
		
	//	Output string to console
		System.out.println(strOutput);
	}
	
	
/**
 * Get URI and version properties of the ITPAM web service server.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getServerConfig
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String set to service uri and version.
 * </blockquote>
 * 
 * @throws Exception
 * 																													*/
	@SuppressWarnings("unused")
	private static void getServiceConfig() throws Exception
	{
	//	Initialize output string
		String strOutput = "uri=" + ItpamWebServiceCLI.strItpamUri;
		if(ItpamWebServiceCLI.strItpamVersion != null) {
			strOutput = strOutput + " version=" + ItpamWebServiceCLI.strItpamVersion;
		}
		
		if ( ItpamWebServiceCLI.strTrustStore!=null)
			strOutput += " truststore="+ItpamWebServiceCLI.strTrustStore;
	//	Print output to standard out
		System.out.println(strOutput);
	}
	
	
/**
 * Get start request parameter information.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getStartRequestParameters
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair startRequestName=[Start Request Form Name]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * <br>
 * Name/value pair startRequestPath=[Start Request Form Library Path (default is "/")]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of attachment properties formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getStartRequestParameters() throws Exception
	{		
	//	Invoke web service client method
		Vector<ItpamParameter> vStartRequestParameters = ItpamWebServiceCLI.client.getStartRequestParameters(ItpamWebServiceCLI.strStartRequestName,
																									 		 ItpamWebServiceCLI.strStartRequestPath);
	
	//	Initialize output string
		String strOutput = ItpamWebServiceCLI.getParametersString(vStartRequestParameters, null);
		
	//	Output string to console
		System.out.println(strOutput);
	}


/**
 * Search for and return available start request forms.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair startRequestName=[Start request form name]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * <br>
 * Name/value pair startRequestPath=[ITPAM library search path (default is root ("/"))
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of start request form parameters formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getStartRequestForms() throws Exception
	{
	//	Convert attachments ID vector to array of strings
		String[] aryKeywords = null;
		if (ItpamWebServiceCLI.vKeywords.size() > 0){
			
			aryKeywords = new String[ItpamWebServiceCLI.vKeywords.size()];
			for (int intId = 0; intId < ItpamWebServiceCLI.vKeywords.size(); intId++){
				
				aryKeywords[intId] = ItpamWebServiceCLI.vKeywords.get(intId);
			}
		}
			
	//	Invoke web service client method
		Vector<ItpamStartRequestForm> vStartRequestForms = ItpamWebServiceCLI.client.getStartRequestForms(aryKeywords,
																								ItpamWebServiceCLI.strSearchPath, 
																								ItpamWebServiceCLI.bolRecursive);
		
	//	Initialize output String
		String strOutput = ItpamWebServiceCLI.getStartRequestFormsString(vStartRequestForms);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	
	
/**
 * Get process name referenced by the start request form specified.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair startRequestName=[Start Request Name]
 * <br>
 * Name/value pair startRequestPath=[Start Request Library Path]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String set to fully qualified process name referenced by the start request form specified.
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void getStartRequestProcessReferenced() throws Exception
	{
	//	Invoke web service client method
		String strOutput = ItpamWebServiceCLI.client.getStartRequestProcessReferenced(ItpamWebServiceCLI.strStartRequestName, ItpamWebServiceCLI.strStartRequestPath);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	
	
/**
 * Get start request status object that includes the following information.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair interactionId=[Start Request Interaction ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String of start request status properties formatted as specified by user. 
 * 
 * </blockquote>
 * 
 * @throws Exception
*/
	@SuppressWarnings("unused")
	private static void getStartRequestStatus() throws Exception
	{
	//	Invoke web service client method
		ItpamStartRequestStatus status = ItpamWebServiceCLI.client.getStartRequestStatus(ItpamWebServiceCLI.lStartRequestId, ItpamWebServiceCLI.strTagId);
		
	//	Initialize output string
		String strOutput = ItpamWebServiceCLI.getStartRequestStatusString(status);
		
	//	Output string to console
		System.out.println(strOutput);
	}
	
	
/**
 * Import file into ITPAM library.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.2 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=importObject
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair sourcePathFileName=[Path/file name of source object definition file to be imported to be imported]
 * <br>
 * Name/value pair targetLibraryPathName=[Target IT PAM library path/name of the object to be created]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * <br>
 * Name/value pair importMakeAvailable=[True or False to make the object(s) resulting from the import available for use]
 * <br>
 * Name/value pair importMakeCurrent=[True or False to make the object(s) resulting from the import the current version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
*/
	@SuppressWarnings("unused")
	private static void importObject() throws Exception
	{
	//	Invoke web service client method
		ItpamWebServiceCLI.client.importObject(ItpamWebServiceCLI.strImportSourcePathFileName,
											   ItpamWebServiceCLI.strImportTargetLibraryPathName,
				                     		   ItpamWebServiceCLI.bolImportMakeAvailable,
				                     		   ItpamWebServiceCLI.bolImportMakeCurrent,
				                     		   ItpamWebServiceCLI.strImportOvewriteAction);
				                     
	//	Record success
		logger.info(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
		
	//	Output success message
		System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
	}
	
/**
 * Reject pending user interaction request.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair interactionId=[Interaction ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void rejectPendingInteraction() throws Exception
	{
	//	Invoke web service client method
		ItpamWebServiceCLI.client.rejectPendingInteraction(ItpamWebServiceCLI.lInteractionId);
	
	//		Record success
		logger.info(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
		
	//	Output success message
		System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
	}
	
	
/**
 * Resume process specified by process ID.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.2 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair processId=[Process ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void resumeProcess() throws Exception
	{
	//	Invoke web service client method
		ItpamWebServiceCLI.client.resumeProcess(ItpamWebServiceCLI.lProcessId);
	}
	
	
/**
 * Set URI and version properties of the ITPAM web service server.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=setServerConfig
 * <br>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 * 																													*/
	@SuppressWarnings("unused")
	private static void setServiceConfig() throws Exception
	{
	//	Invoke utility method
		ItpamWebServiceCLI.storeServiceConfig();
		
	//	Output success message
		System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
	}
	
	
/**
 * Start process specified.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair processName=[Fully qualified process name]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * <br>
 * One or more name/value pairs param.[Parameter Name]=[Parameter Value]
 * <br>
 * Name/value pair startTime=[Start time ('hh:mm:ss', 'hh:mm') (default is now)]
 * <br>
 * Name/value pair startDate=[Start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') required if "startTime" specified
 * (default is today)]
 * <br>
 * One or more name/value pairs attach.[Attachment Name]=[Attachment URI]
 * <br>
 * Name/value pairs attachmentParameterName=[Parameter Name to link attachments to (required if on or more attachment
 * parameters are specified]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String set to process ID.
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void startProcess() throws Exception
	{		
	//	Initialize return value
		long lReturn;
		
	//	Invoke web service method	
		lReturn = ItpamWebServiceCLI.client.startProcess(ItpamWebServiceCLI.strProcessName,
									 	         ItpamWebServiceCLI.propertiesParameters,
									 	         ItpamWebServiceCLI.strStartDate,
									 	         ItpamWebServiceCLI.strStartTime,
									 	         ItpamWebServiceCLI.strTagId,
									 	         ItpamWebServiceCLI.bolIsAutoArchive,
									 	         ItpamWebServiceCLI.strPriority,
									 	         ItpamWebServiceCLI.strAttachmentParameterName,
									 	         ItpamWebServiceCLI.propertiesAttachments);

	//	Print return to standard out
		System.out.println(lReturn);
	}
	
	
/**
 * Initiate process via start request form.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.1 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair startRequestName=[Start Request Name]
 * <br>
 * Name/value pair startRequestPath=[Start Request Path]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * <br>
 * One or more name/value pairs param.[Parameter Name]=[Parameter Value]
 * <br>
 * Name/value pair startTime=[Start time ('hh:mm:ss', 'hh:mm') (default is now)]
 * <br>
 * Name/value pair startDate=[Start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') required if "startTime" specified
 * (default is today)]
 * <br>
 * One or more name/value pairs attach.[Attachment Name]=[Attachment URI]
 * <br>
 * Name/value pairs attachmentParameterName=[Parameter Name to link attachments to (required if on or more attachment
 * parameters are specified]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * String set to start request interaction ID.
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void startRequest() throws Exception
	{		
	//	Initialize return value
		long lReturn;
		
	//	Invoke web service method	
		lReturn = ItpamWebServiceCLI.client.startRequest(ItpamWebServiceCLI.strStartRequestName, 
												 ItpamWebServiceCLI.strStartRequestPath,
									 	         ItpamWebServiceCLI.propertiesParameters,
									 	         ItpamWebServiceCLI.strStartDate,
									 	         ItpamWebServiceCLI.strStartTime,
									 	         ItpamWebServiceCLI.strTagId,
									 	         ItpamWebServiceCLI.bolIsAutoArchive,
									 	         ItpamWebServiceCLI.strPriority,
									 	         ItpamWebServiceCLI.strAttachmentParameterName,
									 	         ItpamWebServiceCLI.propertiesAttachments);

	//	Print return to standard out
		System.out.println(lReturn);
	}
	
/**
 * Suspend process specified by process ID.
 * <br><br>
 * <b>
 * IT PAM version required:
 * </b>
 * <blockquote>
 * 2.2 or higher
 * </blockquote>
 * <b>
 * Requires value(s) for the following parameter(s) be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair action=getPendingInteractions
 * <br>
 * Name/value pair user=[ITPAM User Name]
 * <br>
 * Name/value pair password=[ITPAM User Password]
 * <br>
 * Name/value pair processId=[Process ID]
 * </blockquote>
 * <b>
 * Optional value(s) for the following parameter(s) may be specified via the command line:
 * </b>
 * <blockquote>
 * Name/value pair uri=[URI for ITPAM web service (ex. http://[Server]:[Port]/itpam/soap]
 * <br>
 * Name/value pair version=[ITPAM version]
 * </blockquote>
 * <b>
 * Output when successful:
 * </b>
 * <blockquote>
 * Message indicating command completed successfully
 * </blockquote>
 * 
 * @throws Exception
 */
	@SuppressWarnings("unused")
	private static void suspendProcess() throws Exception
	{
	//	Invoke web service client method
		ItpamWebServiceCLI.client.suspendProcess(ItpamWebServiceCLI.lProcessId);

	//	Record success
		logger.info(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
		
	//	Output success message
		System.out.println(ItpamWebServiceCLI.messages.getLocalized("I_REQUEST_COMPLETED_SUCCESSFULLY"));
	}
}
