package com.ca.dso.itpam.webservice.cli;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;

import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient.*;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClientFactory;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClientFactory.*;
import com.ca.dso.resource.utils.Messages;
import com.ca.dso.resource.utils.Usage;

public class ItpamWebServiceCLIUtils extends ItpamWebServiceCLIConstants {
	
/**	Public string constant required by CA																			*/
	public static final String CACOPYRIGHT = "(c) 2009 CA.  All rights reserved.";
	
/**
 * Methods to read contents from properties files (files consisting of name/value pairs delimited with a 			
 * "equals (=)" sign) and load or append to a Java "Properties" object without corrupting entries containing
 * special characters (particularly back slashes.																	*/

	private static class PropertyFileReader {

	/**	Public string constant required by CA																		*/
		@SuppressWarnings("unused")
		public static final String CACOPYRIGHT = "(c) 2009 CA.  All rights reserved.";
		
	/**	
	 * Append the contents of the specified properties file (file made up of
	 * name value pairs delimited with an equal ("=") sign) to the Java
	 * properties object specified.
	 * <p>
	 * Note even though Java properties object does support a "load" method
	 * to perform the operation, back slash characters are stripped from the
	 * properties.  Since paths on Windows use back slashes as a delimiter by
	 * convention, using the load method would corrupt such settings.  In order
	 * to prevent alteration of the values, the file is read, parsed and the
	 * "setProperty" method is used.
	 * 
	 * @param strPropertiesFileName - String set to path and file name
	 * @param properties - Properties object file content should be appended to
	 * <p>
	 * @return Properties object loaded with original contents and contents of
	 * properties file specified
	 * <p>
	 * @throws Exception
	 *  																		*/
		public static synchronized Properties appendPropertiesFromFile(String strPropertiesFileName, Properties properties) throws Exception{
			
		//	Process file contents	
			try	{
			//	Initialize properties file reader
				FileReader readerFile = new FileReader(strPropertiesFileName);
				
			//	Initialize properties buffer reader
				BufferedReader readerBuffer = new BufferedReader(readerFile);
				
			//	Initialize property string
				String strProperty;
				
				while ((strProperty = readerBuffer.readLine()) != null){
				
				//	Split property string on equal sign
					String[] aryProperty = strProperty.split("=", 2);
						
				//	Process property string if value name value pair
					if(aryProperty.length == 2){
						properties.setProperty(aryProperty[0].trim(), aryProperty[1].trim());
					}
				}
			}
			catch (FileNotFoundException eFileNotFound){

				throw eFileNotFound;
			} 
			catch (IOException eIOException){
				
				throw eIOException;
			}
			
			return properties;
		}
		
		
	/**	
	 * Load the contents of the specified properties file (file made up of
	 * name value pairs delimited with an equal ("=") sign) into a Java
	 * properties object.
	 * <p>
	 * Note even though Java properties object does support a "load" method
	 * to perform the operation, back slash characters are stripped from the
	 * properties.  Since paths on Windows use back slashes as a delimiter by
	 * convention, using the load method would corrupt such settings.  In order
	 * to prevent alteration of the values, the file is read, parsed and the
	 * "setProperty" method is used.
	 * 
	 * @param strPropertiesPathFileName - String set to path and file name
	 * <p>
	 * @return Properties object loaded with original contents and contents of
	 * properties file specified
	 * <p>
	 * @throws Exception
	 *  																		*/
		public static synchronized Properties loadPropertiesFromFile(String strPropertiesPathFileName) throws Exception{
			
		//	Initialize properties object
			Properties properties = new Properties();
			
		//	Process file contents	
			try	{
				
			//	Initialize properties file reader
				appendPropertiesFromFile(strPropertiesPathFileName, properties);
			}
			catch (Exception e){

				throw e;
			}
			
			return properties;
		}

	}

/**	Exception class instantiated and thrown when a required parameter is missing or invalid.						*/
	protected static class ItpamCLIParameterMissingOrInvalidException extends Exception {

		private static final long serialVersionUID = -8201878518626987430L;
		
		private String m_strMessage = null;
		private String m_strLocalizedMessage = null;

	/** Constructor 														*/
		public ItpamCLIParameterMissingOrInvalidException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}

/**	Exception class instantiated and thrown when the return from the web service client is unexpected.				*/
	protected static class ItpamCLIUnexpectedReturnException extends Exception {

		private static final long serialVersionUID = -8201878518626987430L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamCLIUnexpectedReturnException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}		
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}
	
/**	ITPAM web service client wrapper used to communicate with ITPM server											*/
	protected static ItpamWebServiceClient client = null;
	
/**	Logger used to record troubleshooting messages																	*/
	protected static Logger logger = null;

/**	Boolean to store user specified set absolute path for export value												*/
	protected static boolean bolAbsolutePath = false;
	
/**	Boolean to store user specified allow partial match value														*/
	protected static boolean bolAllowPartialMatch = false;
	
/** Boolean to store user specified quiet mode flag																	*/
	protected static boolean bolHelpRequested = false;
	
/**	Boolean to store user specified enable pattern match value														*/
	protected static boolean bolEnablePatternMatch = false;
	
/** Boolean to store user specified quiet mode flag																	*/
	protected static boolean bolInteractiveMode = false;

/**	Boolean to store user specified import make available value														*/
	protected static boolean bolImportMakeAvailable = false;

/** Boolean to store user specified import make current value														*/
	protected static boolean bolImportMakeCurrent = false;

/**	Boolean to store user specified overwrite target file value														*/
	protected static boolean bolOverwriteTargetFile = false;

/**	Boolean to store user specified recursive value																	*/
	protected static boolean bolRecursive = false;
	
/** Boolean to store user specified single subscriber value															*/
	protected static boolean bolEventSingleSubscriber = false;
	
/**	Boolean to store user specified recursive value																	*/
	protected static boolean bolTargetPathIsLocal = false;
	
/**	int to store levels to search in source library																	*/
	protected static int intLevelsToSearch = 0;
	
/**	int to store exit code																							*/
	protected static int intExitCode = 9999;
	
/**	long to store event expiration duration																			*/
	protected static long lEventExpirationDuration = 0;
	
/**	long to store user specified interaction ID																	*/
	protected static long lInteractionId = 0;

/**	long to store user specified process ID																		*/
	protected static long lProcessId = 0;

/**	long to store user specified request ID																		*/
	protected static long lStartRequestId = 0;

/**	Messages object to be linked to appropriate resource bundle														*/
	protected static Messages messages = null;

/**	Properties object to store user specified parameter name/value pairs to be passed with ITPAM requests			*/
	protected static Properties propertiesParameters = null;
	
/**	Properties object to store user specified attachment name/value pairs to be passed with ITPAM requests			*/
	protected static Properties propertiesAttachments = null;

/**	String to store user specified action																			*/
	protected static String strAction = null;
	
/**	String to store user specified action file path and name														*/
	protected static String strActionFile = null;
	
/**	String to store user specified attachment parameter name														*/
	protected static String strAttachmentParameterName = null;
	
/**	String to store user specified content ID																		*/
	protected static String strContentId = null;
	
/**	String to store user specified content type																		*/
	protected static String strContentType = null;
	
/**	String to store user specified end date																			*/
	protected static String strEndDate = null;
	
/**	String to store user specified event destination																*/
	protected static String strEventDestination = null;
	
/**	String to store user specified event expression																	*/
	protected static String strEventExpression = null;	
	
/**	String to store user specified event name																		*/
	protected static String strEventName = null;
	
/**	String to store user specified event pay load																	*/
	protected static String strEventPayload= null;
	
/**	String to store user specified event source																		*/
	protected static String strEventSource = null;
	
/**	String to store user specified event type																		*/
	protected static String strEventType = null;
	
/**	String to store user specified import source path/file name														*/
	protected static String strImportSourcePathFileName = null;
	
/**	String to store user specified import target library path/name													*/
	protected static String strImportTargetLibraryPathName = null;

/** String to store user specified import overwrite action															*/
	protected static String strImportOvewriteAction = null;
	
/** String to store ITPAM user password																				*/
	protected static String strItpamPassword = null;
	
/**	String to store ITPAM target end point URI																		*/
	protected static String strItpamUri = null;

/** String to store ITPAM user name																					*/
	protected static String strItpamUser = null;
	
/**	String to store ITPAM target server version																		*/
    protected static String strItpamVersion = null;
    
/**	String to store user specified log level																		*/
	protected static String strLevel = null;
	
/**	String to store user specified output format																	*/
	protected static String strOutputFormat = ItpamWebServiceCLIUtils.DEFAULT_OUTPUT_FORMAT;
	
/**	String to store user specified process name																		*/
	protected static String strProcessName = null;
	
/**	String to store user specified search path																		*/
	protected static String strSearchPath = null;
	
/**	String to store user specified start date																		*/
	protected static String strStartDate = null;
	
/**	String to store user specified export source library path														*/
	protected static String strSourceLibraryPath = null;
	
/** String to store user specified priority																			*/
	protected static String strPriority = null;

/** Boolean to store user specified isAutoArchive																	*/
	protected static boolean bolIsAutoArchive = true;
	
/** String to store tag ID																							*/
	protected static String strTagId = null;
	
/** String to store Root UUID																						*/
	protected static String strRootUuid = null;

/** long to store Instance ID																						*/
	protected static long lInstanceId = 0;

/**	String to store user specified start request name																*/
	protected static String strStartRequestName = null;
	
/**	String to store user specified start request path																*/
	protected static String strStartRequestPath = null;
	
/**	String to store user specified start time																		*/
	protected static String strStartTime = null;
	
/**	String to store user specified export target file name															*/
	protected static String strTargetFileName = null;
	
/**	String to store user specified export target path																*/
	protected static String strTargetPath = null;
	
/**	String to store usage resource bundle name 																		*/
	protected static String strUsageResourceBundle = ItpamWebServiceCLI.DEFAULT_USAGE_RESOURCE_BUNDLE_NAME;
	
/**	Vector to store dynamically built list of supported actions														*/
	protected static Vector<String> vActionsSupported = new Vector<String>();
	
/**	Vector to store user specified attachment IDs																	*/
	protected static Vector<Long> vAttachmentIds = new Vector<Long>();

/**	Vector to store user specified categories																		*/
	protected static Vector<String> vCategories = new Vector<String>();

/**	Vector to store user specified keywords																			*/
	protected static Vector<String> vKeywords = new Vector<String>();

/**	Vector to store user specified export object type filter strings												*/
	protected static Vector<String> vObjectTypes = new Vector<String>();

	/** String to store the trust store file name\path */
	protected static String strTrustStore = null;
	
	/** String to store the trust store password */
	protected static String strTrustStorePassword = null;
	
	
	
/**
 * Add user specified attachment name and value.
 * 
 * @param strName - String set to parameter name
 * @param strValue - String set to parameter value
 * 																													*/
	private static void addAttachment(String strName, String strValue){
		
	//	Initialize properties object if null
		if (ItpamWebServiceCLIUtils.propertiesAttachments == null) ItpamWebServiceCLIUtils.propertiesAttachments = new Properties();
		
	//	Remove prefix from parameter name
		strName = strName.split("[.]")[1];
		
	//	Add property
		ItpamWebServiceCLIUtils.propertiesAttachments.setProperty(strName, strValue);
	}
	
/**
 * Add user specified category.
 * 
 * @param strValue - String set to category
 * 																													*/
	private static void addAttachmentId(String strValue) throws Exception {
		
		try {
			
			if (!ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

				ItpamWebServiceCLIUtils.vAttachmentIds.add(Long.parseLong(strValue));
			}
		}
		catch (Exception e){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_EVENT_EXPIRATION_DURATION + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_EVENT_EXPIRATION_DURATION + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
/**
 * Add user specified parameter name and value.
 * 
 * @param strName - String set to parameter name
 * @param strValue - String set to parameter value
 * 																													*/
	private static void addParameter(String strName, String strValue){
		
	//	Initialize properties object if null
		if (ItpamWebServiceCLIUtils.propertiesParameters == null) ItpamWebServiceCLIUtils.propertiesParameters = new Properties();
		
	//	Remove prefix from parameter name
		strName = strName.split("[.]")[1];
		
	//	Add property
		ItpamWebServiceCLIUtils.propertiesParameters.setProperty(strName, strValue);
	}


/**
 * Add user specified category.
 * 
 * @param strCategory - String set to category
 * 																													*/
	private static void addCategory(String strCategory){
		
		ItpamWebServiceCLIUtils.vCategories.add(strCategory);
	}
	
	
/**
 * Add user specified keyword.
 * 
 * @param strKeyword - String set to keyword
 * 																													*/
	private static void addKeyword(String strKeyword){
		
		ItpamWebServiceCLIUtils.vCategories.add(strKeyword);
	}
	
	
/**
 * Add user specified object type.
 * 
 * @param strObjectType - String set to object type
 * 																													*/
	private static void addObjectType(String strObjectType){
		
		ItpamWebServiceCLIUtils.vObjectTypes.add(strObjectType);
	}
	
	
	private static String[] appendArgumentsFromActionFile(String[] aryArgs)
	throws Exception
	{		
	//	Initialize boolean flag
		boolean bolActionFileSpecified = false;
		
	//	Initialize action file string
		String strActionFile = null;
		
		try {
			
		//	Inspect arguments to determine if an action file was specified
			for (int intArg = 0; intArg < aryArgs.length; intArg++){
				
				String strArg = aryArgs[intArg];
				if (strArg.toLowerCase().startsWith(ItpamWebServiceCLI.ARG_NAME_ACTION_FILE.toLowerCase())){
					
					bolActionFileSpecified = true;
					strActionFile = strArg.split("=")[1];
					break;
				}
			}
			
		//	Process action file if specified
			if (bolActionFileSpecified){
				
			//	Set action to "actionFile"
				ItpamWebServiceCLI.strAction = "actionFile";
				
			//	Record progress
				ItpamWebServiceCLI.logger.debug("Loading parameters from action file '" + strActionFile + "'");
				
			//	Load action file content into properties object
				Properties propertiesAction = PropertyFileReader.loadPropertiesFromFile(strActionFile);
				
			//	Append command line arguments to properties object
			//	(Note: Command line argument values will supersede action file argument values)
				for (int intArg = 0; intArg < aryArgs.length; intArg++){
					
					String[] aryArg = aryArgs[intArg].split("=");
					if (aryArg.length == 2){
						
						propertiesAction.setProperty(aryArg[0], aryArg[1]);						
					}
				}
				
			//	Record progress
				ItpamWebServiceCLI.logger.debug("Loaded " + propertiesAction.size() + " parameters from action file '" + strActionFile + "'");
				
			//	Resize the arguments array
				aryArgs = new String[propertiesAction.size()];
				
			//	Initialize arrays of property names and values
				Object[] aryActionPropertyNames = propertiesAction.keySet().toArray();
				Object[] aryActionPropertyValues = propertiesAction.values().toArray();
				
			//	Transfer properties to arguments array
				for (int intProperty = 0; intProperty < aryActionPropertyNames.length; intProperty++){
					
					aryArgs[intProperty] = ((String) aryActionPropertyNames[intProperty] + "=" + (String) aryActionPropertyValues[intProperty]).trim();				
				}
		
			}
		}
		catch (FileNotFoundException e) {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_MISSING_ACTION_FILE");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_MISSING_ACTION_FILE") + ": " + e.getMessage();
			
		//	Record progress
			ItpamWebServiceCLI.logger.error(strMessage);
						
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);			
		}
		catch (Exception e) {

			//	Set exception messages
				String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_ACTION_FILE");
				if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
					
					strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
				}
				
				String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_ACTION_FILE") + ": " + e.getMessage();
				
			//	Record progress
				ItpamWebServiceCLI.logger.error(strMessage);
				
			//	throw exception
				throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		
		
	//	Return updated arguments array
		return aryArgs;
	}
	
	
/**
 * Display usage and syntax to the console.
 * 																													*/
	protected static void displayUsage(){
		
		try {
			
		//	Display usage
			Usage.display(ItpamWebServiceCLI.strUsageResourceBundle);
		} 
		catch (Exception e) {
			
		//	Initialize exception message
			String strMessage = "Required usage resource file inaccessible or not found; "
		          			  + ItpamWebServiceCLI.strUsageResourceBundle;

		//	Print exception message to standard error
			System.err.println(strMessage);
			
		//	Record exceptions
			logger.error(strMessage);
		}
		
	}
	
	
/**
 * Parse vector of ItpamAttachment objects and return String formatted as specified by the user.
 * 
 * @param vAttachments - Vector encapsulating ItpamAttachment objects
 * 
 * @return String of attachment info formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getAttachmentsString(Vector<ItpamAttachment> vAttachments)
	throws Exception
	{		
	//	Initialize return string
		String strReturn = "";	
		
	//	Initialize indent, name and value strings
		String strIndent = "";
		String strName = "";
		String strValue = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Iterate vector of interactions
				for (Iterator<ItpamAttachment> iAttachments = vAttachments.iterator(); iAttachments.hasNext();){
					
				//	Get/set interaction object
					ItpamAttachment attachment = iAttachments.next();
					
				//	Get/set parameter name prefix
					String strPrefix = attachment.getAttachmentId() + ".";
					
				//	Append tab delimited attachment properties
					strName = ItpamWebServiceCLIUtils.ARG_NAME_ATTACHMENT_ID;
					strValue = attachment.getAttachmentId();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;	
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_CONTENT_ID;
					strValue = attachment.getContentId();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_CONTENT_TYPE;
					strValue = attachment.getContentType();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_ATTACHMENT_NAME;
					strValue = attachment.getName();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strReturn = strReturn + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				}			
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "attachments" element tag
				strReturn = strReturn
						  + strIndent
						  + "<attachments";
				
				if (!ItpamWebServiceCLIUtils.isNullEmpty(ItpamWebServiceCLIUtils.strContentId)){
					
					strReturn = strReturn + " " + ItpamWebServiceCLI.ARG_NAME_CONTENT_ID + "='" + ItpamWebServiceCLI.strContentId + "'";
				}
				
				if (!ItpamWebServiceCLIUtils.isNullEmpty(ItpamWebServiceCLIUtils.strContentId)){
					
					strReturn = strReturn + " " + ItpamWebServiceCLI.ARG_NAME_CONTENT_TYPE + "='" + ItpamWebServiceCLI.strContentType + "'";
				}
				
				strReturn = ">" + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;;
						  
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLI.DELIMITER_XML_INDENT;
				
			//	Iterate vector of interactions
				for (Iterator<ItpamAttachment> iAttachments = vAttachments.iterator(); iAttachments.hasNext();){
					
				//	Get/set interaction object
					ItpamAttachment attachment = iAttachments.next();
					
				//	Append opening "interaction" element string
					strReturn = strReturn
							  + strIndent
							  + "<attachment"
							  + " id='" + attachment.getAttachmentId() + "'"
							  + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
				//	Increase indent
					strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
					
				//	Append child property element strings
					strName = ItpamWebServiceCLIUtils.ARG_NAME_ATTACHMENT_ID;
					strValue = attachment.getAttachmentId();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_CONTENT_ID;
					strValue = attachment.getContentId();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_CONTENT_TYPE;
					strValue = attachment.getContentType();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_ATTACHMENT_NAME;
					strValue = attachment.getName();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;				
					
				//	Decrease indent
					strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");	
					
				//	Append closing "attachment" element tag
					strReturn = strReturn
					          + strIndent
					          + "</attachment>"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				}
				
			//	Append closing "Interactions" element tag
				strReturn = strReturn + "</attachments>";
			}
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_STATUS_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_STATUS_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Parse ItpamEventData object and return String formatted as specified by the user.
 * 
 * @param eventData - ItpamEventData object data object
 * 
 * @return String of event data info formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getEventDataString(ItpamEventData eventData)
	throws Exception
	{		
	//	Initialize return string
		String strReturn = "";	
		
	//	Initialize indent, name and value strings
		String strIndent = "";
		String strName = "";
		String strValue = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
							
			//	Append delimited event data properties
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_NAME;
				strValue = eventData.getName();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_CREATION_TIME;
				strValue = eventData.getCreationTimeString();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_EXPIRATION_TIME;
				strValue = eventData.getExpirationTimeString();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_DESTINATION;
				strValue = eventData.getDestination();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_EXPRESSION;
				strValue = eventData.getExpression();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_SINGLE_SUBSCRIBER;
				strValue = eventData.getSingleSubscriberString();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_SOURCE;
				strValue = eventData.getSource();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_TYPE;
				strValue = eventData.getType();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_USER;
				strValue = eventData.getUser();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strReturn += ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;	
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "event" element string
				strReturn = strReturn + strIndent + "<event>" + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
				
			//	Append child property element strings
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_NAME;
				strValue = eventData.getName();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_CREATION_TIME;
				strValue = eventData.getCreationTimeString();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_EXPIRATION_TIME;
				strValue = eventData.getExpirationTimeString();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_DESTINATION;
				strValue = eventData.getDestination();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_EXPRESSION;
				strValue = eventData.getExpression();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_SINGLE_SUBSCRIBER;
				strValue = eventData.getSingleSubscriberString();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_SOURCE;
				strValue = eventData.getSource();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_TYPE;
				strValue = eventData.getType();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

				strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_USER;
				strValue = eventData.getUser();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent
						  + "<" + strName + ">"
						  + strValue.replace("\n", "")
						  + "</" + strName + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Decrease indent
				strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");	
				
			//	Append closing "eventData" element tag
				strReturn = strReturn
				          + strIndent
				          + "</event>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
			}
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_EVENT_DATA_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_EVENT_DATA_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Parse vector of ItpamEventData objects and return String formatted as specified by the user.
 * 
 * @param vEventDataObjects - Vector encapsulating ItpamEventData objects
 * 
 * @return String of event data info formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getMatchingEventsDataString(Vector<ItpamEventData> vEventDataObjects)
	throws Exception
	{		
	//	Initialize return string
		String strReturn = "";	
		
	//	Initialize indent, name and value strings
		String strIndent = "";
		String strName = "";
		String strValue = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Iterate vector of interactions
				for (Iterator<ItpamEventData> iEventDataObjects = vEventDataObjects.iterator(); iEventDataObjects.hasNext();){
					
				//	Get/set interaction object
					ItpamEventData eventData = iEventDataObjects.next();
					
				//	Get/set parameter name prefix
					String strPrefix = "event" + iEventDataObjects.hashCode() + ".";
					
				//	Append delimited event data properties
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_NAME;
					strValue = eventData.getName();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_CREATION_TIME;
					strValue = eventData.getCreationTimeString();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_EXPIRATION_TIME;
					strValue = eventData.getExpirationTimeString();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_DESTINATION;
					strValue = eventData.getDestination();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_EXPRESSION;
					strValue = eventData.getExpression();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_SINGLE_SUBSCRIBER;
					strValue = eventData.getSingleSubscriberString();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_SOURCE;
					strValue = eventData.getSource();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_TYPE;
					strValue = eventData.getType();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_USER;
					strValue = eventData.getUser();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strReturn += ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				}			
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "events" element tag
				strReturn = strReturn + strIndent + "<events>" + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;;
						  
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLI.DELIMITER_XML_INDENT;
				
			//	Iterate vector of interactions
				for (Iterator<ItpamEventData> iEventDataObjects = vEventDataObjects.iterator(); iEventDataObjects.hasNext();){
					
				//	Get/set interaction object
					ItpamEventData eventData = iEventDataObjects.next();
					
				//	Append opening "event" element string
					strReturn = strReturn + strIndent + "<event>" + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
				//	Increase indent
					strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
					
				//	Append child property element strings
					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_NAME;
					strValue = eventData.getName();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_CREATION_TIME;
					strValue = eventData.getCreationTimeString();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_EXPIRATION_TIME;
					strValue = eventData.getExpirationTimeString();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_DESTINATION;
					strValue = eventData.getDestination();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_EXPRESSION;
					strValue = eventData.getExpression();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_SINGLE_SUBSCRIBER;
					strValue = eventData.getSingleSubscriberString();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_SOURCE;
					strValue = eventData.getSource();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_TYPE;
					strValue = eventData.getType();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;

					strName = ItpamWebServiceCLIUtils.ARG_NAME_EVENT_USER;
					strValue = eventData.getUser();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
				//	Decrease indent
					strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");	
					
				//	Append closing "eventData" element tag
					strReturn = strReturn
					          + strIndent
					          + "</event>"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				}
				
			//	Append closing "Interactions" element tag
				strReturn = strReturn + "</events>";
			}
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_EVENT_DATA_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_EVENT_DATA_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Process vector of ItpamParameter objects and return String formatted as specified by the user.
 * 
 * @param vParameters - Vector of ItpamParameterObjects
 * 
 * @return String of parameter properties and values formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getParametersString(Vector<ItpamParameter> vParameters, String strIndent)
	throws Exception
	{
	//	Initialize return string
		String strReturn = "";
		
	//	Update indent string
		if (strIndent == null) strIndent = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Iterate parameter properties
				for (Iterator<ItpamParameter> iParameters = vParameters.iterator(); iParameters.hasNext();){
					
				//	Initialize ITPAM parameter object
					ItpamParameter parameter = iParameters.next();
					
				//	Re-initialize attribute object and strings
					String strParameterName = "";
					String strAttribute = "";
					String strAttributes = "";
					String strAttributeName = "";
					Object objAttributeValue;
					
				//	Get/set parameter name string
					if(!ItpamWebServiceCLIUtils.isNullEmpty(parameter.getName())){
						strParameterName = strParameterName + parameter.getName() + ".";
					}
					
				//	Append entry for attribute: defaultValue
					strAttributeName = "defaultValue";
					objAttributeValue = parameter.getDefaultValue();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: description
					strAttributeName = "description";
					objAttributeValue = parameter.getDescription();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: isReadOnly
					strAttributeName = "isReadOnly";
					objAttributeValue = parameter.getIsReadOnly();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: label
					strAttributeName = "label";
					objAttributeValue = parameter.getLabel();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: mask
					strAttributeName = "mask";
					objAttributeValue = parameter.getMask();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: maxLength
					strAttributeName = "maxLength";
					objAttributeValue = parameter.getMaxLength();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: maxValue
					strAttributeName = "maxValue";
					objAttributeValue = parameter.getMaxValue();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: minLength
					strAttributeName = "minLength";
					objAttributeValue = parameter.getMinLength();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: minValue
					strAttributeName = "minValue";
					objAttributeValue = parameter.getMinValue();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: name
					strAttributeName = "name";
					objAttributeValue = parameter.getName();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: parentPageName
					strAttributeName = "parentPageName";
					objAttributeValue = parameter.getParentPageName();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: sequenceNumber
					strAttributeName = "sequenceNumber";
					objAttributeValue = parameter.getSequenceNo();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Append entry for attribute: value
					strAttributeName = "value";
					objAttributeValue = parameter.getValue();
					strAttribute = strAttributeName + "=" + objAttributeValue;
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + strParameterName + strAttribute + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
				//	Reset all values set to the literal string "null" to an empty string
					strAttributes = strAttributes.replaceAll("=null", "=");
					
				//	Append parameter attributes to return string
					if (!ItpamWebServiceCLIUtils.isNullEmpty(strReturn)) {strReturn = strReturn + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;}
					strReturn = strReturn + strAttributes;
				}
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "parameters" element tag
				strReturn = strReturn
						  + strIndent
						  + "<parameters "
						  + ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_NAME
						  + "=\"" + ItpamWebServiceCLI.strStartRequestName + "\" "
						  + ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_PATH
						  + "=\"" + ItpamWebServiceCLI.strStartRequestPath + "\">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
				
			//	Process parameter information
				for (Iterator<ItpamParameter> iParameters = vParameters.iterator(); iParameters.hasNext();){
					
				//	Initialize ITPAM parameter object
					ItpamParameter parameter = iParameters.next();
					
				//	Re-initialize attribute object and strings
					String strAttribute = "";
					String strAttributes = "";
					String strAttributeName = "";
					Object objAttributeValue;
					
				//	Append entry for attribute: name
					strAttributeName = "name";
					objAttributeValue = parameter.getName();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: parentPageName
					strAttributeName = "parentPageName";
					objAttributeValue = parameter.getParentPageName();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: sequenceNumber
					strAttributeName = "sequenceNumber";
					objAttributeValue = parameter.getSequenceNo();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: defaultValue
					strAttributeName = "defaultValue";
					objAttributeValue = parameter.getDefaultValue();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: description
					strAttributeName = "description";
					objAttributeValue = parameter.getDescription();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: isReadOnly
					strAttributeName = "isReadOnly";
					objAttributeValue = parameter.getIsReadOnly();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: label
					strAttributeName = "label";
					objAttributeValue = parameter.getLabel();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: mask
					strAttributeName = "mask";
					objAttributeValue = parameter.getMask();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: maxLength
					strAttributeName = "maxLength";
					objAttributeValue = parameter.getMaxLength();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: maxValue
					strAttributeName = "maxValue";
					objAttributeValue = parameter.getMaxValue();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: minLength
					strAttributeName = "minLength";
					objAttributeValue = parameter.getMinLength();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: minValue
					strAttributeName = "minValue";
					objAttributeValue = parameter.getMinValue();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Append entry for attribute: value
					strAttributeName = "value";
					objAttributeValue = parameter.getValue();
					strAttribute = strAttributeName + "=\"" + objAttributeValue + "\"";
					strAttribute = strAttribute.replaceAll("\n", " ");
					strAttributes = strAttributes + " " + strAttribute;
					
				//	Reset all values set to the literal string "null" to an empty string
					strAttributes = strAttributes.replaceAll("\"null\"", "\"\"");
					
				//	Append to return string
					strReturn = strReturn 
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT 
							  + "<parameter" + strAttributes + "/>" 
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				}
				
			//	Decrease indent
				strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");
				
			//	Append closing "parameters" element tag
				strReturn = strReturn
						  + strIndent
						  + "</parameters>";
			}		
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_STATUS_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_STATUS_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Parse vector of ItpamProcessLogEntry objects and return String formatted as specified by the user.
 * 
 * @param vProcessLogEntries - Vector encapsulating ItpamProcessLogEntry objects
 * 
 * @return String of process log entries formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getProcessLogEntriesString(Vector<ItpamProcessLogEntry> vProcessLogEntries)
	throws Exception
	{		
	//	Initialize return string
		String strReturn = "";	
		
	//	Initialize indent, name and value strings
		String strIndent = "";
		String strName = "";
		String strValue = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Iterate vector of interactions
				for (Iterator<ItpamProcessLogEntry> iEntries = vProcessLogEntries.iterator(); iEntries.hasNext();){
					
				//	Get/set interaction object
					ItpamProcessLogEntry entry = iEntries.next();
					
				//	Append tab delimited log entry properties
					String strMessage = entry.getMessage();
					if (strMessage == null) strMessage = "";
					strMessage = strMessage.replaceAll("\n", "");
					strReturn = strReturn
							  + entry.getTimeStamp()
							  + ", "
							  + entry.getLevel().toUpperCase()
							  + ", "
							  + entry.getCategory()
							  + ": "
							  + strMessage
							  + ItpamWebServiceCLI.DELIMITER_PROPERTIES;
				}			
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "logEntries" element tag
				strReturn = strReturn
						  + strIndent
						  + "<logEntries"
						  + " " + ItpamWebServiceCLI.ARG_NAME_PROCESS_ID + "=" + ItpamWebServiceCLI.lProcessId
						  + " " + ItpamWebServiceCLI.ARG_NAME_LEVEL + "=" + ItpamWebServiceCLI.strLevel
						  + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;;
						  
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLI.DELIMITER_XML_INDENT;
				
			//	Iterate vector of interactions
				for (Iterator<ItpamProcessLogEntry> iEntries = vProcessLogEntries.iterator(); iEntries.hasNext();){
					
				//	Get/set interaction object
					ItpamProcessLogEntry entry = iEntries.next();
					
				//	Append opening "interaction" element string
					strReturn = strReturn
							  + strIndent
							  + "<logEntry"
							  + " id='" + ItpamWebServiceCLI.lInteractionId + "'"
							  + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
				//	Increase indent
					strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
					
				//	Append child property element strings
					strName = ItpamWebServiceCLIUtils.ARG_NAME_CATEGORY;
					strValue = entry.getCategory();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_LEVEL;
					strValue = entry.getLevel();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_MESSAGE;
					strValue = entry.getMessage();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_TIME_STAMP;
					strValue = entry.getTimeStamp();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;				
					
				//	Decrease indent
					strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");	
					
				//	Append closing "logEntry" element tag
					strReturn = strReturn
					          + strIndent
					          + "</logEntry>"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				}
				
			//	Append closing "Interactions" element tag
				strReturn = strReturn + "</logEntries>";
			}
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_STATUS_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_STATUS_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Parse properties object encapsulating process parameter name/value pairs and return String formatted as specified by the user.
 * 
 * @param propertiesParameters - Properties object encapsulating process parameter name/value pairs
 * 
 * @return String of process parameter names and values formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getProcessParametersString(Properties propertiesParameters, String strIndent)
	throws Exception
	{
	//	Initialize return string
		String strReturn = "";
		
	//	Update indent string
		if (strIndent == null) strIndent = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Iterate parameter properties
				for (Iterator<Object> iKeys = propertiesParameters.keySet().iterator(); iKeys.hasNext();){
					
				//	Get/set property key
					String strKey = (String) iKeys.next();
					
				//	Initialize parameter name string
					String strName = ItpamWebServiceCLIUtils.lProcessId
								   + "."
								   + ItpamWebServiceCLIUtils.ARG_PREFIX_PARAMETER
								   + strKey;
					
				//	Initialize parameter value string
					String strValue = propertiesParameters.getProperty(strKey);
					if (strValue == null) strValue = "";
					strValue = strValue.replaceAll("\n", " ");
					
				//	Append to property name/value pair to return string
					strReturn = strReturn
					          + strName
					          + "=" 
					          + strValue 
					          + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				}			
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "parameters" element tag
				strReturn = strReturn
						  + strIndent
						  + "<parameters "
						  + ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_ID
						  + "='" + ItpamWebServiceCLIUtils.lProcessId + "'>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
							
			//	Iterate parameter properties
				for (Iterator<Object> iKeys = propertiesParameters.keySet().iterator(); iKeys.hasNext();){
					
				//	Get/set property key
					String strKey = (String) iKeys.next();
					
				//	Initialize parameter name string
					String strName = strKey;
					
				//	Initialize parameter value string
					String strValue = propertiesParameters.getProperty(strKey);
					if (strValue == null) strValue = "";
					strValue = strValue.replaceAll("\n", " ");
					
				//	Append to property name/value pair to return string
					strReturn = strReturn
							  + strIndent
							  + "<parameter name='" + strName + "'>"
					          + strValue
					          + "</parameter>"
					          + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				}
				
			//	Decrease indent
				strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");
				
			//	Append closing "parameters" element tag
				strReturn = strReturn
						  + strIndent
						  + "</parameters>";
			}		
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_STATUS_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_STATUS_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Parse vector of ItpamInteraction objects and return String formatted as specified by the user.
 * 
 * @param vInteractions - Vector encapsulating ItpamInteractrion objects
 * 
 * @return String of interactions and related properties formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getProcessPendingInteractionsString(Vector<ItpamInteraction> vInteractions)
	throws Exception
	{		
	//	Initialize return string
		String strReturn = "";	
		
	//	Initialize indent, name and value strings
		String strIndent = "";
		String strName = "";
		String strValue = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Iterate vector of interactions
				for (Iterator<ItpamInteraction> iInteractions = vInteractions.iterator(); iInteractions.hasNext();){
					
				//Get/set interaction object
					ItpamInteraction interaction = iInteractions.next();
					
				//	Append property name/value pairs				
					strName = ItpamWebServiceCLIUtils.ARG_NAME_INTERACTION_ID;
					strValue = Long.toString(interaction.getInteractionId());
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;	
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_ID;
					strValue = interaction.getProcessId();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;	
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_DESCRIPTION;
					strValue = interaction.getDescription();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				}			
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "interactions" element tag
				strReturn = strReturn
						  + "<interactions>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;;
				
			//	Iterate vector of interactions
				for (Iterator<ItpamInteraction> iInteractions = vInteractions.iterator(); iInteractions.hasNext();){
					
				//	Get/set interaction object
					ItpamInteraction interaction = iInteractions.next();
					
				//	Append opening "interaction" element string
					strReturn = strReturn
							  + strIndent
							  + "<interaction"
							  + " id='" + ItpamWebServiceCLI.lInteractionId + "'"
							  + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
				//	Increase indent
					strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
					
				//	Append child property element strings
					strName = ItpamWebServiceCLIUtils.ARG_NAME_DESCRIPTION;
					strValue = interaction.getDescription();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_INTERACTION_ID;
					strValue = Long.toString(interaction.getInteractionId());
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_ID;
					strValue = interaction.getProcessId();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
					
				//	Decrease indent
					strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");	
					
				//	Append closing "interaction" element tag
					strReturn = strReturn
					          + strIndent
					          + "</interaction>"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				}
				
			//	Append closing "Interactions" element tag
				strReturn = strReturn + "</interactions>";
			}
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_STATUS_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_STATUS_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Parse vector of ItpamInteraction objects and return String formatted as specified by the user.
 * 
 * @param status - ItpamProcessStatus object encapsulating status properties
 * 
 * @return String of process status properties formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getProcessStatusString(ItpamProcessStatus status)
	throws Exception
	{		
	//	Initialize return string
		String strReturn = "";
		
	//	Initialize indent, name and value strings
		String strIndent = "";
		String strName = "";
		String strValue = "";
		
	//	Parse content and generate return string in format specified
		try {
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Append property name/value pairs			
				strName = ItpamWebServiceCLI.ARG_NAME_CONNNECTION_ID;
				strValue = status.getConnectionId();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_MESSAGE;
				strValue = status.getMessage();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;	
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_MESSAGE_STATUS;
				strValue = status.getMessageStatus();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_DISPLAY_NAME;
				strValue = status.getProcessDisplayName();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_ID;
				strValue = status.getProcessId();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_INSTANCE_NAME;
				strValue = status.getProcessInstanceName();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_LIBRARY_PATH_NAME;
				strValue = status.getProcessLibraryPathName();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_STATE;
				strValue = status.getProcessState();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_SENDER_ADDRESS;
				strValue = status.getSenderAddress();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_SUBMISSION_TIME;
				strValue = status.getSubmissionTime();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strReturn = strReturn + ItpamWebServiceCLIUtils.getProcessParametersString(status.getParameters(), "")
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "status" element tag
				strReturn = strReturn
						  + strIndent
						  + "<status "
						  + ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_ID
						  + "='" + ItpamWebServiceCLIUtils.lProcessId + "'>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
				
			//	Append status property elements
				strName = ItpamWebServiceCLIUtils.ARG_NAME_CONNNECTION_ID;
				strValue = status.getConnectionId();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_MESSAGE;
				strValue = status.getMessage();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_MESSAGE_STATUS;
				strValue = status.getMessageStatus();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_DISPLAY_NAME;
				strValue = status.getProcessDisplayName();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_ID;
				strValue = status.getProcessId();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_INSTANCE_NAME;
				strValue = status.getProcessInstanceName();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_LIBRARY_PATH_NAME;
				strValue = status.getProcessLibraryPathName();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_STATE;
				strValue = status.getProcessState();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_SENDER_ADDRESS;
				strValue = status.getSenderAddress();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_SUBMISSION_TIME;
				strValue = status.getSubmissionTime();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Append parameters
				strReturn = strReturn
						  + ItpamWebServiceCLIUtils.getProcessParametersString(status.getParameters(), strIndent)
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Decrease indent
				strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");
				
			//	Append closing "interactions" element tag
				strReturn = strReturn
						  + "</status>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;;
			}
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_STATUS_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_STATUS_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}


/**
 * Parse vector of ItpamAttachment objects and return String formatted as specified by the user.
 * 
 * @param vAttachments - Vector encapsulating ItpamAttachment objects
 * 
 * @return String of attachment info formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getStartRequestFormsString(Vector<ItpamStartRequestForm> vAttachments)
	throws Exception
	{		
	//	Initialize return string
		String strReturn = "";	
		
	//	Initialize indent, name and value strings
		String strIndent = "";
		String strName = "";
		String strValue = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Iterate vector of interactions
				for (Iterator<ItpamStartRequestForm> iForms = vAttachments.iterator(); iForms.hasNext();){
					
				//	Get/set interaction object
					ItpamStartRequestForm form = iForms.next();
					
				//	Get/set parameter name prefix
					String strPrefix = form.getPath() + form.getName() + ".";
					strPrefix = strPrefix.replaceAll(" ", "_");
					
				//	Append tab delimited form properties
					strName = ItpamWebServiceCLIUtils.ARG_NAME_DESCRIPTION;
					strValue = form.getDescripition();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;	
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_NAME;
					strValue = form.getName();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_PATH;
					strValue = form.getPath();
					if (strValue == null) strValue = "";
					strValue = strValue.replace("\n", "");
					strReturn = strReturn
							  + strPrefix
							  + strName
							  + "="
							  + strValue
							  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
					
					strReturn = strReturn + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				}			
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "forms" element tag
				strReturn = strReturn
						  + strIndent
						  + "<forms"
						  + ">"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;;
						  
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLI.DELIMITER_XML_INDENT;
				
			//	Iterate vector of interactions
				for (Iterator<ItpamStartRequestForm> iAttachments = vAttachments.iterator(); iAttachments.hasNext();){
					
				//	Get/set interaction object
					ItpamStartRequestForm form = iAttachments.next();
					
				//	Append opening "interaction" element string
					strReturn = strReturn
							  + strIndent
							  + "<form"
							  + " id='" + form.getPath() + form.getName() + "'"
							  + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
				//	Increase indent
					strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
					
				//	Append child property element strings
					strName = ItpamWebServiceCLIUtils.ARG_NAME_DESCRIPTION;
					strValue = form.getDescripition();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
	
					strName = ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_NAME;
					strValue = form.getName();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
					
					strName = ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_PATH;
					strValue = form.getPath();
					if (strValue == null) strValue = "";
					strReturn = strReturn + strIndent
							  + "<" + strName + ">"
							  + strValue.replace("\n", "")
							  + "</" + strName + ">"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;				
					
				//	Decrease indent
					strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");	
					
				//	Append closing "form" element tag
					strReturn = strReturn
					          + strIndent
					          + "</form>"
							  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				}
				
			//	Append closing "Interactions" element tag
				strReturn = strReturn + "</forms>";
			}
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){
	
		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_STATUS_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_STATUS_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Parse vector of ItpamInteraction objects and return String formatted as specified by the user.
 * 
 * @param status - ItpamProcessStatus object encapsulating status properties
 * 
 * @return String of process status properties formatted as specified by the user
 * 
 * @throws Exception
 */
	protected static String getStartRequestStatusString(ItpamStartRequestStatus status)
	throws Exception
	{		
	//	Initialize return string
		String strReturn = "";
		
	//	Initialize indent, name and value strings
		String strIndent = "";
		String strName = "";
		String strValue = "";
		
	//	Parse content and generate return string in format specified
		try {
			
			if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_PROPERTIES)){
				
			//	Append property name/value pairs			
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EARLIEST_START_DATE;
				strValue = status.getEarliestStartTime();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_END_DATE;
				strValue = status.getEndTime();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;	
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_INTERACTION_ID;
				strValue = status.getInteractionId();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_ID;
				strValue = status.getProcessId();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_NAME;
				strValue = status.getReferencedProcess();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_START_DATE;
				strValue = status.getStartTime();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_STATE;
				strValue = status.getState();
				if (strValue == null) strValue = "";
				strValue = strValue.replace("\n", "");
				strReturn = strReturn
						  + strName
						  + "="
						  + strValue
						  + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
				
				strReturn = strReturn + ItpamWebServiceCLIUtils.DELIMITER_PROPERTIES;
			}
			else if (ItpamWebServiceCLIUtils.strOutputFormat.equalsIgnoreCase(ItpamWebServiceCLIUtils.OUTPUT_FORMAT_XML)){
				
			//	Append opening "status" element tag
				strReturn = strReturn
						  + strIndent
						  + "<status "
						  + ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_ID
						  + "='" + ItpamWebServiceCLIUtils.lStartRequestId + "'>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Increase indent
				strIndent = strIndent + ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT;
				
			//	Append status property elements
				strName = ItpamWebServiceCLIUtils.ARG_NAME_EARLIEST_START_DATE;
				strValue = status.getEarliestStartTime();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_END_DATE;
				strValue = status.getEndTime();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_INTERACTION_ID;
				strValue = status.getInteractionId();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_ID;
				strValue = status.getProcessId();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_PROCESS_NAME;
				strValue = status.getReferencedProcess();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_START_DATE;
				strValue = status.getStartTime();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
				strName = ItpamWebServiceCLIUtils.ARG_NAME_START_REQUEST_STATE;
				strValue = status.getState();
				if (strValue == null) strValue = "";
				strReturn = strReturn + strIndent + "<statusProperty name='" + strName + "'>"
						  + strValue.replace("\n", "")
						  + "</statusProperty>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;
				
			//	Decrease indent
				strIndent = strIndent.replaceFirst(ItpamWebServiceCLIUtils.DELIMITER_XML_INDENT, "");
				
			//	Append closing "status" element tag
				strReturn = strReturn
						  + "</status>"
						  + ItpamWebServiceCLIUtils.DELIMITER_XML_ELEMENT;;
			}
			
		//	Return string
			return strReturn;
		}
		catch (Exception e){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_UNABLE_TO_PARSE_STATUS_OBJECT");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_UNABLE_TO_PARSE_STATUS_OBJECT") + ": " + e.getMessage();
			
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);				
			
		//	Throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
	

/**
 * Initialize vector of actions supported.
 * 																													*/
	private static void initActionsSupported(){
    	
    	int intMethods = ItpamWebServiceCLI.class.getDeclaredMethods().length;
		
		for (int intMethod = 0; intMethod < intMethods; intMethod++){
			
			String strAction = ItpamWebServiceCLI.class.getDeclaredMethods()[intMethod].getName();
			
			if (!ItpamWebServiceCLI.isNullEmpty(strAction)){
				
				if (!strAction.equalsIgnoreCase("main")){
					
					ItpamWebServiceCLI.vActionsSupported.add(strAction);
				}
			}
		}
	}
	
    
/**
 * Initialize logger.
 * 																													*/
    private static void initLogger()
	{
    //	Get root logger
		ItpamWebServiceCLI.logger = Logger.getRootLogger();
		
	//	Remove all appenders
		ItpamWebServiceCLI.logger.removeAllAppenders();
		
	//	Initialize new rolling file log appender
		RollingFileAppender appenderRollingFile = new RollingFileAppender();
		
	//	Set appender properties
		appenderRollingFile.setFile(ItpamWebServiceCLI.LOG_PATH_FILE);
		appenderRollingFile.setLayout(new PatternLayout(ItpamWebServiceCLI.LOG_LAYOUT));
		appenderRollingFile.setMaxBackupIndex(ItpamWebServiceCLI.LOG_MAX_BACKUPS);
		appenderRollingFile.setMaxFileSize(ItpamWebServiceCLI.LOG_MAX_FILE_SIZE);
		
	//	Activate appender options
		appenderRollingFile.activateOptions();
		
	//	Add rolling file appender to logger
		logger.addAppender(appenderRollingFile);
		
	//	Set log level
		String strLogLevel = System.getProperty(ItpamWebServiceCLI.ARG_NAME_LOG_LEVEL,
				                                ItpamWebServiceCLI.DEFAULT_LOG_LEVEL);
		
		if (strLogLevel.equalsIgnoreCase("fatal")){

			ItpamWebServiceCLI.logger.setLevel(Level.FATAL);	
		}
		else if (strLogLevel.equalsIgnoreCase("error")){

			ItpamWebServiceCLI.logger.setLevel(Level.ERROR);	
		}
		else if (strLogLevel.equalsIgnoreCase("warn")){

			ItpamWebServiceCLI.logger.setLevel(Level.WARN);	
		}
		else if (strLogLevel.equalsIgnoreCase("info")){

			ItpamWebServiceCLI.logger.setLevel(Level.INFO);	
		}
		else if (strLogLevel.equalsIgnoreCase("debug")){

			ItpamWebServiceCLI.logger.setLevel(Level.DEBUG);	
		}
		else if (strLogLevel.equalsIgnoreCase("all")){

		//	Initialize console appender
			PatternLayout layout = new PatternLayout(ItpamWebServiceCLI.LOG_LAYOUT);
			ConsoleAppender appenderConsole = new ConsoleAppender(layout, "System.out");
			appenderConsole.activateOptions();
			
		//	Add console appender to logger
			ItpamWebServiceCLI.logger.addAppender(appenderConsole);
			
		//	Reset log level
			ItpamWebServiceCLI.logger.setLevel(Level.ALL);	
		}
	}
    
    
    /**
 * Initialize all utility objects.
 * 
 * @param args - array of Strings set to command line arguments specified by user
 * 
 * @throws Exception
 * 																													*/
	protected static void initUtils(String[] args)
	throws Exception
	{
	//	Set log level
		ItpamWebServiceCLIUtils.setLogLevel(args);
		
	//	Initialize logger
		ItpamWebServiceCLIUtils.initLogger();
		
	//	Record progress
		ItpamWebServiceCLI.logger.info("Web service CLI request started");
		
	//	Set interactive mode flag
		ItpamWebServiceCLIUtils.setInteractiveModeFlag(args);
		
	//	Record progress
		ItpamWebServiceCLI.logger.debug("Initializing message resource bundle");
		
	//	Initialize message resource bundle
		ItpamWebServiceCLI.messages = new Messages(ItpamWebServiceCLI.class.getPackage().getName().concat(".resource.Message"));
				
	//	Record progress
		ItpamWebServiceCLI.logger.debug("Initializing supported actions");
		
	//	Initialize supported actions vector
		ItpamWebServiceCLIUtils.initActionsSupported();
		
	//	Record progress
		ItpamWebServiceCLI.logger.debug("Initializing cached web service configuration properties");
		
	//	Read cached service configuration properties from file
		File fileServerConfig = new File(ItpamWebServiceCLI.SERVER_CONFIG_FILE);
		if (fileServerConfig.exists()){

			try {
				
				FileInputStream fisServerConfig;
				fisServerConfig = new FileInputStream(fileServerConfig);
				Properties propertiesServerConfig = new Properties();
				propertiesServerConfig.load(fisServerConfig);
				
				ItpamWebServiceCLI.strItpamUri = propertiesServerConfig.getProperty(ItpamWebServiceCLI.ARG_NAME_ITPAM_URI);
				ItpamWebServiceCLI.strItpamVersion = propertiesServerConfig.getProperty(ItpamWebServiceCLI.ARG_NAME_ITPAM_VERSION);
				ItpamWebServiceCLI.strTrustStore = propertiesServerConfig.getProperty(ItpamWebServiceCLI.ARG_NAME_TRUST_STORE);
				if ( ItpamWebServiceCLI.strTrustStore != null)
					setTrustStore(ItpamWebServiceCLI.strTrustStore);
			}
			catch (FileNotFoundException e) {

			//	Set exception messages
				String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_ACTION_FILE");
				if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
					
					strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
				}
				
				String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_ACTION_FILE") + ": " + e.getMessage();
				
			//	Record exception
				ItpamWebServiceCLIUtils.logger.error(strMessage);				
				
			//	throw exception
				throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
			} 
			catch (IOException e) {

			//	Set exception messages
				String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_ACTION_FILE");
				if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
					
					strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
				}
				
				String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_ACTION_FILE") + ": " + e.getMessage();
				
			//	Record exception
				ItpamWebServiceCLIUtils.logger.error(strMessage);					
				
			//	throw exception
				throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
			}
		}
		
	//	Append entries from action file to arguments
		args = ItpamWebServiceCLIUtils.appendArgumentsFromActionFile(args);
		
	//	Parse command line arguments
		ItpamWebServiceCLIUtils.parseArguments(args);
	
	//	Record progress
		ItpamWebServiceCLI.logger.debug("Initializing web service client");
		
	//	Initialize ITPAM web service client if required to complete specified action
		if ((!bolHelpRequested)
		&& (!ItpamWebServiceCLI.LIST_ACTIONS_CLIENT_NOT_REQUIRED.contains(ItpamWebServiceCLIUtils.strAction))){
			
			ItpamWebServiceCLIUtils.initWebServiceClient();			
		}
	}
	
	
/**
 * Initialize ITPAM web service client used to communicate with specified ITPM server.
 * 
 * @throws ItpamWebServiceClientFactoryException
 * 																													*/
	private static void initWebServiceClient()
	throws ItpamWebServiceClientFactoryException
	{
		
		ItpamWebServiceClientFactory itpamWebServiceClientFactory = new ItpamWebServiceClientFactory();
		ItpamWebServiceCLI.client = itpamWebServiceClientFactory.getInstanceOf(ItpamWebServiceCLIUtils.strItpamVersion,
																			   ItpamWebServiceCLIUtils.strItpamUri,
																			   ItpamWebServiceCLIUtils.strItpamUser,
																			   ItpamWebServiceCLIUtils.strItpamPassword);
		if(ItpamWebServiceCLIUtils.strItpamVersion == null || ItpamWebServiceCLIUtils.strItpamVersion.length() == 0) {
			ItpamWebServiceCLIUtils.strItpamVersion = client.get_strTargetEndpointVersion();
		}
	}
	
	
/**
 * Inspect string specified to determine if null or empty.
 * 
 * @param strString - String to inspect
 * 
 * @return boolean true if null or empty otherwise false
 * 																													*/
	protected static boolean isNullEmpty(String strString){
		
	//	Initialize boolean return value
		boolean bolNullEmpty = false;
		
	//	Reset boolean value if string is null or empty
		if (strString == null){
			
			bolNullEmpty = true;
		}
		else if (strString.equals("")){
			
			bolNullEmpty = true;
		}
		
		
	//	Return boolean value
		return bolNullEmpty;
	}
	
	
/**
 * Inspect the string specified to determine if represents a numeric value.
 * 
 * @param strNumeric - String to be inspected
 * 
 * @return boolean true if string could successfully be converted to numeric
 * otherwise false
 * 																			*/
	protected boolean isNumeric(String strNumeric){
	
		try {
			Integer.parseInt(strNumeric);
			return true;
		}
		catch ( Exception e){			
			return false;
		}
	}

	
/**
 * Parse and validate command line arguments specified by user.
 * 
 * @param args - array of Strings set to command line arguments specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void parseArguments (String[] args)
	throws Exception
	{
	//	Record progress
		ItpamWebServiceCLI.logger.debug("Parsing parameters");
		
		for (int intArg = 0; intArg < args.length; intArg++){
			
		//	Split argument name/value
			String[] aryNameValue = args[intArg].split("=", 2);
			
		//	Process name/value pair if correct number of elements
			if (aryNameValue.length == 2){
				
			//	Set name and value strings
				String strName = aryNameValue[0];
				String strValue = aryNameValue[1];
				
			//	Inspect name and update appropriate property or collection
				if (ItpamWebServiceCLI.isNullEmpty(strName)){

				//	Set exception messages
					String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_MISSING_PARAMETER_NAME");
					
					String strMessage = ItpamWebServiceCLI.messages.get("E_MISSING_PARAMETER_NAME") + ": " + args[intArg] + ": " + args[intArg];
						
				//	Record exception
					ItpamWebServiceCLIUtils.logger.error(strMessage);
					
				//	throw exception
					throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ACTION)){
					
					ItpamWebServiceCLIUtils.setAction(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ACTION_FILE)){
					
					ItpamWebServiceCLIUtils.setActionFile(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ALLOW_PARTIAL_MATCH)){
					
					ItpamWebServiceCLIUtils.setAllowPartialMatch(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ROOT_UUID)){
					
					ItpamWebServiceCLIUtils.setRootUuid(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_INSTANCE_ID)) {
					
					ItpamWebServiceCLIUtils.setInstanceId(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ATTACHMENT_ID)){
					
					ItpamWebServiceCLIUtils.addAttachmentId(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ATTACHMENT_PARAMETER_NAME)){
					
					ItpamWebServiceCLIUtils.setAttachmentParameterName(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_CATEGORY)){
					
					ItpamWebServiceCLIUtils.addCategory(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_CONTENT_ID)){
					
					ItpamWebServiceCLIUtils.setContentId(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_CONTENT_TYPE)){
					
					ItpamWebServiceCLIUtils.setContentType(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ENABLE_PATTERN_MATCH)){
					
					ItpamWebServiceCLIUtils.setEnablePatternMatch(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_END_DATE)){
					
					ItpamWebServiceCLIUtils.setEndDate(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_EVENT_DESTINATION)){
					
					ItpamWebServiceCLIUtils.setEventDestination(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_EVENT_EXPRESSION)){
					
					ItpamWebServiceCLIUtils.setEventExpression(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_EVENT_EXPIRATION_DURATION)){
					
					ItpamWebServiceCLIUtils.setEventExpirationDuration(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_EVENT_NAME)){
					
					ItpamWebServiceCLIUtils.setEventName(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_EVENT_PAYLOAD)){
					
					ItpamWebServiceCLIUtils.setEventPayload(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_EVENT_SINGLE_SUBSCRIBER)){
					
					ItpamWebServiceCLIUtils.setEventSingleSubscriber(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_EVENT_SOURCE)){
					
					ItpamWebServiceCLIUtils.setEventSource(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_EVENT_TYPE)){
					
					ItpamWebServiceCLIUtils.setEventType(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_IMPORT_MAKE_AVAILABLE)){
					
					ItpamWebServiceCLIUtils.setImportMakeAvailable(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_IMPORT_MAKE_CURRENT)){
					
					ItpamWebServiceCLIUtils.setImportMakeCurrent(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_IMPORT_SOURCE_PATH_FILE_NAME)){
					
					ItpamWebServiceCLIUtils.setImportSourcePathFileName(strValue);
				}
				else if ( strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_IMPORT_OVERWRITE_ACTION)){
					
					ItpamWebServiceCLIUtils.setImportOverwriteAction(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_IMPORT_TARGET_LIBRARY_PATH_NAME)){
					
					ItpamWebServiceCLIUtils.setImportTargetLibraryPathName(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_INTERACTION_ID)){
					
					ItpamWebServiceCLIUtils.setInteractionId(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ITPAM_PASSWORD)){
					
					ItpamWebServiceCLIUtils.setPassword(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ITPAM_URI)){
					
					ItpamWebServiceCLIUtils.setUri(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ITPAM_USER)){
					
					ItpamWebServiceCLIUtils.setUser(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ITPAM_VERSION)){
					
					ItpamWebServiceCLIUtils.setVersion(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_KEYWORD)){
					
					ItpamWebServiceCLIUtils.addKeyword(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_LEVEL)){
					
					ItpamWebServiceCLIUtils.setLevel(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_LEVELS_TO_SEARCH)){
					
					ItpamWebServiceCLIUtils.setLevelsToSearch(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_LOG_LEVEL)){
					
					ItpamWebServiceCLI.logger.info("Log level set to " + strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_OBJECT_TYPE)){
					
					ItpamWebServiceCLIUtils.addObjectType(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_OUTPUT_FORMAT)){
					
					ItpamWebServiceCLIUtils.setOutputFormat(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_OVERWRITE_TARGET_FILE)){
					
					ItpamWebServiceCLIUtils.setOverwriteTargetFile(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_PROCESS_ID)){
					
					ItpamWebServiceCLIUtils.setProcessId(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_PROCESS_NAME)){
					
					ItpamWebServiceCLIUtils.setProcessName(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_RECUSIVE)){
					
					ItpamWebServiceCLIUtils.setRecursive(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_SEARCH_PATH)){
					
					ItpamWebServiceCLIUtils.setSearchPath(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_ABSOLUTE_PATH)){
					
					ItpamWebServiceCLIUtils.setAbsolutePath(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_SOURCE_LIBRARY_PATH)){
					
					ItpamWebServiceCLIUtils.setSourceLibraryPath(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_START_DATE)){
					
					ItpamWebServiceCLIUtils.setStartDate(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_START_REQUEST_ID)){
					
					ItpamWebServiceCLIUtils.setStartRequestId(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_START_REQUEST_NAME)){
					
					ItpamWebServiceCLIUtils.setStartRequestName(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_START_REQUEST_PATH)){
					
					ItpamWebServiceCLIUtils.setStartRequestPath(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_START_TIME)){
					
					ItpamWebServiceCLIUtils.setStartTime(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_TAG_ID)){
					
					ItpamWebServiceCLIUtils.setTagId(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_PRIORITY)){
					
					ItpamWebServiceCLIUtils.setPriority(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_IS_AUTO_ARCHIVE)){
					
					ItpamWebServiceCLIUtils.setIsAutoArchive(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_TARGET_FILE_NAME)){
					
					ItpamWebServiceCLIUtils.setTargetFileName(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_TARGET_PATH)){
					
					ItpamWebServiceCLIUtils.setTargetPath(strValue);
				}
				else if (strName.equalsIgnoreCase(ItpamWebServiceCLIConstants.ARG_NAME_TARGET_PATH_IS_LOCAL)){
					
					ItpamWebServiceCLIUtils.setTargetPathIsLocal(strValue);
				}
				else if (strName.toLowerCase().startsWith(ItpamWebServiceCLIConstants.ARG_PREFIX_ATTACHMENT)){
					
					ItpamWebServiceCLIUtils.addAttachment(strName, strValue);
				}
				else if (strName.toLowerCase().startsWith(ItpamWebServiceCLIConstants.ARG_PREFIX_PARAMETER)){
					
					ItpamWebServiceCLIUtils.addParameter(strName, strValue);
				}
				else if ( strName.toLowerCase().equals(ItpamWebServiceCLIConstants.ARG_NAME_TRUST_STORE)){
					ItpamWebServiceCLIUtils.setTrustStore(strValue);
				}
				else if ( strName.toLowerCase().equals(ItpamWebServiceCLIConstants.ARG_NAME_TRUST_STORE_PASSWORD)){
					ItpamWebServiceCLIUtils.setTrustStorePassword(strValue);
				}
				else {

				//	Set exception messages
					String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_PARAMETER_NAME") + ": " +args[intArg];
					
					String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_PARAMETER_NAME") + ": " + args[intArg];
						
				//	Record exception
					ItpamWebServiceCLIUtils.logger.error(strMessage);
					
				//	throw exception
					throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
				}
			}
			else if (  (aryNameValue[0].toLowerCase().startsWith("?"))
					|| (aryNameValue[0].toLowerCase().startsWith("-?"))
					|| (aryNameValue[0].toLowerCase().startsWith("/?"))
					|| (aryNameValue[0].toLowerCase().startsWith("h"))
					|| (aryNameValue[0].toLowerCase().startsWith("-h"))
					|| (aryNameValue[0].toLowerCase().startsWith("/h")))
			{
				
			//	Set help requested flag to true
				ItpamWebServiceCLI.bolHelpRequested = true;
				
			//	Set interactive mode flag to true
				ItpamWebServiceCLI.bolInteractiveMode = true;
				
			//	Record quiet mode option specified
				logger.info("User requested help");
			}
			else if (  (aryNameValue[0].equalsIgnoreCase("i"))
					|| (aryNameValue[0].equalsIgnoreCase("-i"))
					|| (aryNameValue[0].equalsIgnoreCase("/i")))
			{
				
			//	Set interactive mode flag to true
				ItpamWebServiceCLI.bolInteractiveMode = true;
				
			//	Record quiet mode option specified
				logger.info("User specified interactive mode");
			}
			else {

			//	Set exception messages
				String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_PARAMETER_SYNTAX") + ": " +args[intArg];
						
				String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_PARAMETER_SYNTAX") + ": " + args[intArg];
							
			//	Record exception
				ItpamWebServiceCLIUtils.logger.error(strMessage);
						
			//	throw exception
				throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
			}
		}
		
		if ((!ItpamWebServiceCLI.bolHelpRequested)
		&& (ItpamWebServiceCLI.isNullEmpty(ItpamWebServiceCLIUtils.strAction))) {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_MISSING_PARAMETER_VALUE") + ": " + ItpamWebServiceCLI.ARG_NAME_ACTION;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_MISSING_PARAMETER_VALUE") + ": " + ItpamWebServiceCLI.ARG_NAME_ACTION;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		
		// has the trustStore & trustStorePassword been specified when
		// a https uri is specified?
		if ( 	ItpamWebServiceCLIUtils.getUri().startsWith("https://")){
			if ( ItpamWebServiceCLIUtils.strAction.toLowerCase().equals("setserviceconfig")){
				// only requires the truststore parameter
				if ( ItpamWebServiceCLIUtils.getTrustStore()==null){
					
					//Set exception messages
					String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_MISSING_TRUST_STORE_INFO_SET_SERVICE_CONFIG") ;
							
					String strMessage = ItpamWebServiceCLI.messages.get("E_MISSING_TRUST_STORE_INFO_SET_SERVICE_CONFIG");
							
					//	Record exception
					ItpamWebServiceCLIUtils.logger.error(strMessage);
						
					//	throw exception
					throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
				
				}
					
			}
			// don't test for truststore when performing a getServiceConfig action
			else if ( !ItpamWebServiceCLIUtils.strAction.toLowerCase().equals("getserviceconfig") ){
				if (  (ItpamWebServiceCLIUtils.getTrustStore()==null) ||
					  (ItpamWebServiceCLIUtils.getTrustStorePassword()==null)){
				
					//Set exception messages
					String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_MISSING_TRUST_STORE_INFO") ;
							
					String strMessage = ItpamWebServiceCLI.messages.get("E_MISSING_TRUST_STORE_INFO");
						
					//	Record exception
					ItpamWebServiceCLIUtils.logger.error(strMessage);
						
					//	throw exception
					throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
				}
			}
		}
	}
	
	private static void setImportOverwriteAction(String strValue) {
		ItpamWebServiceCLIUtils.strImportOvewriteAction = strValue;
	
	}

	/**
	 * reesi02-added to support https uris via the command line tools
	 * @param strValue
	 */
	private static void setTrustStorePassword(String strValue) {
		strTrustStorePassword = strValue; 
		System.setProperty("javax.net.ssl.trustStorePassword", strValue);
	}
	
	private static String getTrustStorePassword(){
		return strTrustStorePassword;
	}

	/**
	 * reesi02 - added to support https uris via command line web services tools 
	 * @param strValue
	 */
	private static void setTrustStore(String strValue) {
		strTrustStore = strValue;
		System.setProperty("javax.net.ssl.trustStore", strValue);
	}
	
	private static String getTrustStore(){
		return strTrustStore;
	}

/**
 * Validate and set property value.
 * 
 * @param strValue - String set to make available flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setAbsolutePath(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_ABSOLUTE_PATH + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_ABSOLUTE_PATH + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolAbsolutePath = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolAbsolutePath = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_ABSOLUTE_PATH + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_ABSOLUTE_PATH + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	

/**
 * Validate and set action property.
 * 
 * @param strValue - String set to action specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setAction(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_MISSING_PARAMETER_VALUE") + ": " + ItpamWebServiceCLI.ARG_NAME_ACTION;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_MISSING_PARAMETER_VALUE") + ": " + ItpamWebServiceCLI.ARG_NAME_ACTION;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else {
		
			boolean bolActionNotSupported = true;
			
			for (Iterator<String> iAction = ItpamWebServiceCLI.vActionsSupported.iterator(); iAction.hasNext();){
				
				String strActionSupported =iAction.next();
				
				if (strValue.equalsIgnoreCase(strActionSupported)){
					
					ItpamWebServiceCLI.strAction = strActionSupported;
					bolActionNotSupported = false;
					break;
				}
			}
			
			if (bolActionNotSupported){

			//	Set exception messages
				String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_ACTION_PARAMETER_VALUE") + ": "
				                           + ItpamWebServiceCLI.ARG_NAME_ACTION + "=" + strValue;
						
				String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_ACTION_PARAMETER_VALUE") + ": "
                + ItpamWebServiceCLI.ARG_NAME_ACTION + "=" + strValue;
							
			//	Record exception
				ItpamWebServiceCLIUtils.logger.error(strMessage);
						
			//	throw exception
				throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
			}
		}
		
	//	Reset usage resource bundle name
		ItpamWebServiceCLI.strUsageResourceBundle = ItpamWebServiceCLI.strUsageResourceBundle + "_" + ItpamWebServiceCLI.strAction;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to action file path and name specified by the user
 * 																													*/
	private static void setActionFile(String strValue){
		
		ItpamWebServiceCLIUtils.strActionFile = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to make available flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setAllowPartialMatch(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_ALLOW_PARTIAL_MATCH + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_ALLOW_PARTIAL_MATCH + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolAllowPartialMatch = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolAllowPartialMatch = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_ALLOW_PARTIAL_MATCH + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_ALLOW_PARTIAL_MATCH + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	

/**
 * Validate and set property value.
 * 
 * @param strValue - String set to attachment parameter name specified by the user
 * 																													*/
	private static void setAttachmentParameterName(String strValue){
		
		ItpamWebServiceCLIUtils.strAttachmentParameterName = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to content ID specified by the user
 * 																													*/
	private static void setContentId(String strValue){
		
		ItpamWebServiceCLIUtils.strContentId = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to content type specified by the user
 * 																													*/
	private static void setContentType(String strValue){
		
		ItpamWebServiceCLIUtils.strContentType = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to make available flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setEnablePatternMatch(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_ENABLE_PATTERN_MATCH + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_ENABLE_PATTERN_MATCH + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolEnablePatternMatch = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolEnablePatternMatch = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_ENABLE_PATTERN_MATCH + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_ENABLE_PATTERN_MATCH + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to content type specified by the user
 * 																													*/
	private static void setEndDate(String strValue){
		
		ItpamWebServiceCLIUtils.strEndDate = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to event destination specified by the user
 * 																													*/
	private static void setEventDestination(String strValue){
		
		ItpamWebServiceCLIUtils.strEventDestination = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to event expression specified by the user
 * 																													*/
	private static void setEventExpression(String strValue){
		
		ItpamWebServiceCLIUtils.strEventExpression = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to event expiration duration in seconds specified by the user
 * 
 * @throws Exception 
 */
	private static void setEventExpirationDuration(String strValue) throws Exception {
		
		try {
			
			ItpamWebServiceCLIUtils.lEventExpirationDuration = Long.parseLong(strValue);
		}
		catch (Exception e){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_EVENT_EXPIRATION_DURATION + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_EVENT_EXPIRATION_DURATION + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to event name specified by the user
 * 																													*/
	private static void setEventName(String strValue){
		
		ItpamWebServiceCLIUtils.strEventName = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to event pay load specified by the user
 * 																													*/
	private static void setEventPayload(String strValue){
		
		ItpamWebServiceCLIUtils.strEventPayload = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to single subscriber flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setEventSingleSubscriber(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_EVENT_SINGLE_SUBSCRIBER + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_EVENT_SINGLE_SUBSCRIBER + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolEventSingleSubscriber = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolEventSingleSubscriber = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_EVENT_SINGLE_SUBSCRIBER + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_EVENT_SINGLE_SUBSCRIBER + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to event source specified by the user
 * 																													*/
	private static void setEventSource(String strValue){
		
		ItpamWebServiceCLIUtils.strEventSource = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to event type specified by the user
 * 																													*/
	private static void setEventType(String strValue){
		
		ItpamWebServiceCLIUtils.strEventType = strValue;
	}
	
	
/**
 * Set/reset exit code based on exception type.
 * 
 * @param e - Generic InvocationTargetException object thrown by reflection method invocation
 * encapsulation the actual exception thrown.
 * 																													*/
	protected static void setExitCode(InvocationTargetException e){
		
	//	Get/set exception type
		String strExceptionType = e.getCause().getClass().getSimpleName();

	//	Set exception code based on exception type
		if (ItpamWebServiceCLI.isNullEmpty(strExceptionType)){
			
			ItpamWebServiceCLIUtils.intExitCode = 9999;
		}
		else if (strExceptionType.equalsIgnoreCase("ItpamCLIUnexpectedReturnException")){
			
			ItpamWebServiceCLIUtils.intExitCode = 9002;
		}
		else if (strExceptionType.equalsIgnoreCase("ItpamEntityNotFoundException")){
			
			ItpamWebServiceCLIUtils.intExitCode = 9003;
		}
		else if (strExceptionType.equalsIgnoreCase("ItpamMethodNotSupportedException")){
			
			ItpamWebServiceCLIUtils.intExitCode = 9004;
		}
		else if (strExceptionType.equalsIgnoreCase("ItpamParameterMissingOrInvalidException")){
			
			ItpamWebServiceCLIUtils.intExitCode = 9005;
		}
		else if (strExceptionType.equalsIgnoreCase("ItpamStringToDateException")){
			
			ItpamWebServiceCLIUtils.intExitCode = 9006;
		}
		else if (strExceptionType.equalsIgnoreCase("ItpamUnknownHostException")){
			
			ItpamWebServiceCLIUtils.intExitCode = 9007;
		}
		else if (strExceptionType.equalsIgnoreCase("ItpamUserPermissionsException")){
			
			ItpamWebServiceCLIUtils.intExitCode = 9008;
		}
		else if (strExceptionType.equalsIgnoreCase("ItpamWebServiceClientException")){
			
			ItpamWebServiceCLIUtils.intExitCode = 9009;
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to make available flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setImportMakeAvailable(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_IMPORT_MAKE_AVAILABLE + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_IMPORT_MAKE_AVAILABLE + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolImportMakeAvailable = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolImportMakeAvailable = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_IMPORT_MAKE_AVAILABLE + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_IMPORT_MAKE_AVAILABLE + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to make current flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setImportMakeCurrent(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_IMPORT_MAKE_CURRENT + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_IMPORT_MAKE_CURRENT + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolImportMakeCurrent = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolImportMakeCurrent = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_IMPORT_MAKE_CURRENT + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_IMPORT_MAKE_CURRENT + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw ItpamWebServiceCLI.messages.
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to import source path and file name specified by the user
 * 																													*/
	private static void setImportSourcePathFileName(String strValue){
		
		ItpamWebServiceCLIUtils.strImportSourcePathFileName = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to import target library path and name specified by the user
 * 																													*/
	private static void setImportTargetLibraryPathName(String strValue){
		
		ItpamWebServiceCLIUtils.strImportTargetLibraryPathName = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to interaction ID specified by the user
 * @throws Exception 
 * 																													*/
	private static void setInteractionId(String strValue) throws Exception{
		
		try {
			
			ItpamWebServiceCLIUtils.lInteractionId = Long.parseLong(strValue);
		}
		catch (Exception e){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_INTERACTION_ID + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_INTERACTION_ID + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Iterate command line arguments and set interactive mode flag to true if "/i" was specified.
 * 
 * @param aryArgs - String array of command line arguments
 * 																													*/
	private static void setInteractiveModeFlag(String[] aryArgs){
		
	//	Iterate command line arguments and set interactive mode flag to true if "/i" was specified
		for (int intArg = 0; intArg < aryArgs.length; intArg++){
			
			if (   (aryArgs[intArg].toLowerCase().startsWith("/i"))
				|| (aryArgs[intArg].toLowerCase().startsWith("-i"))){
				
				ItpamWebServiceCLI.bolInteractiveMode = true;
				ItpamWebServiceCLI.logger.debug("Interactive mode enabled");
				break;
			}
		}
	}
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to log level specified by the user
 * 																													*/
	private static void setLevel(String strValue){
		
		ItpamWebServiceCLIUtils.strLevel = strValue;
	}
	

/**
 * Validate and set property value.
 * 
 * @param strValue - String set to levels to search relative to source library path specified by the user
 * 
 * @throws Exception 
 */
	private static void setLevelsToSearch(String strValue) throws Exception {
		
		try {
			
			ItpamWebServiceCLIUtils.intLevelsToSearch = Integer.parseInt(strValue);
		}
		catch (Exception e){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_INTEGER_PARAMETER_VALUE") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_LEVELS_TO_SEARCH + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_INTEGER_PARAMETER_VALUE") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_LEVELS_TO_SEARCH + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
private static void setLogLevel(String[] args){
	
	if (args != null){
		
		for (int intArg=0; intArg < args.length; intArg++){
			
			if (args[intArg].startsWith(ItpamWebServiceCLI.ARG_NAME_LOG_LEVEL)){
				
				if (args[intArg].split("=").length == 2){
					
					System.setProperty(ItpamWebServiceCLI.ARG_NAME_LOG_LEVEL,
							             args[intArg].split("=")[1]);
				}
			}
		}
	}
	
}

/**
 * Validate and set output format property.
 * 
 * @param strValue - String set to action specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setOutputFormat(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_OUTPUT_FORMAT_PARAMETER_VALUE") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_OUTPUT_FORMAT + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_OUTPUT_FORMAT_PARAMETER_VALUE") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_OUTPUT_FORMAT + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (!((strValue.equalsIgnoreCase(ItpamWebServiceCLI.OUTPUT_FORMAT_PROPERTIES))
				|| (strValue.equalsIgnoreCase(ItpamWebServiceCLI.OUTPUT_FORMAT_XML)))){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_OUTPUT_FORMAT_PARAMETER_VALUE") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_OUTPUT_FORMAT + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_OUTPUT_FORMAT_PARAMETER_VALUE") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_OUTPUT_FORMAT + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else {
			
			ItpamWebServiceCLI.strOutputFormat = strValue;
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to make available flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setOverwriteTargetFile(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_OVERWRITE_TARGET_FILE + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_OVERWRITE_TARGET_FILE + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolOverwriteTargetFile = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolOverwriteTargetFile = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_OVERWRITE_TARGET_FILE + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_OVERWRITE_TARGET_FILE + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set ITPAM user password property.
 * 
 * @param strPassword - String set to ITPAM password specified by the user
 * 																													*/
	private static void setPassword(String strPassword){
			
		ItpamWebServiceCLIUtils.strItpamPassword = strPassword;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to process ID specified by the user
 * 																													*/
	private static void setProcessId(String strValue) throws Exception{
		
		try {
			
			ItpamWebServiceCLIUtils.lProcessId = Long.parseLong(strValue);
		}
		catch (Exception e){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_PROCESS_ID + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_PROCESS_ID + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to process name specified by the user
 * 																													*/
	private static void setProcessName(String strValue){
		
		ItpamWebServiceCLIUtils.strProcessName = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to recursive flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setRecursive(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_RECUSIVE + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_RECUSIVE + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolRecursive = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolRecursive = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_RECUSIVE + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_RECUSIVE + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to search path specified by the user
 * 																													*/
	private static void setSearchPath(String strValue){
		
		ItpamWebServiceCLIUtils.strSearchPath = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to search path specified by the user
 * 																													*/
	private static void setSourceLibraryPath(String strValue){
		
		ItpamWebServiceCLIUtils.strSourceLibraryPath = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to start date specified by the user
 * 																													*/
	private static void setStartDate(String strValue){
		
		ItpamWebServiceCLIUtils.strStartDate = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to start request ID specified by the user
 * 																													*/
	private static void setStartRequestId(String strValue) throws Exception{
		
		try {
			
			ItpamWebServiceCLIUtils.lStartRequestId = Long.parseLong(strValue);
		}
		catch (Exception e){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_START_REQUEST_ID + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_START_REQUEST_ID + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to start date specified by the user
 * 																													*/
	private static void setStartRequestName(String strValue){
		
		ItpamWebServiceCLIUtils.strStartRequestName = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to start request path specified by the user
 * 																													*/
	private static void setStartRequestPath(String strValue){
		
		ItpamWebServiceCLIUtils.strStartRequestPath = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to start time specified by the user
 * 																													*/
	private static void setStartTime(String strValue){
		
		ItpamWebServiceCLIUtils.strStartTime = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to search path specified by the user
 * 																													*/
	private static void setTargetFileName(String strValue){
		
		ItpamWebServiceCLIUtils.strTargetFileName = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to search path specified by the user
 * 																													*/
	private static void setTargetPath(String strValue){
		
		ItpamWebServiceCLIUtils.strTargetPath = strValue;
	}
	
	
/**
 * Validate and set property value.
 * 
 * @param strValue - String set to target path is local flag specified by the user
 * 
 * @throws ItpamCLIParameterMissingOrInvalidException
 * 																													*/
	private static void setTargetPathIsLocal(String strValue)
	throws Exception
	{
		
		if (ItpamWebServiceCLIUtils.isNullEmpty(strValue)){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_TARGET_PATH_IS_LOCAL + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_TARGET_PATH_IS_LOCAL + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (strValue.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolTargetPathIsLocal = true;		
		}
		else if (strValue.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolTargetPathIsLocal = false;		
		}
		else {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_TARGET_PATH_IS_LOCAL + "=" + strValue;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALIDE_PARAMETER_VALUE_BOOLEAN_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_TARGET_PATH_IS_LOCAL + "=" + strValue;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}
	

/**
 * Validate and set ITPAM URI property.
 * 
 * @param strUri - String set to ITPAM URI specified by the user
 * 																													*/
	private static void setUri(String strUri){
		
		ItpamWebServiceCLIUtils.strItpamUri = strUri;
	}
	
	private static String getUri(){
		return ItpamWebServiceCLIUtils.strItpamUri;
	}
	
/**
 * Validate and set ITPAM user name property.
 * 
 * @param strUser - String set to ITPAM user name specified by the user
 * 																													*/
	private static void setUser(String strUser){
		
		ItpamWebServiceCLIUtils.strItpamUser = strUser;
	}
	
	
/**
 * Validate and set ITPAM version property.
 * 
 * @param strVersion - String set to ITPAM version specified by the user
 * 																													*/
	private static void setVersion(String strVersion){
		
		ItpamWebServiceCLI.strItpamVersion = strVersion;
	}	
	
	private static void setIsAutoArchive(String bolIsAutoArchive) {
		if (bolIsAutoArchive.equalsIgnoreCase("true")) {
			
			ItpamWebServiceCLIUtils.bolIsAutoArchive = true;		
		}
		else if (bolIsAutoArchive.equalsIgnoreCase("false")) {
			
			ItpamWebServiceCLIUtils.bolIsAutoArchive = false;		
		} 
		else {
			
			ItpamWebServiceCLIUtils.bolIsAutoArchive = true;		
		}
	}

	private static void setTagId(String strTagId) {
		ItpamWebServiceCLIUtils.strTagId = strTagId;
	}

	private static void setPriority(String strPriority) {
		ItpamWebServiceCLIUtils.strPriority = strPriority;
	}	
	
	private static void setInstanceId(String instanceId) throws Exception{
		
		try {
			
			ItpamWebServiceCLIUtils.lInstanceId = Long.parseLong(instanceId);
		}
		catch (Exception e){

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
			                           + ItpamWebServiceCLI.ARG_NAME_INTERACTION_ID + "=" + instanceId;
					
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_PARAMETER_VALUE_NUMERIC_REQUIRED") + ": "
                              + ItpamWebServiceCLI.ARG_NAME_INTERACTION_ID + "=" + instanceId;
						
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
					
		//	throw exception
			throw new ItpamCLIParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
	}

	private static void setRootUuid(String strRootUuid) {
		ItpamWebServiceCLIUtils.strRootUuid = strRootUuid;
	}
	

/**
 * Store ITPAM server URI and version.
 * 
 * @throws ItpamCLIUnexpectedReturnException
 * 																													*/
	protected static void storeServiceConfig()
	throws Exception
	{

		try {
			
		//	Initialize server configuration properties object
			Properties propertiesServerConfig = new Properties();
			
		//	Set property values
			propertiesServerConfig.setProperty(ItpamWebServiceCLI.ARG_NAME_ITPAM_URI, ItpamWebServiceCLI.strItpamUri);
			if(ItpamWebServiceCLI.strItpamVersion != null) {
				propertiesServerConfig.setProperty(ItpamWebServiceCLI.ARG_NAME_ITPAM_VERSION, ItpamWebServiceCLI.strItpamVersion);
			}
			if (ItpamWebServiceCLI.strTrustStore != null)
				propertiesServerConfig.setProperty(ItpamWebServiceCLI.ARG_NAME_TRUST_STORE, ItpamWebServiceCLI.strTrustStore);
			
		//	Initialize file and file output stream
			File fileServerConfig = new File(ItpamWebServiceCLI.SERVER_CONFIG_PATH);
			fileServerConfig.mkdirs();
			fileServerConfig = new File(ItpamWebServiceCLI.SERVER_CONFIG_FILE);
            FileOutputStream fosServerConfig = new FileOutputStream(fileServerConfig);
			
		//	Save properties
			propertiesServerConfig.store(fosServerConfig, "ITPAM Server Configuration");
			
		//	Close file output stream
			fosServerConfig.close();
		}
		catch (IOException e) {

		//	Set exception messages
			String strLocalizedMessage = ItpamWebServiceCLI.messages.getLocalized("E_INVALID_CONFIG_FILE");
			if (!ItpamWebServiceCLI.isNullEmpty(e.getLocalizedMessage())){
				
				strLocalizedMessage = strLocalizedMessage + ": " + e.getLocalizedMessage();
			}
			
			String strMessage = ItpamWebServiceCLI.messages.get("E_INVALID_CONFIG_FILE") + ": " + e.getMessage();
				
		//	Record exception
			ItpamWebServiceCLIUtils.logger.error(strMessage);
			
		//	throw exception
			throw new ItpamCLIUnexpectedReturnException(strMessage, strLocalizedMessage);
		}
	}
}
