package com.ca.dso.itpam.webservice.client;

import java.rmi.RemoteException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import org.apache.log4j.Logger;

import com.ca.dso.resource.utils.Messages;


public abstract class ItpamWebServiceClient {	
	
/**	Public string constant required by CA																			*/
	public static final String CACOPYRIGHT = "(c) 2012 CA.  All rights reserved.";
	
/**	Exception class instantiated and thrown when an ITPAM entity is inaccessible or not found.						*/
	public class ItpamEntityNotFoundException extends Exception {

		private static final long serialVersionUID = -8201878518626987430L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamEntityNotFoundException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to localized exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}

/**	Exception class instantiated and thrown when a web service method is not supported.								*/
	public class ItpamMethodNotSupportedException extends Exception {

		private static final long serialVersionUID = -8201878518626987430L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamMethodNotSupportedException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to localized exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}

/**	Exception class instantiated and thrown when a required parameter is missing or invalid.						*/
	public class ItpamParameterMissingOrInvalidException extends Exception {

		private static final long serialVersionUID = -8201878518626987430L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamParameterMissingOrInvalidException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to localized exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}
	
/**	Exception class instantiated and thrown when an string to date conversion exception is thrown.					*/
	public class ItpamStringToDateException extends Exception {
	
		private static final long serialVersionUID = 8572778331476423880L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamStringToDateException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to localized exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}

/**	Exception class instantiated and thrown when a user does not have permissions to complete the operation.		*/
	public class ItpamUnknownHostException extends Exception {

		private static final long serialVersionUID = -8201878518626987430L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamUnknownHostException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to localized exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}
	
/**	Exception class instantiated and thrown when a user does not have permissions to complete the operation.		*/
	public class ItpamUserPermissionsException extends Exception {

		private static final long serialVersionUID = -8201878518626987430L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamUserPermissionsException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to localized exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}
	
/**	Exception class instantiated and thrown when an unsupported IT PAM parameter is detected.	*/
	public class ItpamUnsupportedParameterTypeException extends Exception {
	
		private static final long serialVersionUID = 8572778331476423880L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamUnsupportedParameterTypeException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to localized exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}
	
/**	Exception class instantiated and thrown when an unexpected exception is detected.								*/
	public class ItpamWebServiceClientException extends Exception {
	
		private static final long serialVersionUID = 8572778331476423880L;
		
		private String m_strLocalizedMessage = null;
		private String m_strMessage = null;

	/** Constructor 														*/
		public ItpamWebServiceClientException(String strMessage, String strLocalizedMessage){
			
			super();
			this.m_strLocalizedMessage = strLocalizedMessage;
			this.m_strMessage = strMessage;
		}
		
	/**
	 * Get localized exception message.
	 * 
	 * @return String set to localized exception message
	 * 																		*/
		public String getLocalizedMessage(){
			
			return this.m_strLocalizedMessage;
		}
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return this.m_strMessage;
		}
	}

/** Abstract ITPAM attachment class extended by version specific classes											*/
	public abstract class ItpamAttachment {
		
	/**	Constructor																									*/
		protected ItpamAttachment(){};
		
	/**	Get attachment ID. @return String set to attachment ID														*/
		public abstract String getAttachmentId();
		
	/**	Get content ID. @return String set to attachment content ID													*/
		public abstract String getContentId();
		
	/**	Get content type. @return String set to attachment content type												*/
		public abstract String getContentType();
		
	/** Get attachment name. @return String set to attachment name													*/
		public abstract String getName();
	}

/** Abstract ITPAM event data class extended by version specific classes											*/
	public abstract class ItpamEventData {
		
	/**	Constructor																									*/
		protected ItpamEventData(){};
		
	/**	Get event creation time. @return Calendar object set to event creation date and time						*/
		public abstract Calendar getCreationTime();
		
	/**	Get event creation time as a string. @return String representation of event creation date and time			*/
		public abstract String getCreationTimeString();
		
	/**	Get event destination. @return String set to event destination												*/
		public abstract String getDestination();
		
	/**	Get event expiration time. @return Calendar object set to event expiration date and time					*/
		public abstract Calendar getExpirationTime();
		
	/**	Get event expiration time as a string. @return String representation of event expiration date and time			*/
		public abstract String getExpirationTimeString();
		
	/**	Get event expression. @return String set to representation of event payload parameter names and
	 * 	values in format suitable for use as event search criteria													*/
		public abstract String getExpression();
		
	/** Get event name. @return String set to event name															*/
		public abstract String getName();
		
	/** Get single subscriber flag. @return boolean set to true if event is intended to be completely consumer by
	 *  first subscriber otherwise false																			*/
		public abstract boolean getSingleSubscriber();
		
	/** Get single subscriber flag as a string. @return String set to "true" if event is intended to be completely
	 * 	consumer by first subscriber otherwise false																*/
		public abstract String getSingleSubscriberString();
		
	/** Get event source. @return String set to event source														*/
		public abstract String getSource();
		
	/** Get event type. @return String set to event type															*/
		public abstract String getType();
		
	/** Get even user. @return String set to user that created the event											*/
		public abstract String getUser();
	}

/** Abstract ITPAM interaction class extended by version specific classes											*/
	public abstract class ItpamInteraction{
		
	/**	Constructor																									*/
		protected ItpamInteraction(){};
		
	/**	Get interaction description.																					*/
		public abstract String getDescription();
		
	/**	Get interaction ID.																							*/
		public abstract long getInteractionId();
		
	/**	Get "is approval required (supported)" flag.																							*/
		public abstract boolean getIsApprovalRequired();
		
	/**	Get interaction process ID.																					*/
		public abstract String getProcessId();
	}
	
/**	Abstract ITPAM parameter class extended by version specific concrete classes									*/
	public abstract class ItpamParameter{
		
	/**	Private String to store parameter data type property value													*/
		protected String strDataType = null;
		
	/**	Private Object to store default value property value														*/
		protected Object objDefaultValue = null;
		
	/**	Private String to store description property value															*/
		protected String strDescription = null;
		
	/**	Private boolean to store is read only property value														*/
		protected boolean bolIsReadOnly = false;
		
	/**	Private String to store label property value																*/
		protected String strLabel = null;
		
	/**	Private String to store mask property value																	*/
		protected String strMask = null;
		
	/**	Private long to store maximum length property value															*/
		protected long lngMaxLength = 0;
		
	/**	Private double to store maximum integer value property value												*/
		protected double dblMaxValue = 0;
		
	/**	Private long to store minimum length property value															*/
		protected long lngMinLength = 0;
		
	/**	Private double to store minimum integer value property value												*/
		protected double dblMinValue = 0;
		
	/**	Private String to store name property value																	*/
		protected String strName = null;
		
	/**	Protected String to store parent page name																	*/
		protected String strParentPageName = null;
		
	/**	Private int to store sequence number property value															*/
		protected int intSequenceNo = 0;
		
	/**	Private Object to store value property value																*/
		protected Object objValue = null;
		
	/** Constructor																									*/
		protected ItpamParameter(){};
		
	/** Get default value (version 2.2 or higher).																	*/
		public Object getDefaultValue(){
			
			return this.objDefaultValue;
		}
		
	/** Get description.																								*/
		public String getDescription(){
			
			return this.strDescription;
		}
		
	/** Get is read only.																							*/
		public boolean getIsReadOnly(){
			
			return this.bolIsReadOnly;
		}
		
	/** Get label.																									*/
		public String getLabel(){
			
			return this.strLabel;
		}
		
	/** Get mask.																									*/
		public String getMask(){
			
			return this.strMask;
		}
		
	/** Get maximum length.																							*/
		public long getMaxLength(){
			
			return this.lngMaxLength;
		}
		
	/** Get maximum value.																							*/
		public double getMaxValue(){
			
			return this.dblMaxValue;
		}
		
	/** Get minimum length.																							*/
		public long getMinLength(){
			
			return this.lngMinLength;
		}
		
	/** Get minimum value.																							*/
		public double getMinValue(){
			
			return this.dblMinValue;
		}
		
	/** Get name.																									*/
		public String getName(){
			
			return this.strName;
		}
		
	/** Get parent page name.																									*/
		public String getParentPageName(){
			
			return this.strParentPageName;
		}
		
	/** Get sequence number (version 2.2 or higher).																	*/
		public int getSequenceNo(){
			
			return this.intSequenceNo;
		}
		
	/** Get value.																									*/
		public Object getValue(){
			
			return this.objValue;
		}
	}

/**	Abstract ITPAM process log entry class extended by version specific concrete classes							*/
	public abstract class ItpamProcessLogEntry{
		
	/**	Constructor																									*/
		protected ItpamProcessLogEntry(){}
		
	/**	Get process log entry category. @return String set to process log entry category							*/
		public abstract String getCategory();
		
	/**	Get process log entry severity level.  @return String set to process log entry severity level				*/
		public abstract String getLevel();
		
	/** Get process log entry message.  @return String set to process log entry message								*/
		public abstract String getMessage();
		
	/**	Get process log entry time stamp.  @return String set to process log entry time stamp								*/
		public abstract String getTimeStamp();
	}
	
/**	Abstract ITPAM process status class extended by version specific concrete classes,								*/
	public abstract class ItpamProcessStatus {
			
	/**	Constructor																									*/
		protected ItpamProcessStatus(){};
		
	/** Get process connection ID. @return String set to process connection ID										*/
		public abstract String getConnectionId();
		
	/**	Get process message. @return String set to process message													*/
		public abstract String getMessage();
		
	/**	Get process status message. @return String set process status message										*/
		public abstract String getMessageStatus();
		
	/** Get process display name. @return String set to process display name										*/
		public abstract String getProcessDisplayName();
		
	/**	Get process ID. @return String set to process ID															*/
		public abstract String getProcessId();
		
	/** Get process instance name. @return String set to process instance name.										*/
		public abstract String getProcessInstanceName();
		
	/**	Get fully qualified process library path and name. @return string set to fully qualified process library
	 *	and name																									*/
		public abstract String getProcessLibraryPathName();

	/** Get process state. @return String set to process state														*/
		public abstract String getProcessState();
		
	/**	Get process sender address. @return String set to process sender address									*/
		public abstract String getSenderAddress();
		
	/**	Get process submission time. @return String set to process submission time									*/
		public abstract String getSubmissionTime();
		
	/**	Get process parameters. @return Properties object encapsulating process parameter names and values			*/
		public abstract Properties getParameters();
	};
	
/**	Abstract ITPAM start request status class extended by version specific concrete classes							*/
	public abstract class ItpamStartRequestForm {
		
	/**	Constructor																									*/
		protected ItpamStartRequestForm(){};
		
	/**	Get start request form description.  @return String set to start request form description					*/
		public abstract String getDescripition();
		
	/**	Get start request form name.  @return String set to start request form name									*/
		public abstract String getName();
		
	/**	Get start request form path.  @return String set to start request form path									*/
		public abstract String getPath();
	}
	
/**	Abstract ITPAM start request status class extended by version specific concrete classes							*/
	public abstract class ItpamStartRequestStatus {
		
		protected ItpamStartRequestStatus (){};

	/** Get start request status information. @return String set to request earliest start date and time				*/
		public abstract String getEarliestStartTime();
		
	/**	Get start request status information. @return String set to request end date and time						*/
		public abstract String getEndTime();
		
	/**	Get start request status information. @return String set to request interaction ID value						*/
		public abstract String getInteractionId();
		
	/**	Get start request status information. @return String set to corresponding process ID value					*/
		public abstract String getProcessId();
		
	/**	Get start request status information, @return String set to referenced process fully qualified name			*/
		public abstract String getReferencedProcess();
		
	/**	Get start request status information. @return String set to request start date and time						*/
		public abstract String getStartTime();
		
	/**	Get start request status information @return String set to request state (ex. "Running")					*/
		public abstract String getState();
	};
	
/** Protected logger to record exception messages																	*/	
	protected static Logger logger = Logger.getRootLogger();
	
/** Protected string to store exception messages																	*/	
	protected Messages _messages = null;
	
/**	Protected string to store reference to ITPAM user password														*/
	protected String _strPassword = null;
	
/**	Protected string to store process name																			*/
	protected String _strProcessName = null;
	
/** Protected String to store start request name																	*/
	protected String _strStartRequestName = null;
	
/**	Protected String to store start request path																	*/
	protected String _strStartRequestPath = null;

/**	Protected string to store reference to target end point URI														*/
	protected String _strTargetEndpointURI = null;
	
/**	Protected string sto store reference to target end point version												*/
	protected String _strTargetEndpointVersion = null;
	
/**	Protected string to store reference to authorized ITPAM user name												*/
	protected String _strUser = null;
	
	
/**
 * Protected constructor "supered" by extending version specific concrete classes.
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	protected ItpamWebServiceClient(String strTargetEndpointVersion,
									String strTargetEndpointURI,
									String strUser,
									String strPassword)
	throws Exception
	{		
		this._strTargetEndpointVersion = strTargetEndpointVersion;
		this._strTargetEndpointURI = strTargetEndpointURI;
		this._strUser = strUser;
		this._strPassword = strPassword;
		
	//	Initialize message resource bundle
		this._messages = new Messages(ItpamWebServiceClient.class.getPackage().getName().concat(".resource.Message"));
	}
	
	
	public String get_strTargetEndpointVersion() {
		return _strTargetEndpointVersion;
	}


/**
 * Abort process specified by process ID (2.2 and higher).
 * 
 * @param lProcessId - Long process ID value
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void abortProcess(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Approve pending user interaction request.
 * 
 * @param lInteractionId - long user interaction ID value
 * @param propertiesParameters - Properties object encapsulating parameter names and values
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void approvePendingInteraction(long lInteractionId, Properties propertiesParameters)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Delete archived instances in date range specified (2.2 and higher).
 * 
 * @param strStartDate - String set to start date of date range
 * @param strEndDate - String set to end date of date rangef
 * 
 * <br>
 * <br>Supported date formats 'yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy'
 * <br>Supported time formats 'hh:mm:ss', 'hh:mm'
 * <br>
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void deleteArchivedInstances(String strStartDate, String strEndDate)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Delete specified attachment.
 * 
 * @param lAttachmentId - long attachment ID value to be deleted
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void deleteAttachment(long lAttachmentId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;


/**
 * Delete specified attachments.
 * 
 * @param aryAttachmentIds - Array of long attachment ID values
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void deleteAttachments(long[] aryAttachmentIds)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Export object definition(s) to XML file (3.0 and higher).
 * 
 * @param strSourcePath - String set to source library path for object definition(s) to be exported
 * @param intLevelsToSearch - Integer set to levels to search for object definition(s) to be exported
 * @param bolIsAbsoluteSourcePath - Boolean set to true if source path references in the exported object
 * 		definition(s) should be absolute or false to use relative references
 * @param strTargetPath - String set to target path for exported definition(s)
 * @param strTargetFileName - String set to target file name for exported definition(s)
 * @param bolTargetPathIsLocal - Boolean set to true if target path/file name is local
 * @param bolOverwriteTargetFile - Boolean set to true to overwrite target file if it exists
 * @param aryObjectTypeFilterStrings - Array of strings set to object types of objects in the source path to
 * 		be exported
 * <br><br>	
 * 		Supported object type filter strings:
 * <br>	- Agenda
 * <br>	- Calendar
 * <br>	- Commander
 * <br>	- CustomIcon
 * <br>	- CustomOperator
 * <br>	- CustomSensor
 * <br>	- Dataset
 * <br>	- InteractionRequestForm
 * <br>	- LogViewer
 * <br>	- Package
 * <br>	- Process
 * <br>	- ProcessWatch
 * <br>	- Resources
 * <br>	- StatePolicy
 * <br>	- System
 * <br>
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public abstract void exportObject(String strSourcePath, int intLevelsToSearch, boolean bolIsAbsoluteSourcePath, 
							 		  String strTargetPath, String strTargetFileName, boolean bolTargetPathIsLocal, 
							 		  boolean bolOverwriteTargetFile, String[] aryObjectTypeFilterStrings)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;


/**
 * Generate event specified (3.0 and higher).
 * 
 * @param strEventName - Set set to an event name that will recognized by the process operator(s) waiting for the event 
 * @param strEventType - String set to an event type that will recognized by the process operator(s) waiting for the event
 * 	or null
 * @param strEventDestination - String set to an event destination that will recognized by the process operator(s) waiting
 *	for the event or null
 * @param strEventSource - String set to an event source that will recognized by the process operator(s) waiting
 *	for the event or null
 * @param lEventExpirationDuration - long set to time in seconds before the event expires
 * @param bolEventSingleSubscriber - boolean value set to true if one and only one process operator may consume the event
 *  (i.e., the first operator waiting for a matching event will "consume" the event - it will not be visible to other
 *  waiting process operators waiting for events)
 * @param strEventPayload - String set to content that will be used by the waiting process(es) or null
 * @param propertiesParameters - Properties object encapsulating name/value pairs of parameter names and corresponding
 *  values to be used by the waiting process(es) or null
 * <br><br>
 * <b>NOTE:</b> Event pay load will be superseded by parameters.
 * <br>
 * 
 * @return ItpamEventData object for the event generated
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public abstract ItpamEventData generateEvent(String strEventName, String strEventType, String strEventDestination,
												 String strEventSource, long lEventExpirationDuration, boolean bolEventSingleSubscriber,
												 String strEventPayload, Properties propertiesParameters)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
/**
 * Get information for existing attachments.
 * 
 * @param strContentId - String set to content ID or null
 * @param strContentType - String set to content Type or null
 * 
 * @return Vector of ItpamAttachmentInfo objects
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract Vector<ItpamAttachment> getAttachments(String strContentId, String strContentType)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;

	
/**
 * Get event data for events matching specified criteria (3.0 and higher).
 * 
 * @param strEventName - String set to an event name that will recognized by the process operator(s) waiting for the event (required) 
 * @param strEventType - String set to an event type that will recognized by the process operator(s) waiting for the event or null
 * @param strEventDestination - String set to an event destination that will recognized by the process operator(s) waiting
 *	for the event or null
 * @param strEventSource - String set to an event source that will recognized by the process operator(s) waiting
 *	for the event or null
 * @param strEventExpression - String set to an expression that will be compared to event "payload" parameter(s)
 * 	(ex. payload.paramName=="paramValue") or null
 * @param bolAllowPartialMatch - boolean value set to true to return data for events that partially match the specified
 * 	criteria
 * @param bolEnablePatternMatch - boolean value set to true if criteria specified includes regex-type patterns
 * <br>
 * 
 * @return Vector of ItpamEventData objects for events matching specified criteria
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public abstract Vector<ItpamEventData> getMatchingEvents(String strEventName, String strEventType, String strEventDestination,
								  String strEventSource, String strEventExpression, boolean bolAllowPartialMatch,
								  boolean bolEnablePatternMatch)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Get information related to pending user interaction specified.
 * 
 * @param lInteractionId - long pending user interaction request ID value
 * 
 * @return ItpamInteraction object
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract ItpamInteraction getPendingInteraction(long lInteractionId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;

	
/**
 * Get parameter information for pending user interaction specified.
 * 
 * @param lInteractionId - long pending user interaction request ID value
 * 
 * @return Vector of ItpamParameter objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public abstract Vector<ItpamParameter> getPendingInteractionParameters(long lInteractionId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamUnsupportedParameterTypeException,
    	   ItpamWebServiceClientException;

/**
 * Get pending interaction information.
 * 
 * @return Vector of ItpamInteraction objects.
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract Vector<ItpamInteraction> getPendingInteractions()
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;


/**
 * Get process logs (2.2 and higher).
 * 
 * @param lProcessId - long process ID value
 * @param strLevel - String set to minimum severity level to be returned
 * 		  (Valid Levels: Level: error ,warning, notice, and normal; If the user specifies an invalid level, it will be treated as normal (the lowest level)
 * @param aryCategories - Array of Strings set to log categories
 *        (i.e. Operator, Process, userMsg; ignored if invalid string specified)
 * 
 * @return Vector of ItpamProcessLogEntry objects
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract Vector<ItpamProcessLogEntry> getProcessLogs(long lProcessId, String strLevel, String[] aryCategories)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;

	
/**
 * Get list of pending interactions related to the process ID specified.
 * 
 * @param lProcessId - long process ID value
 * 
 * @return Vector of ItpamInteraction objects
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract Vector<ItpamInteraction> getProcessPendingInteractions(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;


/**
 * Get current status of process specified.
 * 
 * @param lProcessId - long process ID value
 * 
 * @return ItpamProcessStatus object encapsulating status properties
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract ItpamProcessStatus getProcessStatus(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;

/**
 * Get current status of process specified.  (4.0 and higher).
 * 
 * @param lProcessId - long process ID value
 * @param tagId - String tag Id value
 * 
 * @return ItpamProcessStatus object encapsulating status properties
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public  ItpamProcessStatus getProcessStatus(long lProcessId, String tagId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException {
		logger.debug("Calling getProcessStatus against PAM 3.0 stubs, ignoring tagId input as it is only supported in 4.0 stubs.");
		return getProcessStatus(lProcessId); 
	}
	
/**
 * Get current status of the ITPAM web service server.
 * 
 * @return String set to server status
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract String getServerStatus()
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	/**
	 * Get Version of the ITPAM web service server.
	 * Default implementation returns empty string
	 * 
	 * @return String set to version
	 * 
	 * @throws ItpamEntityNotFoundException
	 * @throws ItpamMethodNotSupportedException
	 * @throws ItpamParameterMissingOrInvalidException
	 * @throws ItpamStringToDateException
	 * @throws ItpamUnknownHostException
	 * @throws ItpamUserPermissionsException
	 * @throws ItpamWebServiceClientException
	 */	
	public String getITPamVersionInfo()
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException {
		return "";
	}
	
/**
 * Search for and return available start request forms.
 * 
 * @param aryKeywords - Array of Strings set to to keywords to used in search filter (Note: Ignored for version 2.1)
 * @param strSearchPath - String set to search path to be set as the root folder of the search (Note: Ignored for version 2.1)
 * @param bolRecursive - boolean set to true to search through child folders of the root folder specified (Note: Ignored for version 2.1)
 * 
 * @return Vector of ItpamStartRequestForm objects
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract Vector<ItpamStartRequestForm> getStartRequestForms(String[] aryKeywords, String strSearchPath, boolean bolRecursive)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;


/**
 * Get parameters for specified start request form.
 * 
 * @param strStartRequestName - String set to start request form name
 * @param strStartRequestPath - String set to start request form source library path
 * 
 * @return Vector of ItpamParameter objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamUnsupportedParameterTypeException
 * @throws ItpamWebServiceClientException 
 */
	public abstract Vector<ItpamParameter> getStartRequestParameters(String strStartRequestName, String strStartRequestPath)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamUnsupportedParameterTypeException,
    	   ItpamWebServiceClientException;


/**
 * Get process name referenced by the start request form specified (3.0 and higher).
 * 
 * @param strStartRequestName - String set to start request form name
 * @param strStartRequestPath - String set to start request form library path
 * 
 * @return String set to fully qualified process name referenced by the start request form specified
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract String getStartRequestProcessReferenced(String strStartRequestName, String strStartRequestPath)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;

	
/**
 * Get start request status object that includes the following information:
 * <br>- Earliest Start Time
 * <br>- End Time
 * <br>- Interaction ID
 * <br>- Referenced Process
 * <br>- Start Time
 * <br>- State
 * 
 * @param lStartRequestId - long start request ID value
 * 
 * @return ItpamStartRequestStatus object that exposes methods to detailed status information
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract ItpamStartRequestStatus getStartRequestStatus(long lStartRequestId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;

/**
 * Get start request status object that includes the following information: (4.0 and higher).
 * <br>- Earliest Start Time
 * <br>- End Time
 * <br>- Interaction ID
 * <br>- Referenced Process
 * <br>- Start Time
 * <br>- State
 * 
 * @param lStartRequestId - long start request ID value
 * @param tagId - String tag ID
 * 
 * @return ItpamStartRequestStatus object that exposes methods to detailed status information
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public  ItpamStartRequestStatus getStartRequestStatus(long lStartRequestId, String tagId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException {
		logger.debug("Calling getStartRequestStatus against PAM 3.0 stubs, ignoring tagId input as it is only supported in 4.0 stubs.");
		return getStartRequestStatus(lStartRequestId); 
	}

/**
 * Set an instance of a Process or Start Request Form to be archived.
 * 
 * @param rootUuid - String Root UUID
 * @param instanceId - String Instance ID
 * @param tagId - String Tag ID
 * 
 * @return controlInstanceStatus object encapsulating status properties
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
public  String controlInstance(String rootUuid, long instanceId, String tagId)
throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException {	
	// This method is supported in PAM 4.0 and higher

	//	Method not supported
	String strMessage = this._messages.get("E_METHOD_NOT_SUPPORTED") + " " + this._strTargetEndpointVersion;
	String strLocalizedMessage = this._messages.getLocalized("E_METHOD_NOT_SUPPORTED") + " " + this._strTargetEndpointVersion;

	//	Record exception
	ItpamWebServiceClient.logger.error(strMessage);

	throw new ItpamMethodNotSupportedException(strMessage, strLocalizedMessage);
}

/**
 * Instantiate and return type specific exception based on properties of generic exception specified.
 * 
 * @param e - Exception object
 * @param strMessage - String set to exception message by calling method
 * @param strLocalized - String set to localized exception message set by calling method
 * 
 * @return type specific Exception
 * 
 * @throws Exception
 */
	protected Exception getTypeSpecificException(Exception e, String strMessage, String strLocalizedMessage)
	{
	//	Initialize return exception
		Exception eReturn = new Exception();
			
		try {
			
		//	Initialize exception messages
			if (strMessage == null) strMessage = "";
			if (!strMessage.equals("")) strMessage = strMessage + "; ";
			if (strLocalizedMessage == null) strLocalizedMessage = "";
			if (!strLocalizedMessage.equals("")) strLocalizedMessage = strLocalizedMessage + "; ";
			
		//	Initialize message strings if null
		//	Inspection exception properties and record/throw specific exception type
			if (e.getClass().getSimpleName().equals("AxisFault")){
				
				RemoteException eRemote = (RemoteException) e;
				
				if (e.getMessage().contains("instance does not exist")){
					
				//	Set exception messages
					strLocalizedMessage = strLocalizedMessage + this._messages.getLocalized("E_MISSING_ITPAM_ENTITY");
					if (!this.isNullEmpty(e.getLocalizedMessage())){
						
						strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
					}
					strMessage = strMessage + this._messages.get("E_MISSING_ITPAM_ENTITY") + "; " + e.getMessage();
					
					strLocalizedMessage = strLocalizedMessage.replaceAll("  Please refresh the screen.", "");
					strMessage = strMessage.replaceAll("  Please refresh the screen.", "");
					
				//	Record exception
					ItpamWebServiceClient.logger.error(strMessage);
					
				//	Set return exception
					eReturn = new ItpamEntityNotFoundException(strMessage, strLocalizedMessage);
				}
				else if (e.getMessage().contains("Authentication failed")){
					
				//	Set exception message
					strLocalizedMessage = strLocalizedMessage + this._messages.getLocalized("E_ACCESS_DENIED");
					if (!this.isNullEmpty(e.getLocalizedMessage())){
						
						strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
					}
					strMessage = strMessage + this._messages.get("E_ACCESS_DENIED") + "; " + e.getMessage();
					
				//	Set return exception
					eReturn = new ItpamUserPermissionsException(strMessage, strLocalizedMessage);
				}
				if (eRemote.detail != null){
					if (eRemote.detail.getClass().getSimpleName().equals("UnknownHostException")){
						
					//	Set exception message
						strLocalizedMessage = strLocalizedMessage + this._messages.getLocalized("E_INVALID_HOST");
						if (!this.isNullEmpty(e.getLocalizedMessage())){
							
							strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
						}
						strMessage = strMessage + this._messages.get("E_INVALID_HOST") + "; " + e.getMessage();
						
					//	Record exception
						ItpamWebServiceClient.logger.error(strMessage);
						
					//	Set return exception
						eReturn = new ItpamUnknownHostException(strMessage, strLocalizedMessage);
					}
					else {
						
					//	Set exception message
						strLocalizedMessage = strLocalizedMessage + this._messages.getLocalized("E_UNEXPECTED_EXCEPTION");
						if (!this.isNullEmpty(e.getLocalizedMessage())){
							
							strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
						}
						strMessage = strMessage + this._messages.get("E_UNEXPECTED_EXCEPTION") + "; " + e.getMessage();
						
					//	Record exception
						ItpamWebServiceClient.logger.error(strMessage);
						
					//	Set return exception
						eReturn = new ItpamWebServiceClientException(strMessage, strLocalizedMessage);
					}
				}
				else {
						
				//	Set exception message
					strLocalizedMessage = strLocalizedMessage + this._messages.getLocalized("E_UNEXPECTED_EXCEPTION");
					if (!this.isNullEmpty(e.getLocalizedMessage())){
						
						strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
					}
					strMessage = strMessage + this._messages.get("E_UNEXPECTED_EXCEPTION") + "; " + e.getMessage();
					
				//	Record exception
					ItpamWebServiceClient.logger.error(strMessage);
					
				//	Set return exception
					eReturn = new ItpamWebServiceClientException(strMessage, strLocalizedMessage);
				}		
			}
			else if (e.getClass().getSimpleName().equals("ItpamMethodNotSupportedException")){
						
			//	Set exception message
				strLocalizedMessage = strLocalizedMessage + this._messages.getLocalized("E_METHOD_NOT_SUPPORTED");
				if (!this.isNullEmpty(e.getLocalizedMessage())){
					
					strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
				}
				strMessage = strMessage + this._messages.get("E_METHOD_NOT_SUPPORTED") + "; " + e.getMessage();
				
			//	Record exception
				ItpamWebServiceClient.logger.error(strMessage);
				
			//	Set return exception
				eReturn = new ItpamWebServiceClientException(strMessage, strLocalizedMessage);
			}
			else if (e.getClass().getSimpleName().equals("ItpamParameterMissingOrInvalidException")){
				
			//	Set exception message
				strLocalizedMessage = strLocalizedMessage + this._messages.getLocalized("E_MISSING_OR_INVALID_PARAMETER");
				if (!this.isNullEmpty(e.getLocalizedMessage())){
					
					strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
				}
				strMessage = strMessage + this._messages.get("E_MISSING_OR_INVALID_PARAMETER") + "; " + e.getMessage();
				
			//	Record exception
				ItpamWebServiceClient.logger.error(strMessage);
				
			//	Set return exception
				eReturn = new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
			}
			else if (e.getClass().getSimpleName().equals("ItpamStringToDateException")){
				
			//	Set exception message
				strLocalizedMessage = strLocalizedMessage +  this._messages.getLocalized("E_DATE_PARSING_EXCEPTION");
				if (!this.isNullEmpty(e.getLocalizedMessage())){
					
					strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
				}
				strMessage = strMessage + this._messages.get("E_DATE_PARSING_EXCEPTION") + "; " + e.getMessage();
				
			//	Record exception
				ItpamWebServiceClient.logger.error(strMessage);
				
			//	Set return exception
				eReturn = new ItpamStringToDateException(strMessage, strLocalizedMessage);
			}
			else {
				
			//	Set exception message
				strLocalizedMessage = strLocalizedMessage + this._messages.getLocalized("E_UNEXPECTED_EXCEPTION");
				if (!this.isNullEmpty(e.getLocalizedMessage())){
					
					strLocalizedMessage = strLocalizedMessage + "; " + e.getLocalizedMessage();
				}
				strMessage = strMessage + this._messages.get("E_UNEXPECTED_EXCEPTION") + "; " + e.getMessage();
				
			//	Record exception
				ItpamWebServiceClient.logger.error(strMessage);
				
			//	Set return exception
				eReturn = new ItpamWebServiceClientException(strMessage, strLocalizedMessage);
			}
		}
		catch (Exception eUnexpected){
			
			eReturn = eUnexpected;
		}
		
	//	Return exception
		return eReturn;
	}


/**
 * Import file into ITPAM library (2.2 and higher).
 * 
 * @param strSourcePathFileName - String set to path and file name of file to be imported
 * @param strTargetLibraryPathName - String set to target library path and object name
 * @param bolMakeAvailable - boolean set to true if imported file should be flagged as available
 * @param bolMakeCurrent - boolean set to true if imported file should be flagged as the current version
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void importObject(String strSourcePathFileName, String strTargetLibraryPathName, boolean bolMakeAvailable, boolean bolMakeCurrent)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;

/**
 * Import file into ITPAM library (4.0 and higher).
 * 
 * @param strSourcePathFileName - String set to path and file name of file to be imported
 * @param strTargetLibraryPathName - String set to target library path and object name
 * @param bolMakeAvailable - boolean set to true if imported file should be flagged as available
 * @param bolMakeCurrent - boolean set to true if imported file should be flagged as the current version
 * @param OverwriteAction - string set to incrementObjectVersion, skipImport, or replaceObject
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public void importObject(String strSourcePathFileName, String strTargetLibraryPathName, boolean bolMakeAvailable, boolean bolMakeCurrent, String overwriteAction)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException {
		logger.debug("Calling importObject against PAM 3.0 stubs, ignoring overwriteAction input as it is only supported in 4.0 stubs.");
		importObject(strSourcePathFileName, strTargetLibraryPathName, bolMakeAvailable, bolMakeCurrent);
	}

	
/**
 * Reject pending user interaction request.
 * 
 * @param lInteractionId - long user interaction ID value
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void rejectPendingInteraction(long lInteractionId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Resume process specified by process ID (2.2 and higher).
 * 
 * @param lProcessId - long process ID value
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void resumeProcess(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Start process specified.
 * 
 * @param strProcessName - String set to fully qualified process name
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') or null. Required if
 * parameter 'strStartTime' is specified
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm') or null
 * @param strParameterNameAttachments - String set to parameter name attachments will be linked or null. Required if attachment
 * properties are specified.
 * @param propertiesAttachments - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return long process ID value
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract long startProcess(String strProcessName,
							 		  Properties propertiesParameters,
							 		  String strStartDate, String strStartTime,
							 		  String strParameterNameAttachments, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Start process specified.  (4.0 and higher).
 * 
 * @param strProcessName - String set to fully qualified process name
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') or null. Required if
 * parameter 'strStartTime' is specified
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm') or null
 * @param tagId - String tag ID
 * @param isAutoArchive - Boolean 
 * @param priority - String
 * @param strParameterNameAttachments - String set to parameter name attachments will be linked or null. Required if attachment
 * properties are specified.
 * @param propertiesAttachments - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return long process ID value
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public long startProcess(String strProcessName,
							 		  Properties propertiesParameters,
							 		  String strStartDate, String strStartTime,
							 		  String tagId, Boolean isAutoArchive, String priority, 
							 		  String strParameterNameAttachments, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException {
		logger.debug("Calling startProcess against PAM 3.0 stubs, ignoring input: tagId, isAutoArchive, and priority as they are only supported in 4.0 stubs.");
		return startProcess(strProcessName, propertiesParameters, strStartDate, strStartTime, strParameterNameAttachments, propertiesAttachments);
	}
	

/**
 * Initiate process via start request form.
 * 
 * @param strName - String set to start request form name
 * @param strPath - String set to start request form library path
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') or null
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm') or null
 * @param strParameterNameAttachments - String set to parameter name attachments will be linked or null. Required if attachment
 * properties are specified.
 * @param propertiesAttachments - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return long process ID value
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract long startRequest(String strName, String strPath, Properties propertiesParameters,
									    String strStartDate, String strStartTime,
									    String strParameterNameAttachments, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
	
/**
 * Initiate process via start request form.  (4.0 and higher).
 * 
 * @param strName - String set to start request form name
 * @param strPath - String set to start request form library path
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') or null
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm') or null
 * @param tagId - String tag ID
 * @param isAutoArchive - Boolean 
 * @param priority - String
 * @param strParameterNameAttachments - String set to parameter name attachments will be linked or null. Required if attachment
 * properties are specified.
 * @param propertiesAttachments - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return long process ID value
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public long startRequest(String strName, String strPath, Properties propertiesParameters,
									    String strStartDate, String strStartTime,
									    String tagId, Boolean isAutoArchive, String priority, 
									    String strParameterNameAttachments, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException {
		logger.debug("Calling startRequest against PAM 3.0 stubs, ignoring input: tagId, isAutoArchive, and priority as they are only supported in 4.0 stubs.");
		return startRequest(strName, strPath, propertiesParameters,	strStartDate, strStartTime, strParameterNameAttachments, propertiesAttachments);
	}
	
	
/**
 * Suspend process specified by process ID (2.2 and higher).
 * 
 * @param lProcessId - long process ID value
 * 
 * @throws ItpamEntityNotFoundException
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamUnknownHostException
 * @throws ItpamUserPermissionsException
 * @throws ItpamWebServiceClientException
 */
	public abstract void suspendProcess(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException;
	
/**
 * Get date from string formatted as a date/time.
 * 
 * @param strDate - String set to date/time in on of the recognized formats
 * 
 * <br>
 * <br>Supported date formats 'yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy'
 * <br>Supported time formats 'hh:mm:ss', 'hh:mm'
 * <br>
 * 
 * @throws ItpamStringToDateException
 */
	protected Date getDateFromString(String strDate)
	throws Exception
	{
		
	//	Initialize return value
		Date date = null;
		
	//	Attempt to get date from string if string is not null or empty
		if (!this.isNullEmpty(strDate)){
			
		//	Inspect date string to determine correct format string to use for parsing
			String strDateFormat = null;
			
			if (strDate.matches("[1-2][0-9][0-9][0-9]-.*")){
				strDateFormat = "yyyy-MM-dd";
			}
			else if (strDate.matches("[1-2][0-9][0-9][0-9]/.*")){
				strDateFormat = "yyyy/MM/dd";
			}
			else if (strDate.matches("[0-9]*-.*")){
				strDateFormat = "MM-dd-yyyy";
			}
			else if (strDate.matches("[0-9]*/.*")){
				strDateFormat = "MM/dd/yyyy";
			}
			else {
				
			//	Set exception message
				String strLocalizedMessage = this._messages.getLocalized("E_INVALID_DATE_FORMAT") + ": " + strDate;
				String strMessage = this._messages.get("E_INVALID_DATE_FORMAT") + ": " + strDate;
				
			//	Record exception
				ItpamWebServiceClient.logger.error(strMessage);
				
			//	Throw exception
				throw new ItpamStringToDateException(strMessage, strLocalizedMessage);
			}
			
			if (strDate.contains(":")){
				
				if (strDate.matches(".*[0-9]:[0-9][0-9]:[0-9][0-9]")){
					strDateFormat = strDateFormat + " hh:mm:ss";
				}
				else if (strDate.matches(".*[0-9]:[0-9][0-9]")){
					strDateFormat = strDateFormat + " hh:mm";
				}
				else {
					
				//	Set exception message
					String strLocalizedMessage = this._messages.getLocalized("E_INVALID_TIME_FORMAT") + ": " + strDate;
					String strMessage = this._messages.get("E_INVALID_TIME_FORMAT") + ": " + strDate;
					
				//	Record exception
					ItpamWebServiceClient.logger.error(strMessage);
					
				//	Throw exception
					throw new ItpamStringToDateException(strMessage, strLocalizedMessage);
				}
			}
			
		//	Attempt to parse date string using format string
		    try {
		    	
			    SimpleDateFormat dateFormat = new SimpleDateFormat(strDateFormat); 
				date = dateFormat.parse(strDate);
			}
		    catch (ParseException e) {
				
			//	Set exception message
				String strLocalizedMessage = this._messages.getLocalized("E_DATE_PARSING_EXCEPTION") + ": " + strDate;
				String strMessage = this._messages.get("E_DATE_PARSING_EXCEPTION") + ": " + strDate;
				
			//	Record exception
				ItpamWebServiceClient.logger.error(strMessage);
				
			//	Throw exception
				throw new ItpamStringToDateException(strMessage, strLocalizedMessage);
			}
			
		}
		else {
				
		//	Set exception message
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_PARAMETER") + ": " + strDate;
			String strMessage = this._messages.get("E_MISSING_PARAMETER") + ": " + strDate;
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
			//	Throw exception
				throw new ItpamStringToDateException(strMessage, strLocalizedMessage);
		}
	    
	//	Return date
	    return date;
	}
	
	
/**
 * Get single line string of property name/value pairs from properties object specified.
 * 
 * @param properties - Properties object
 * 
 * @return String set to single line of property name/value pairs from properties object specified
 */
	protected String getPropertiesList(Properties properties){
		
	//	Initialize return string
		String strProperties = "";
		
	//	Iterate properties
		for (Iterator<Object> iKeys = properties.keySet().iterator(); iKeys.hasNext();){
			
			String strName = (String) iKeys.next();
			String strValue = properties.getProperty(strName);
			
			if (!strProperties.equals("")) strProperties = strProperties + " ";
			
			strProperties = strProperties + strName + "=" + strValue;
		}		
		
	//	Return string
		return strProperties;
	}
	
	
/**
 * Inspect the string specified to determine if represents a numeric value.
 * 
 * @param strNumeric - String to be inspected
 * 
 * @return boolean true if string could successfully be converted to numeric
 * otherwise false
 * 																			*/
	protected boolean isNumeric(String strNumeric){
	
		try {
			Integer.parseInt(strNumeric);
			return true;
		}
		catch ( Exception e){			
			return false;
		}
	}
	
/**
 * Inspect string specified to determine if null or empty.
 * 
 * @param strString - String to inspect
 * 
 * @return boolean true if null or empty otherwise false
 */
	protected boolean isNullEmpty(String strString){
		
	//	Initialize boolean return value
		boolean bolNullEmpty = false;
		
	//	Reset boolean value if string is null or empty
		if (strString == null){
			
			bolNullEmpty = true;
		}
		else if (strString.equals("")){
			
			bolNullEmpty = true;
		}
		
		
	//	Return boolean value
		return bolNullEmpty;
	}
}
