package com.ca.dso.itpam.webservice.client;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import javax.activation.DataHandler;

import org.apache.axis2.context.MessageContext;
import org.apache.axis2.databinding.types.Time;
import org.apache.axis2.databinding.types.URI;
import org.apache.axis2.databinding.types.URI.MalformedURIException;

import com.ca.dso.itpam.webservice.stub_3_0.ItpamServiceStub;
import com.ca.dso.itpam.webservice.stub_3_0.ItpamServiceStub.*;

public class ItpamWebServiceClient_3_0 extends ItpamWebServiceClient {
	
	
/**	Public string constant required by CA																			*/
	public static final String CACOPYRIGHT = "(c) 2009 CA.  All rights reserved.";
	
/** Private version specific concrete class extending public abstract class ItpamAttachmentInfo						*/
	private class m_ItpamAttachment extends ItpamAttachment {
		
	/** Private instance of version specific Attachment object														*/
		private AttachmentInfo m_Attachment = null;
		
	/**	Constructor																									*/
		protected m_ItpamAttachment(AttachmentInfo attachmentInfo){
			
			super();
			this.m_Attachment = attachmentInfo;
		};
		
	/**	Get attachment ID. @return String set to attachment ID														*/
		public String getAttachmentId(){
			
			return Long.toString(this.m_Attachment.getAttachmentID());
		}
		
	/**	Get content ID. @return String set to attachment content ID													*/
		public String getContentId(){
			
			return this.m_Attachment.getContentId();
		}
		
	/**	Get content type. @return String set to attachment content type												*/
		public String getContentType(){
			
			return this.m_Attachment.getContentType();
		}
		
	/** Get attachment name. @return String set to attachment name													*/
		public String getName(){
			
			return this.m_Attachment.getName();
		}
	}
	
/** Abstract ITPAM event data class extended by version specific classes											*/
	private class m_ItpamEventData extends ItpamEventData {
		
	/** Private instance of version specific EventData object														*/
		private EventData m_EventData = null;
		
	/**	Constructor																									*/
		protected m_ItpamEventData(EventData m_EventData){
			
			super();
			this.m_EventData = m_EventData;
		};
		
	/**	Get event creation time. @return Calendar object set to event creation date and time						*/
		public Calendar getCreationTime(){
			
			return this.m_EventData.getEventCreationTime();
		}
		
	/**	Get event creation time as a string. @return String representation of event creation date and time			*/
		public String getCreationTimeString(){
			
			SimpleDateFormat sdf = new SimpleDateFormat();
			return sdf.format(this.m_EventData.getEventCreationTime().getTime());
		}
		
	/**	Get event destination. @return String set to event destination												*/
		public String getDestination(){
			
			return this.m_EventData.getEventDestination();
		}
		
	/**	Get event expiration time. @return Calendar object set to event expiration date and time					*/
		public Calendar getExpirationTime(){
			
			return this.m_EventData.getEventExpirationTime();
		}
		
	/**	Get event expiration time as a string. @return String representation of event expiration date and time			*/
		public String getExpirationTimeString(){
			
			SimpleDateFormat sdf = new SimpleDateFormat();
			return sdf.format(this.m_EventData.getEventExpirationTime().getTime());
		}
		
	/**	Get event expression. @return String set to representation of event payload parameter names and
	 * 	values in format suitable for use as event expression search criteria										*/
		public String getExpression(){
			
		//	Initialize return expression string
			String strExpression = "";
			
		//	Process payload parameters expressions
			Params parameters = this.m_EventData.getEventDataChoice_type0().getParams();
			
			if (parameters != null){
				
				Param[] aryParameters = parameters.getParam();
				
				if (aryParameters != null){

					for (int intParameter = 0; intParameter < aryParameters.length; intParameter++){						

						Param parameter = aryParameters[intParameter];
						String strName = "payload." + parameter.getName();
						String strValue = parameter.getString();
						strExpression += strName + "==" + strValue;
					}
				}			
			}
			else {
				
			//	Initialize payload string
				String strPayload = this.m_EventData.getEventDataChoice_type0().getPayload();
				if (!isNullEmpty(strPayload)) {strExpression = "payload==" + strPayload;}
			}
			
		//	Return vector
			return strExpression;
		}
			
		
	/** Get event name. @return String set to event name															*/
		public String getName(){
			
			return this.m_EventData.getEventName();
		}
		
	/** Get single subscriber flag. @return boolean set to true if event is intended to be completely consumer by
	 *  first subscriber otherwise false																			*/
		public boolean getSingleSubscriber(){
			
			return this.m_EventData.getToSingleSubscriber();
		}
		
	/** Get single subscriber flag as a string. @return String set to "true" if event is intended to be completely
	 * 	consumer by first subscriber otherwise false																*/
		public String getSingleSubscriberString(){
			
		//	Initialize return string
			String strSingleSubscriber = "false";
			
		//	Reset string based on event single subscriber property
			if (this.m_EventData.getToSingleSubscriber()){strSingleSubscriber = "true";}
			
		//	Return string
			return strSingleSubscriber;
		}
		
	/** Get event source. @return String set to event source														*/
		public String getSource(){
			
			return this.m_EventData.getEventSource();
		}
		
	/** Get event type. @return String set to event type															*/
		public String getType(){
			
			return this.m_EventData.getEventType();
		}
		
	/** Get even user. @return String set to user that created the event											*/
		public String getUser(){
			
			return this.m_EventData.getUser();
		}
	}

/** Abstract ITPAM interaction class extended by version specific classes											*/
	private class m_ItpamInteraction extends ItpamInteraction{
		
	/** Private instance of version specific UserInteractin object													*/
		private GetPendingInteractionRequestFormResponse m_Interaction = null;
		
	/** Private long to store interaction ID																		*/
		private long m_strInteracitonId;
		
	/**	Constructor																									*/
		private m_ItpamInteraction(long lInteractionId, GetPendingInteractionRequestFormResponse interaction){
			
			super();
			this.m_strInteracitonId = lInteractionId;
			this.m_Interaction = interaction;
		}
		
	/**	Get interaction description																					*/
		public String getDescription(){
			
			return this.m_Interaction.getDescription().getDescription();
		}
		
	/**	Get interaction ID																							*/
		public long getInteractionId(){
			
			return this.m_strInteracitonId;
		}
		
	/**	Get "is approval required (supported)" flag																							*/
		public boolean getIsApprovalRequired(){
			
			return this.m_Interaction.getIsApprovalRequired();
		}
		
	/**	Get interaction title																						*/
		public String getProcessId(){
			
			return Long.toString(this.m_Interaction.getProcessID());
		}
	}
	
/**	Private version specific concrete class extending public abstract class ItpamParameter							*/
	private class m_ItpamParameter extends ItpamParameter{
		
	/** Private String constant set to class name for supported ITPAM boolean parameter type 						*/
		private final String TYPE_ITPAM_BOOLEAN = "ItpamBoolean";
		
	/** Private String constant set to class name for supported ITPAM date parameter type							*/
		private final String TYPE_ITPAM_DATE = "ItpamDate";
		
	/** Private String constant set to class name for supported ITPAM double parameter type 						*/
		private final String TYPE_ITPAM_DOUBLE = "ItpamDouble";
		
	/** Private String constant set to class name for supported ITPAM int parameter type 							*/
		private final String TYPE_ITPAM_INT = "ItpamInt";
		
	/** Private String constant set to class name for supported ITPAM long parameter type 							*/
		private final String TYPE_ITPAM_LONG = "ItpamLong";
		
	/** Private String constant set to class name for supported ITPAM password parameter type 						*/
		private final String TYPE_ITPAM_PASSWORD = "ItpamValue";
		
	/** Private String constant set to class name for supported ITPAM string parameter type 						*/
		private final String TYPE_ITPAM_STRING = "ItpamString";
		
	/** Constructor																									*/
		private m_ItpamParameter(Object objParameter){
			
			super();
			
		//	Set properties
			this.setPropeties(objParameter);			
		}
		
	/** Constructor																									*/
		private m_ItpamParameter(Object objParameter, String strParentPageName){
			
			super();
			
		//	Set properties
			this.setPropeties(objParameter);
			
		//	Set parent page name property
			this.strParentPageName = strParentPageName;
		}
		
	/** Set class level property values based on parameter object type												*/
		private void setPropeties(Object objParameter){
			
			this.strDataType = objParameter.getClass().getSimpleName();
			
			if (strDataType != null){
			
				if (this.strDataType.equals(this.TYPE_ITPAM_BOOLEAN)) {
					
					ItpamBoolean parameter = (ItpamBoolean) objParameter;
					this.objDefaultValue = parameter.getDefaultValue();
					if (parameter.getDescription() != null){this.strDescription = parameter.getDescription().getDescription();					}
					if (parameter.getIsReadOnly() != null) {this.bolIsReadOnly = parameter.getIsReadOnly().getIsReadOnly();}
					if (this.strLabel != null){this.strLabel = parameter.getLabel();}
					if (parameter.getName() != null){this.strName = parameter.getName().getName();}
					if (parameter.getSequenceNo() != null){this.intSequenceNo = parameter.getSequenceNo().intValue();}
					this.objValue = parameter.getValue();
				}
				else if (this.strDataType.equals(this.TYPE_ITPAM_DATE)) {
					
					ItpamDate parameter = (ItpamDate) objParameter;
					if (parameter.getDefaultValue() != null) {this.objDefaultValue = parameter.getDefaultValue();}
					if (parameter.getDescription() != null){this.strDescription = parameter.getDescription().getDescription();					}
					if (parameter.getIsReadOnly() != null) {this.bolIsReadOnly = parameter.getIsReadOnly().getIsReadOnly();}
					if (this.strLabel != null){this.strLabel = parameter.getLabel();}
					if (parameter.getName() != null){this.strName = parameter.getName().getName();}
					if (parameter.getSequenceNo() != null){this.intSequenceNo = parameter.getSequenceNo().intValue();}
					if (parameter.getValue() != null) {this.objValue = parameter.getValue();}
				}
				else if (this.strDataType.equalsIgnoreCase(this.TYPE_ITPAM_DOUBLE)) {
					
					ItpamDouble parameter = (ItpamDouble) objParameter;
					this.objDefaultValue = parameter.getDefaultValue();
					if (parameter.getDescription() != null){this.strDescription = parameter.getDescription().getDescription();					}
					if (parameter.getIsReadOnly() != null) {this.bolIsReadOnly = parameter.getIsReadOnly().getIsReadOnly();}
					if (this.strLabel != null){this.strLabel = parameter.getLabel();}
					this.dblMaxValue = parameter.getMaxval();
					this.dblMinValue = parameter.getMinval();
					if (parameter.getName() != null){this.strName = parameter.getName().getName();}
					if (parameter.getSequenceNo() != null){this.intSequenceNo = parameter.getSequenceNo().intValue();}
					this.objValue = parameter.getValue();
				}
				else if (this.strDataType.equals(this.TYPE_ITPAM_INT)) {
					
					ItpamInt parameter = (ItpamInt) objParameter;
					if (parameter.getDefaultValue() != null) {this.objDefaultValue = parameter.getDefaultValue();}
					if (parameter.getDescription() != null){this.strDescription = parameter.getDescription().getDescription();					}
					if (parameter.getIsReadOnly() != null) {this.bolIsReadOnly = parameter.getIsReadOnly().getIsReadOnly();}
					if (this.strLabel != null){this.strLabel = parameter.getLabel();}
					this.dblMaxValue = parameter.getMaxval().doubleValue();
					this.dblMinValue = parameter.getMinval().doubleValue();
					if (parameter.getName() != null){this.strName = parameter.getName().getName();}
					if (parameter.getSequenceNo() != null){this.intSequenceNo = parameter.getSequenceNo().intValue();}
					if (parameter.getValue() != null){this.objValue = parameter.getValue();}
				}
				else if (this.strDataType.equals(this.TYPE_ITPAM_LONG)) {
					
					ItpamLong parameter = (ItpamLong) objParameter;
					this.objDefaultValue = parameter.getDefaultValue();
					if (parameter.getDescription() != null){this.strDescription = parameter.getDescription().getDescription();					}
					if (parameter.getIsReadOnly() != null) {this.bolIsReadOnly = parameter.getIsReadOnly().getIsReadOnly();}
					if (this.strLabel != null){this.strLabel = parameter.getLabel();}
					this.dblMaxValue = parameter.getMaxval();
					this.dblMinValue = parameter.getMinval();
					if (parameter.getName() != null){this.strName = parameter.getName().getName();}
					if (parameter.getSequenceNo() != null){this.intSequenceNo = parameter.getSequenceNo().intValue();}
					this.objValue = parameter.getValue();
				}
				else if (this.strDataType.equals(this.TYPE_ITPAM_PASSWORD)) {
					
					ItpamValue parameter = (ItpamValue) objParameter;
					if (parameter.getDescription() != null){this.strDescription = parameter.getDescription().getDescription();					}
					if (parameter.getIsReadOnly() != null) {this.bolIsReadOnly = parameter.getIsReadOnly().getIsReadOnly();}
					if (this.strLabel != null){this.strLabel = parameter.getLabel();}
					if (parameter.getName() != null){this.strName = parameter.getName().getName();}
					if (parameter.getSequenceNo() != null){this.intSequenceNo = parameter.getSequenceNo().intValue();}
				}
				else if (this.strDataType.equals(this.TYPE_ITPAM_STRING)) {
					
					ItpamString parameter = (ItpamString) objParameter;
					if (parameter.getDefaultValue() != null) {this.objDefaultValue = parameter.getDefaultValue();}
					if (parameter.getDescription() != null){this.strDescription = parameter.getDescription().getDescription();					}
					if (parameter.getIsReadOnly() != null) {this.bolIsReadOnly = parameter.getIsReadOnly().getIsReadOnly();}
					if (parameter.getLabel() != null){this.strLabel = parameter.getLabel();}
					if (parameter.getMask() != null){strMask = parameter.getMask();}
					if (parameter.getMaxlength() != null){this.lngMaxLength = parameter.getMaxlength().longValue();}
					if (parameter.getMinlength() != null){this.lngMinLength = parameter.getMinlength().longValue();}
					if (parameter.getName() != null){this.strName = parameter.getName().getName();}
					if (parameter.getSequenceNo() != null){this.intSequenceNo = parameter.getSequenceNo().intValue();}
				}
			}
		}
	}
	
/**	Private version specific concrete class extending public abstract class ItpamProcessLogEntry					*/
	private class m_ItpamProcessLogEntry extends ItpamProcessLogEntry{
		
	/**	Private instance of */
		private ProcessLog m_ProcessLog = null;
		
	/** Constructor																									*/
		private m_ItpamProcessLogEntry(ProcessLog processLog){
		
			super();
			this.m_ProcessLog = processLog;
		}
		
	/**	Get process log entry category. @return String set to process log entry category							*/
		public String getCategory(){
			
			return this.m_ProcessLog.getCategory();
		}
		
	/**	Get process log entry severity level.  @return String set to process log entry severity level				*/
		public String getLevel(){
			
			return this.m_ProcessLog.getLevel();
		}
		
	/** Get process log entry message.  @return String set to process log entry message								*/
		public String getMessage(){
			
			return this.m_ProcessLog.getLogMessage();
		}
		
	/**	Get process log entry time stamp.  @return String set to process log entry time stamp								*/
		public String getTimeStamp(){
			
			SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss:SSS");
			StringBuilder builder = new StringBuilder(formatter.format(this.m_ProcessLog.getTime().getTimeInMillis()));			
			return builder.toString();
		}
	}
	
/**	Private version specific concrete class extending public abstract class ItpamProceswsStatus						*/
	private class m_ItpamProcessStatus extends ItpamProcessStatus {
		
	/** Private String constant set to parameter/property name for "InstanceName"									*/
		private final String PARAM_NAME_DISPLAY_NAME = "DisplayName";
		
	/** Private String constant set to parameter/property name for "InstanceName"									*/
		private final String PARAM_NAME_INSTANCE_NAME = "InstanceName";
		
	/** Private String constant set to parameter/property name for "InstanceName"									*/
		private final String PARAM_NAME_PROCESS_NAME = "ProcessName";
		
	/**	Private instance of CheckFlowResult object																	*/
		private CheckFlowResult m_result = null;
		
	/**	Private instance of parameters Properties object															*/
		private Properties m_propertiesParameters = null;
		
	/**	Constructor @param result - CheckFlowResult object															*/
		public m_ItpamProcessStatus (CheckFlowResult result){
			
			super();
			this.m_result = result;
		}
		
	/** Get process connection ID. @return long set to process connection ID										*/
		public String getConnectionId(){
			
			return this.m_result.getConnectionId();
		}
		
	/**	Get process message. @return String set to process message													*/
		public String getMessage(){
			
			return this.m_result.getMessage();
		}
		
	/**	Get process status message. @return String set process status message										*/
		public String getMessageStatus(){
			
			return this.m_result.getMessageStatus();
		}
		
	/** Get process display name. @return String set to process display name										*/
		public String getProcessDisplayName(){
			
		//	Get parameters properties object
			Properties properties = this.getParameters();
			
		//	Return "InstanceName" property value
			return properties.getProperty(this.PARAM_NAME_DISPLAY_NAME);
		}
		
	/**	Get process ID. @return long set to process ID																*/
		public String getProcessId(){
			
			return this.m_result.getROID();
		}
		
	/** Get process instance specific name. @return String set to process instance specific name					*/
		public String getProcessInstanceName(){
			
		//	Get parameters properties object
			Properties properties = this.getParameters();
			
		//	Return "InstanceName" property value
			return properties.getProperty(PARAM_NAME_INSTANCE_NAME);
		}
			
	/**	Get fully qualified process library path and name. @return string set to fully qualified process library
	 * and name																										*/
		public String getProcessLibraryPathName(){
			
		//	Get parameters properties object
			Properties properties = this.getParameters();
			
		//	Return "InstanceName" property value
			return properties.getProperty(this.PARAM_NAME_PROCESS_NAME);
		}

	/** Get process state. @return String set to process state														*/
		public String getProcessState(){
			
			return this.m_result.getFlowState();
		}
		
	/**	Get process sender address. @return String set to process sender address									*/
		public String getSenderAddress(){
			
			return this.m_result.getSenderAddress();
		}
		
	/**	Get process submission time. @return String set to process submission time									*/
		public String getSubmissionTime(){
			
			return this.m_result.getSubmissionTime();
		}
		
	/**	Get process parameters. @return Properties object encapsulating process parameter names and values			*/
		public Properties getParameters(){
		
		//	Initialize properties if not yet initialized
			if (this.m_propertiesParameters == null){
				
			//	Initialize return object
				this.m_propertiesParameters = new Properties();
				
			//	Initialize parameters object
				Params parameters = this.m_result.getParams();
				
			//	Iterate parameters and set properties
				if (parameters != null){
					
					if (parameters.getParam() != null) {
						
						for (int intParameter = 0; intParameter < parameters.getParam().length; intParameter++){
							
							String strParameterName = parameters.getParam()[intParameter].getName();
							String strParameterValue = parameters.getParam()[intParameter].getString();
							this.m_propertiesParameters.setProperty(strParameterName, strParameterValue);
						}
					}
				}
			}			
			
		//	Return object
			return this.m_propertiesParameters;
		}
	};

/**	Private version specific concrete class extending public abstract class ItpamStartRequestForm					*/
	private class m_ItpamStartRequestForm extends ItpamStartRequestForm{
		
	/** Private instance of version specific StartRequestForm object												*/
		private StartRequest m_StartRequest = null;
		
	/** Constructor																									*/
		private m_ItpamStartRequestForm(StartRequest startRequest){
			
			super();
			this.m_StartRequest = startRequest;
		}
		
	/**	Get start request form description.  @return String set to start request form description					*/
		public String getDescripition(){
			
			return this.m_StartRequest.getDescription().getDescription();
		}
		
	/**	Get start request form name.  @return String set to start request form name									*/
		public String getName(){
			
			return this.m_StartRequest.getName().getName();
		}
		
	/**	Get start request form path.  @return String set to start request form path									*/
		public String getPath(){

			return this.m_StartRequest.getRefPath().getRefPath();
		}
	}
	
/**	Private version specific concrete class extending public abstract class ItpamStartRequestStatus					*/
	private class m_ItpamStartRequestStatus extends ItpamStartRequestStatus {
		
	/** Private version specific StartRequestStatus object reference												*/
		private StartRequestStatus m_status = null;
		
	/**	Constructor @param status - Version specific StartRequestStatus object										*/
		public m_ItpamStartRequestStatus(StartRequestStatus status){
			
			super();
			this.m_status = status;
		};

	/** Get start request status information @return String set to request earliest start date and time				*/
		public String getEarliestStartTime() {
			
			return this.m_status.getEarliestStartTime().getTime().toString();
		}
		
	/**	Get start request status information @return String set to request end date and time						*/
		public String getEndTime() {
			
			String strEndTime = null;
			
			if (this.m_status.getEndTime() != null) {
				
				strEndTime = this.m_status.getEndTime().getTime().toString();
			}
			
			return strEndTime;
		}
		
	/**	Get start request status information @return String set to request interaction ID value						*/
		public String getInteractionId() {
			
			return Long.toString(this.m_status.getInteractionId());
		}
		
	/**	Get start request status information @return String set to corresponding process ID value					*/
		public String getProcessId(){
			
			return Long.toString(this.m_status.getProcessID());
			
		}
		
	/**	Get start request status information @return String set to referenced process fully qualified name			*/
		public String getReferencedProcess() {
			
			return this.m_status.getRefProcess();
		}
		
	/**	Get start request status information @return String set to request start date and time						*/
		public String getStartTime() {
			
			return this.m_status.getStartTime().getTime().toString();
		}
		
	/**	Get start request status information @return String set to request state (ex. "Running")					*/
		public String getState(){
			
			return this.m_status.getState();
		}
	};
		
/**	Private version specific authentication object																	*/
	protected Auth m_auth = null;
	
/**	Private version specific web service stub object																*/
	protected ItpamServiceStub m_stubItpamService = null;
	
/**	Private version specific user authentication object																*/
	protected UserAuth m_userAuth;	
	

/**
 * Constructor to instantiate version specific client
 * 
 * @param strTargetEndpointVersion - String set to ITPAM version * 
 * @param strTargetEndpointURI - String set to ITPAM SOAP service URI
 * @param strUser - String set to authorized ITPM user name
 * @param strPassword - String set to ITPAM user password
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public ItpamWebServiceClient_3_0(String strTargetEndpointVersion,
									 String strTargetEndpointURI,
			 						 String strUser,
			 						 String strPassword)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{

		super(strTargetEndpointVersion, strTargetEndpointURI, strUser, strPassword);
		
		this.initCommonObjects();
	}
	
	/**
	 * Get current status of the ITPAM web service server.
	 * 
	 * @return String set to server status
	 * 
	 * @throws ItpamMethodNotSupportedException
	 * @throws ItpamParameterMissingOrInvalidException
	 * @throws ItpamStringToDateException
	 * @throws ItpamEntityNotFoundException
	 * @throws ItpamUserPermissionsException
	 * @throws ItpamUnknownHostException
	 * @throws ItpamWebServiceClientException
	 */
		public String getITPamVersionInfo()
		throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	    {
			
			String strServerVersion = null;

			try {
				
				GetITPamVersion request = new GetITPamVersion();		
				request.setAuth(this.m_auth);
				
				ITPamVersionDetails response = this.m_stubItpamService.getITPamVersionInfo(request);
				
				if(response.getMajorVersion() != null && response.getMajorVersion().length() > 0) {
					strServerVersion = response.getMajorVersion();
				}
				
				if(response.getMinorVersion() != null && response.getMinorVersion().length() > 0) {
					strServerVersion = strServerVersion + "." + response.getMinorVersion();
				}
			} 
			catch (Exception e) {

				try {
				//	Initialize exception messages
					String strMessage = this._messages.get("E_METHOD_GET_VERSION_INFO");
					String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_VERSION_INFO");
					
				//	Get type specific exception
					e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
				}
				catch (Exception eUnexpected){
					
					e = eUnexpected;
				}
				
			//	Record exception
				ItpamWebServiceClient.logger.error(e.getMessage());
				
			//	Throw specific exception type
				throw(e);
			}	
			
			return strServerVersion;
		}
	
	
/**
 * Abort process specified by process ID.
 * 
 * @param lProcessId - String set to process ID
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void abortProcess(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Suspend process ID " + lProcessId);
	
	//	Initialize action object
		Action action = Action.abort;
		
	//	Invoke process control method
		this.controlProcess(lProcessId, action);
	}

	
/**
 * Approve pending interaction request.
 * 
 * @param lInteractionId - long pending interaction ID value
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void approvePendingInteraction(long lInteractionId, Properties propertiesParameters)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Approve pending interaction ID " + lInteractionId);

	//	Initialize boolean flag
		boolean bolApprove = true;
		
	//	Invoke process control method
		this.executePendingInteraction(lInteractionId, bolApprove, propertiesParameters);
	}

	
/**
 * Delete archived instances in date range specified.
 * 
 * @param strStartDate - String set to start date of date range
 * @param strEndDate - String set to end date of date rangef
 * 
 * <br>
 * <br>Supported date formats 'yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy'
 * <br>Supported time formats 'hh:mm:ss', 'hh:mm'
 * <br>
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void deleteArchivedInstances(String strStartDate, String strEndDate)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
		
		try {
			
		//	Initialize filter object
			DateRange dateRange = getInstanceOfDateRange(strStartDate, strEndDate);
			
		//	Initialize request object
			DeleteArchivedInstances request = new DeleteArchivedInstances();
			request.setAuth(this.m_auth);
			request.setDateRange(dateRange);
			
		//	Initiate web service request
			DeleteArchivedInstancesResponse response = this.m_stubItpamService.deleteArchivedInstances(request);
			System.out.println(response.getSuccessMessage());
				
		//	Record success
			String strMessage = "Successfully completed request to delete ";
			strMessage = strMessage + response.getSuccessMessage().split(" ")[0];
			strMessage = strMessage + " archived instances ";
			if (!this.isNullEmpty(strStartDate)) strMessage = strMessage + " start date " + strStartDate;
			if (!this.isNullEmpty(strEndDate)) strMessage = strMessage + " end date " + strEndDate;			
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_DELETE_ARCHIVED_INSTANCES")
								  + " (" + strStartDate + ", " + strEndDate + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_DELETE_ARCHIVED_INSTANCES")
								  + " (" + strStartDate + ", " + strEndDate + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
	}
	
/**
 * Delete specified attachment.
 * 
 * @param lAttachmentId - long attachment ID value to be deleted
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void deleteAttachment(long lAttachmentId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
			
	//	Record progress
		ItpamWebServiceClient.logger.debug("Delete attachment ID " + lAttachmentId);

	//	Initialize single element array
		long[] aryAttachmentIds = new long[1];
		aryAttachmentIds[0] = lAttachmentId;

	//	Invoke method to delete attachments and return success message
		this.deleteAttachments(aryAttachmentIds);
		
	//	Record progress
		ItpamWebServiceClient.logger.debug("Deleted attachment ID " + lAttachmentId);
	}


/**
 * Delete specified attachments.
 * 
 * @param arayAttachmentIds - Array of long attachment ID values to be deleted
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void deleteAttachments(long[] aryAttachmentIds)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Delete attachments");
		
		try {
			
		//	Initialize attachment IDs object
			AttachmentIds attachmentIds = this.getInstanceOfAttachmentIds(aryAttachmentIds);
			
		//	Initialize request object
			DeleteAttachments request = new DeleteAttachments();
			request.setAttachments(attachmentIds);
			request.setAuth(this.m_auth);
			
		//	Initiate web service request
			DeleteAttachmentsResponse response = this.m_stubItpamService.deleteAttachments(request);

		//	Record progress
			ItpamWebServiceClient.logger.info("Successfully completed request to delete attachments (" + response.getSuccessMessage() + ")");
		}
		catch (Exception e){

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_DELETE_ATTACHMENTS");
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_DELETE_ATTACHMENTS");
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
		
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);		
		}
	}
	
	
/**
 * Export object definition(s) to XML file.
 * 
 * @param strSourcePath - String set to source library path for object definition(s) to be exported
 * @param intLevelsToSearch - Integer set to levels to search for object definition(s) to be exported
 * @param bolIsAbsoluteSourcePath - Boolean set to true if source path references in the exported object
 * 		definition(s) should be absolute or false to use relative references
 * @param strTargetPath - String set to target path for exported definition(s)
 * @param strTargetFileName - String set to target file name for exported definition(s)
 * @param bolTargetPathIsLocal - Boolean set to true if target path/file name is local
 * @param bolOverwriteTargetFile - Boolean set to true to overwrite target file if it exists
 * @param aryObjectTypeFilterStrings - Array of strings set to object types of objects in the source path to
 * 		be exported
 * <br><br>	
 * 		Supported object type filter strings:
 * <br>	- Agenda
 * <br>	- Calendar
 * <br>	- Commander
 * <br>	- CustomIcon
 * <br>	- CustomOperator
 * <br>	- CustomSensor
 * <br>	- Dataset
 * <br>	- InteractionRequestForm
 * <br>	- LogViewer
 * <br>	- Package
 * <br>	- Process
 * <br>	- ProcessWatch
 * <br>	- Resources
 * <br>	- StatePolicy
 * <br>	- System
 * <br>
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void exportObject(String strSourcePath, int intLevelsToSearch, boolean bolIsAbsoluteSourcePath, 
							 String strTargetPath, String strTargetFileName, boolean bolTargetPathIsLocal,
							 boolean bolOverwriteTargetFile, String[] aryObjectTypeFilterStrings)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
		
		try {
			
		//	Validate required source path argument
			if ((this.isNullEmpty(strTargetPath)) || (this.isNullEmpty(strTargetFileName))){
				
			//	Set exception messages
				String strMessage = this._messages.get("E_MISSING_OR_INVALID_SOURCE_PATH");
				String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_SOURCE_PATH");
				
			//	Record exception
				ItpamWebServiceClient.logger.error(strMessage);
				
			//	Throw exception
				throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
			}
			else {
				
			//	Initialize request object
				ExportObject request = new ExportObject();
				request.setAuth(this.m_auth);
				request.setFolderName(strSourcePath);
				request.setIsAbsolute(bolIsAbsoluteSourcePath);
				request.setLevel(intLevelsToSearch);
				
			//	Verify existence of or create target path directory structure if local target path specified
			//	otherwise instantiate and pass down load file name object with request
				if (bolTargetPathIsLocal) {
					
				//	Append end delimiter to path if missing
					if (!strTargetPath.endsWith(System.getProperty("file.separator"))) {strTargetPath += System.getProperty("file.separator");}
					
				//	Verify existence of or create target path directory structure
					try {
						File filePath = new File(strTargetPath);
						if (!filePath.exists()){filePath.mkdirs();}
					}
					catch (Exception e){
						
					//	Set exception messages
						String strMessage = this._messages.get("E_MISSING_OR_INVALID_TARGET_PATH");
						String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_TARGET_PATH");
						
					//	Record exception
						ItpamWebServiceClient.logger.error(strMessage);
						
					//	Throw exception
						throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
					}
				}
				else {
					
					DownloadFileName downloadFileName = this.getInstanceOfDownloadFileName(strTargetPath, strTargetFileName, bolOverwriteTargetFile);
					request.setDownloadFileName(downloadFileName);
				}
				
			//	Set object type filter if specified
				ExportLibFilter filter = this.getInstanceOfExportLibFilter(aryObjectTypeFilterStrings);
				if (filter != null) {request.setFilter(filter);}
					
			//	Initiate web service request
				ExportLibraryResponse response = this.m_stubItpamService.exportObject(request);
				
			//	Attempt to write file to local file system if target path was specified as local
				if (bolTargetPathIsLocal){
					
					MessageContext msgCxt =   this.m_stubItpamService._getServiceClient().getLastOperationContext().getMessageContext("In");
		            try {
		            	
		            	DataHandler handler=  msgCxt.getAttachment(response.getExportLibObject().getSwaRef().getPath());
		            	handler.writeTo(new FileOutputStream(new File(strTargetPath + strTargetFileName)));
		            }
		            catch (IOException e) {

		            //	Throw exception to be caught by generic catch block below and processed
		            	throw e;
		            }
		            finally {
		            	msgCxt.getTransportOut().getSender().cleanup(msgCxt);
		            }
				}
				
			//	Record success
				String strMessage = "Successfully completed request to export object(s) from '" + strSourcePath + "' "
								  + "to '" + strTargetPath + "' file '" + strTargetFileName + "'";
				if (bolTargetPathIsLocal){
					
					strMessage += " locally";
				}
				else {
					
					strMessage += " on server";
				}
				ItpamWebServiceClient.logger.info(strMessage);
			}			
		}
		catch (Exception e) {
			
			e.printStackTrace();
	
			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_EXPORT_OBJECT")
								  + " (" + strSourcePath + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_EXPORT_OBJECT")
				  				  + " (" + strSourcePath + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
	}
	
	
/**
 * Generate event specified (3.0 and higher).
 * 
 * @param strEventName - Set set to an event name that will recognized by the process operator(s) waiting for the event 
 * @param strEventType - String set to an event type that will recognized by the process operator(s) waiting for the event
 * 	or null
 * @param strEventDestination - String set to an event destination that will recognized by the process operator(s) waiting
 *	for the event or null
 * @param strEventSource - String set to an event source that will recognized by the process operator(s) waiting
 *	for the event or null
 * @param lEventExpirationDuration - long set to time in seconds before the event expires
 * @param bolEventSingleSubscriber - boolean value set to true if one and only one process operator may consume the event
 *  (i.e., the first operator waiting for a matching event will "consume" the event - it will not be visible to other
 *  waiting process operators waiting for events)
 * @param strEventPayload - String set to content that will be used by the waiting process(es) or null
 * @param propertiesParameters - Properties object encapsulating name/value pairs of parameter names and corresponding
 *  values to be used by the waiting process(es) or null
 * <br><br>
 * <b>NOTE:</b> Event pay load will be superseded (ignored) if parameters are specified.
 * <br>
 * 
 * @return ItpamEventData object for the event generated
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public ItpamEventData generateEvent(String strEventName, String strEventType, String strEventDestination,
			 							String strEventSource, long lEventExpirationDuration, boolean bolEventSingleSubscriber,
			 							String strEventPayload, Properties propertiesParameters)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
    {
	//	Initialize return event data object
		ItpamEventData eventData = null;
		
		try {		

		//	Initialize parameters object
			Params parameters = this.getInstanceOfParameters(propertiesParameters);
			
		//	Initialize event choice object (Note: Parameters will supersede pay load)
			GenerateEventChoice_type0 eventChoice = new GenerateEventChoice_type0();
			if (parameters == null){
				eventChoice.setPayload(strEventPayload);
			}
			else {
				eventChoice.setParams(parameters);
			}
			
		//	Initialize request object
			GenerateEvent request = new GenerateEvent();
			request.setAuth(this.m_auth);
			request.setEventDestination(strEventDestination);
			request.setEventExpDuration(lEventExpirationDuration);
			request.setEventName(strEventName);
			request.setEventSource(strEventSource);
			request.setEventType(strEventType);
			request.setGenerateEventChoice_type0(eventChoice);
			request.setToSingleSubscriber(bolEventSingleSubscriber);
			
		//	Initiate web service request
			GenerateEventResponse response = this.m_stubItpamService.generateEvent(request);
			
		//	Reset return status message
			eventData = new m_ItpamEventData(response.getEvent());
			
		//	Record success
			String strMessage = "Successfully completed request to generated event '" + strEventName + "'";
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GENERATE_EVENT")
								  + " (" + strEventName + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GENERATE_EVENT")
				  				  + " (" + strEventName + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return event status message string
		return eventData;
	}
	
	
/**
 * Get existing attachments.
 * 
 * @param strContentId - String set to content ID or null
 * @param strContentType - String set to content Type or null
 * 
 * @return Vector of ItpamAttachmentInfo objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public Vector<ItpamAttachment> getAttachments(String strContentId, String strContentType)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Initialize return vector
		Vector<ItpamAttachment> vAttachments = new Vector<ItpamAttachment>();
		
		try {
			
		//	Initialize filter object
			AttachmentFilter filter = getInstanceOfAttachmentFilter(strContentId, strContentType);
			
		//	Initialize request object
			GetAttachments request = new GetAttachments();
			request.setAuth(this.m_auth);
			request.setFilter(filter);
			
		//	Initiate web service request
			GetAttachmentsResponse response = this.m_stubItpamService.getAttachments(request);
			
		//	Initialize return object
			AttachmentsInfo attachmentsInfo = response.getAttachments();
			AttachmentInfo[] attachmentInfo = attachmentsInfo.getAttachment();
			
			if (attachmentInfo != null) {
					
			//	Iterate attachment info elements and add to vector
				for (int intAttachment = 0; intAttachment < attachmentInfo.length; intAttachment++){
					
					m_ItpamAttachment attachment = new m_ItpamAttachment(attachmentInfo[intAttachment]);
					vAttachments.add(attachment);
				}
				
			//	Record success
				String strMessage = "Successfully completed request to get attachments; " + attachmentInfo.length + " attachments found";
				if (!this.isNullEmpty(strContentId)) strMessage = strMessage + " content ID " + strContentId;
				if (!this.isNullEmpty(strContentType)) strMessage = strMessage + " content ID " + strContentType;			
				ItpamWebServiceClient.logger.info(strMessage);
			}
			else {
				
			//	Record success
				String strMessage = "Successfully completed request to get attachments; 0 attachments found";
				if (!this.isNullEmpty(strContentId)) strMessage = strMessage + " content ID " + strContentId;
				if (!this.isNullEmpty(strContentType)) strMessage = strMessage + " content ID " + strContentType;			
				ItpamWebServiceClient.logger.info(strMessage);
			}
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_ATTACHMENTS")
								  + " (" + strContentId + ", " + strContentType + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_ATTACHMENTS")
				  				  + " (" + strContentId + ", " + strContentType + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
	
	//	Return vector
		return vAttachments;
	}
	
	
/**
 * Get event data for events matching specified criteria (3.0 and higher).
 * 
 * @param strEventName - String set to an event name that will recognized by the process operator(s) waiting for the event (required) 
 * @param strEventType - String set to an event type that will recognized by the process operator(s) waiting for the event or null
 * @param strEventDestination - String set to an event destination that will recognized by the process operator(s) waiting
 *	for the event or null
 * @param strEventSource - String set to an event source that will recognized by the process operator(s) waiting
 *	for the event or null
 * @param strEventExpression - String set to an expression that will be compared to event "payload" parameter(s)
 * 	(ex. payload.paramName=="paramValue") or null
 * @param bolAllowPartialMatch - boolean value set to true to return data for events that partially match the specified
 * 	criteria
 * @param bolEnablePatternMatch - boolean value set to true if criteria specified includes regex-type patterns
 * <br>
 * 
 * @return Vector of ItpamEventData objects for events matching specified criteria
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public Vector<ItpamEventData> getMatchingEvents(String strEventName, String strEventType, String strEventDestination,
								  String strEventSource, String strEventExpression, boolean bolAllowPartialMatch,
								  boolean bolEnablePatternMatch)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Initialize return string
		Vector<ItpamEventData> vEventData = new Vector<ItpamEventData>();
		
	//	Initialize criteria string used in messages
		String strCriteria = "(";
		if (!this.isNullEmpty(strEventName)) {
			strCriteria += "Name=" + strEventName;
		}
		if (!this.isNullEmpty(strEventType)) {
			
			if (!strCriteria.endsWith("(")) {strCriteria += ", ";}
			strCriteria += "Type=" + strEventType;
		}
		if (!this.isNullEmpty(strEventDestination)) {
			
			if (!strCriteria.endsWith("(")) {strCriteria += ", ";}
			strCriteria += "Destination=" + strEventDestination;
		}
		if (!this.isNullEmpty(strEventSource)) {
			
			if (!strCriteria.endsWith("(")) {strCriteria += ", ";}
			strCriteria += "Source=" + strEventSource;
		}
		if (!this.isNullEmpty(strEventExpression)) {
			
			if (!strCriteria.endsWith("(")) {strCriteria += ", ";}
			strCriteria += "Expression=" + strEventExpression;
		}		
		if (!strCriteria.endsWith("(")) {
			strCriteria += ", ";
		}
		if (bolAllowPartialMatch) {
			strCriteria += "AllowPartialMatch=True";
		}
		else {
			strCriteria += "AllowPartialMatch=True";
		}		
		if (!strCriteria.endsWith("(")) {
			strCriteria += ", ";
		}
		if (bolEnablePatternMatch) {
			strCriteria += "EnablePatternMatch=True";
		}
		else {
			strCriteria += "EnablePatternMatch=True";
		}
		strCriteria += ")";
		
		try {	
		
			GetMatchingEvents request = new GetMatchingEvents();
			request.setAuth(this.m_auth);
			request.setAllowPartialMatch(bolAllowPartialMatch);
			request.setEnablePatternMatch(bolEnablePatternMatch);
			request.setEventDestination(strEventDestination);
			request.setEventExpression(strEventExpression);
			request.setEventName(strEventName);
			request.setEventSource(strEventSource);
			request.setEventType(strEventType);
			
			GetMatchingEventsResponse response = this.m_stubItpamService.getMatchingEvents(request);
			
		//	Process matching events
			MatchEvents[] aryMatchingEvents = response.getEvents();
			
			if (aryMatchingEvents != null) {
					
			//	Iterate matching events elements
				for (int intEvent = 0; intEvent < aryMatchingEvents.length; intEvent++){
					
				//	Get event data objects
					EventData[] aryEventDataObjects = aryMatchingEvents[intEvent].getEvent();
					
				//	Iterate event data objects
					if (aryEventDataObjects != null){
						
						for (int intObject = 0; intObject < aryEventDataObjects.length; intObject++){
							
						//	Create and add event data object to return vector
							m_ItpamEventData eventData = new m_ItpamEventData(aryEventDataObjects[intObject]);
							vEventData.add(eventData);
						}
					}
					
				}
			}
				
		//	Record success
			String strMessage = "Successfully completed request to get matching events " + strCriteria
							  + "; " + vEventData.size() + " attachments found";		
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {
	
			try {
			//	Initialize exception messages				  
				String strMessage = this._messages.get("E_METHOD_GET_MATCHING_EVENTS") 
								  + " " + strCriteria;
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_MATCHING_EVENTS")
				  				  + " " + strCriteria;
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return event status message string
		return vEventData;
	}
	
	
/**
 * Get information related to pending user interaction specified.
 * 
 * @param lInteractionId - long pending user interaction request ID value
 * 
 * @return ItpamInteraction object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public ItpamInteraction getPendingInteraction(long lInteractionId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Return progress
		ItpamWebServiceClient.logger.debug("Get pending interaction ID '" + lInteractionId + "'");
		
	//	Initialize return value
		ItpamInteraction interaction = null;
		
		try {
		
		//	Initialize user interaction ID object
			UserInteractionID id = new UserInteractionID();
			id.setUserInteractionID(lInteractionId);
			
		//	Initialize request object
			GetPendingInteractionRequestForm request = new GetPendingInteractionRequestForm();
			request.setAuth(this.m_auth);
			request.setUserInteractionID(id);
			
		//	Initiate web service request
			GetPendingInteractionRequestFormResponse response = this.m_stubItpamService.getPendingInteractionRequestForm(request);
			
		//	Update return object
			interaction = new m_ItpamInteraction(lInteractionId, response);
			
		//	Record progress
			String strMessage = "Successfully completed request to get pending interaction request form for interation ID " + lInteractionId;
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_PENDING_INTERACTION")
								  + " (" + lInteractionId + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_PENDING_INTERACTION")
				  				  + " (" + lInteractionId + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return value
		return interaction;
	}
	
	
/**
 * Get parameter information for pending user interaction specified.
 * 
 * @param strInteractionId - long pending user interaction request ID value
 * 
 * @return Vector of ItpamParameter objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public Vector<ItpamParameter> getPendingInteractionParameters(long lInteractionId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamUnsupportedParameterTypeException,
    	   ItpamWebServiceClientException
	{
	//	Return progress
		ItpamWebServiceClient.logger.debug("Get pending interaction ID '" + lInteractionId + "'");
		
	//	Initialize return value
		Vector<ItpamParameter> vParameters = new Vector<ItpamParameter>();
		
		try {
		
		//	Initialize user interaction ID object
			UserInteractionID id = new UserInteractionID();
			id.setUserInteractionID(lInteractionId);
			
		//	Initialize request object
			GetPendingInteractionRequestForm request = new GetPendingInteractionRequestForm();
			request.setAuth(this.m_auth);
			request.setUserInteractionID(id);
			
		//	Initiate web service request
			GetPendingInteractionRequestFormResponse response = this.m_stubItpamService.getPendingInteractionRequestForm(request);
			
		//	Process parameter pages
			vParameters = this.getParametersFromPages(response.getPages());
			
		//	Record progress
			String strMessage = "Successfully completed request to get pending interaction parameters for interation ID " + lInteractionId;
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_PENDING_INTERACTION_PARAMETERS")
								  + " (" + lInteractionId + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_PENDING_INTERACTION_PARAMETERS")
				  				  + " (" + lInteractionId + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return value
		return vParameters;
	}


/**
 * Get pending interaction information.
 * 
 * @return Vector of ItpamInteraction objects.
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public Vector<ItpamInteraction> getPendingInteractions()
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Return progress
		ItpamWebServiceClient.logger.debug("Get pending interactions");
		
	//	Initialize return value
		Vector<ItpamInteraction> vPendingInteracitons = new Vector<ItpamInteraction>();
		
		try {
			
		//	Initialize request object
			GetPendingUserInteractions request = new GetPendingUserInteractions();
			request.setAuth(this.m_auth);
			
		//	Initiate web service request
			GetPendingUserInteractionsResponse response = this.m_stubItpamService.getPendingUserInteractions(request);
			
		//	Update return vector
			if (response.getUserInteractions()!= null){

				//	Initialize array of process log entries
					UserInteraction[] aryUserInteractions = response.getUserInteractions().getUserInteraction();
					
				//	Determine if any user interactions found
					if (aryUserInteractions != null) {
					
					//	Iterate process log entries
						for (int intInteraction = 0; intInteraction < aryUserInteractions.length; intInteraction++){
							
						//	Get interaction ID
							long lInteractionId = aryUserInteractions[intInteraction].getId().getId();
							
						//	Get interaction object
							ItpamInteraction interaction = this.getPendingInteraction(lInteractionId);
							
						//	Add ITPAM process log entry object to return vector
							vPendingInteracitons.add(interaction);	
						}
					}
				}
			
		//	Record success
			String strMessage = "Successfully completed request to get pending interactions";
			strMessage = strMessage + "; " + vPendingInteracitons.size() + " entries found";
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_PENDING_INTERACTIONS");
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_PENDING_INTERACTIONS");
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return vector
		return vPendingInteracitons;
	}


/**
 * Get process logs.
 * 
 * @param lProcessId - long set to process ID
 * @param strLevel - String set to minimum severity level to be returned
 * 		  (Valid Levels: Level: error ,warning, notice, and normal; If the user specifies an invalid level, it will be treated as normal (the lowest level)
 * @param aryCategores - Array of Strings set to log categories
 *        (i.e. Operator, Process, userMsg; ignored if invalid string specified)
 * 
 * @return Vector of ItpamProcessLogEntry objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public Vector<ItpamProcessLogEntry> getProcessLogs(long lProcessId, String strLevel, String[] aryCategories)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
    {			
	//	Record progress
		ItpamWebServiceClient.logger.debug("Get process logs for process ID " + lProcessId);		
		
	//	Instantiate return object
		Vector<ItpamProcessLogEntry> vProcessLogEntries = new Vector<ItpamProcessLogEntry>();
		
		try {
			
		//	Initialize Categories object
			Categories categories = new Categories();
			if (aryCategories != null) categories.setCategory(aryCategories);
			
		//	Initialize filter object
			LogFilter filter = new LogFilter();
			if (aryCategories != null) filter.setCategories(categories);
			if (!this.isNullEmpty(strLevel)) filter.setLevel(strLevel);
			
		//	Initialize request object
			GetProcessLogs request = new GetProcessLogs();
			request.setAuth(this.m_auth);
			request.setFilter(filter);
			request.setProcessID(lProcessId);
			
		//	Initiate web service request
			GetProcessLogsResponse response = this.m_stubItpamService.getProcessLogs(request);
			
		//	Update return vector
			if (response.getProcessLogs().getProcesslog() != null){

			//	Initialize array of process log entries
				ProcessLog[] aryProcessLogEntries = response.getProcessLogs().getProcesslog();
				
			//	Iterate process log entries
				for (int intLogEntry = 0; intLogEntry < aryProcessLogEntries.length; intLogEntry++){
					
				//	Add ITPAM process log entry object to return vector
					ItpamProcessLogEntry processLogEntry = new m_ItpamProcessLogEntry(aryProcessLogEntries[intLogEntry]);
					vProcessLogEntries.add(processLogEntry);			
				}
			}			
			
		//	Record success
			String strMessage = "Successfully completed request to get process logs for process ID " + lProcessId;
			strMessage = strMessage + "; " + vProcessLogEntries.size() + " entries found";
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_PROCESS_LOGS")
				                  + " (" + lProcessId + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_PROCESS_LOGS")
                				  + " (" + lProcessId + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return object
		return vProcessLogEntries;
    }
   


/**
 * Get list of pending interactions related to the process ID specified.
 * 
 * @param lProcessId - long set to process ID
 * 
 * @return Vector of ItpamInteraction objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public Vector<ItpamInteraction> getProcessPendingInteractions(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Return progress
		ItpamWebServiceClient.logger.debug("Get pending interactions for process ID '" + lProcessId + "'");
		
	//	Initialize return vector
		Vector<ItpamInteraction> vProcessPendingInteractions = new Vector<ItpamInteraction>();
		
	//	Get pending interactions
		Vector<ItpamInteraction> vPendingInteractions = this.getPendingInteractions();
		
	//	Iterate pending interactions and add interactions referencing the specified process ID to the return vector
		for (Iterator<ItpamInteraction> iInteractions = vPendingInteractions.iterator(); iInteractions.hasNext();){
	
			ItpamInteraction interaction = iInteractions.next();			
			if (interaction.getProcessId().equals(Long.toString(lProcessId))) vProcessPendingInteractions.add(interaction);
		}
		
	//	Record success
		String strMessage = "Successfully completed request to get pending interactions for process ID '" + lProcessId + "'";
		strMessage = strMessage + "; " + vPendingInteractions.size() + " entries found";
		ItpamWebServiceClient.logger.info(strMessage);
		
	//	Return vector
		return vProcessPendingInteractions;
	}


/**
 * Get current status of process specified.
 * 
 * @param lProcessId - long set to process ID
 * 
 * @return
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public ItpamProcessStatus getProcessStatus(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
    {			
	//	Record progress
		ItpamWebServiceClient.logger.debug("Get status for process ID " + lProcessId);
		
	//	Instantiate return object
		ItpamProcessStatus status = null;
		
		try {

		//	Initialize flow object
			Execflow flow = new Execflow();
			flow.setAuth(this.m_auth);
			flow.setAction("Check");
			flow.setROID(Long.toString(lProcessId));
			
		//	Initialize request object
			GetProcessStatus request = new GetProcessStatus();
			request.setFlow(flow);
			
		//	Initiate web service request
			CheckFlowResponse response = this.m_stubItpamService.getProcessStatus(request);
			
		//	Initialize return object
			status = new m_ItpamProcessStatus(response.getCheckFlowResult());
			
		//	Record success
			String strMessage = "Successfully completed request to get status of process ID '" + lProcessId + "'";
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_PROCESS_STATUS")
				                  + " (" + lProcessId + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_PROCESS_STATUS")
                				  + " (" + lProcessId + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return object
		return status;
	}
	
	
/**
 * Get current status of the ITPAM web service server.
 * 
 * @return String set to server status
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public String getServerStatus()
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
    {
		
		String strServerStatus = null;

		try {
			
			CheckServerStatus request = new CheckServerStatus();		
			request.setAuth(this.m_auth);
		
			CheckServerStatusResponse response = this.m_stubItpamService.checkServerStatus(request);
			
			strServerStatus = response.getServerStatus();
		} 
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_PROCESS_STATUS");
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_PROCESS_STATUS");
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}	
		
		return strServerStatus;
	}
	
	
/**
 * Search for and return available start request form parameters.
 * 
 * @param aryKeywords - Array of Strings set to to keywords to used in search filter (Note: Ignored for version 2.1)
 * @param strSearchPath - String set to search path to be set as the root folder of the search (Note: Ignored for version 2.1)
 * @param bolRecursive - boolean set to true to search through child folders of the root folder specified (Note: Ignored for version 2.1)
 * 
 * @return Vector of ItpamStartRequestForm objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public Vector<ItpamStartRequestForm> getStartRequestForms(String[] aryKeywords, String strSearchPath, boolean bolRecursive)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Set search to root if not specified
		if (this.isNullEmpty(strSearchPath)) strSearchPath = "/";
		
	//	Record progress
		ItpamWebServiceClient.logger.debug("Get start request forms in '" + strSearchPath + "'");
		
	//	Initialize status object to be returned
		Vector<ItpamStartRequestForm> vStartRequestForms = new Vector<ItpamStartRequestForm>();
		
		try {
			
		//	Initialize keywords object
			Keywords keywords = this.getInstanceOfKeywords(aryKeywords);
			
		//	Initialize look up path object
			LookUpPath path = new LookUpPath();
			path.setIsRecursive(bolRecursive);
			path.setString(strSearchPath);
			
		//	Initialize filter object
			SearchFilter filter = new SearchFilter();
			filter.setKeywords(keywords);
			filter.setLookUpPath(path);
			
		//	Initialize request object
			GetStartRequestForms request = new GetStartRequestForms();
			request.setAuth(this.m_auth);
			request.setFilter(filter);
			
		//	Initiate web service request
			GetStartRequestFormsResponse response = this.m_stubItpamService.getStartRequestForms(request);
				
		//	Update return vector
			if (response.getStartRequests().getStartRequest() != null){

				//	Initialize array of process log entries
					StartRequest[] aryStartRequestForms = response.getStartRequests().getStartRequest();
					
				//	Iterate process log entries
					for (int intForm = 0; intForm < aryStartRequestForms.length; intForm++){
						
					//	Add ITPAM process log entry object to return vector
						ItpamStartRequestForm form = new m_ItpamStartRequestForm(aryStartRequestForms[intForm]);
						vStartRequestForms.add(form);			
					}
				}
			
		//	Record success
			String strMessage = "Successfully completed request to get start request forms in '" + strSearchPath + "'";
			if (bolRecursive) strMessage = strMessage + " searching recursively";
			if (aryKeywords != null){
				
				strMessage = strMessage + " with kewords";
				for (int intKeyword = 0; intKeyword < aryKeywords.length; intKeyword++){
					
					strMessage = strMessage + " '" + aryKeywords[intKeyword] + "'";
				}
			}
			strMessage = strMessage + "; " + vStartRequestForms.size() + " entries found";
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
				
			//	Set recursive string
				String strRecursive = "Non-Recursive";
				if (bolRecursive) strRecursive = "Recursive";
				
			//	Set keywords string
				String strKeywords = "Keywords: ";
				if (aryKeywords != null){
					
					for (int intKeyword = 0; intKeyword < aryKeywords.length; intKeyword++){
						
						strKeywords = strKeywords + " '" + strKeywords + "'";
					}
				}
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_START_REQUEST_FORMS")
								  + " (" + strSearchPath + ", " + strRecursive + ", " + strKeywords + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_START_REQUEST_FORMS")
				  				  + " (" + strSearchPath + ", " + strRecursive + ", " + strKeywords + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return vector
		return vStartRequestForms;
	}

	
/**
 * Get parameters for specified start request form.
 * 
 * @param strStartRequestName - String set to start request form name
 * @param strStartRequestPath - String set to start request form source library path
 * 
 * @return Vector of ItpamParameter objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamUnsupportedParameterTypeException
 * @throws ItpamWebServiceClientException
 */
	public Vector<ItpamParameter> getStartRequestParameters(String strStartRequestName, String strStartRequestPath)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamUnsupportedParameterTypeException,
    	   ItpamWebServiceClientException
	{
	//	Set search to root if not specified
		if (this.isNullEmpty(strStartRequestPath)) strStartRequestPath = "/";
		
	//	Record progress
		ItpamWebServiceClient.logger.debug("Get start request form '" + strStartRequestName + "' in '" + strStartRequestPath + "'");
		
	//	Initialize status object to be returned
		Vector<ItpamParameter> vParameters = new Vector<ItpamParameter>();
		
		try {
			
		//	Initialize object location object
			ObjLocation location = this.getInstanceOfLocation(strStartRequestName, strStartRequestPath);
			
		//	Initialize request object
			GetStartRequestForm request = new GetStartRequestForm();
			request.setAuth(this.m_auth);
			request.setGetParamSequence(true);
			request.setObjLocation(location);
			
		//	Initiate web service request
			GetStartRequestFormResponse response = this.m_stubItpamService.getStartRequestForm(request);
				
		//	Process response
			vParameters = this.getParametersFromPages(response.getPages());
			
		//	Record success
			String strMessage = "Successfully completed request to get start request form '" + strStartRequestName + "' in '" + strStartRequestPath + "'";
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {

			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_START_REQUEST_FORM_PARAMETERS")
								  + " (" + strStartRequestName + ", " + strStartRequestPath + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_START_REQUEST_FORM_PARAMETERS")
				  				  + " (" + strStartRequestName + ", " + strStartRequestPath + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return vector
		return vParameters;
	}


/**
 * Get process name referenced by the start request form specified.
 * 
 * @param strStartRequestName - String set to start request form name
 * @param strStartRequestPath - String set to start request form library path
 * 
 * @return String set to fully qualified process name referenced by the start request form specified
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public String getStartRequestProcessReferenced(String strStartRequestName, String strStartRequestPath)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{		
	//	Record progress
		ItpamWebServiceClient.logger.debug("Get start request form '" + strStartRequestName + "'");
		
	//	Initialize status object to be returned
		String strProcessName = null;
		
		try {
			
		//	Initialize location object
			ObjLocation location = this.getInstanceOfLocation(strStartRequestName, strStartRequestPath);
			
		//	Initialize request object
			GetStartRequestForm request = new GetStartRequestForm();
			request.setAuth(this.m_auth);
			request.setObjLocation(location);
			
		//	Initiate web service request
			GetStartRequestFormResponse response = this.m_stubItpamService.getStartRequestForm(request);
				
		//	Update return object
			strProcessName = response.getRefProcess();
			
		//	Record success
			String strMessage = "Successfully completed request to get process name referenced ('" + strProcessName + "')"
							  + " by start request form '" + this._strStartRequestName + "' in '" + this._strStartRequestPath;
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_START_REQUEST_PROCESS_REFERENCED")
								  + " (" + this._strStartRequestName + ", " + this._strStartRequestPath + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_START_REQUEST_PROCESS_REFERENCED")
				  				  + " (" + this._strStartRequestName + ", " + this._strStartRequestPath + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return vector
		return strProcessName;
	}


/**
 * Get start request status object that includes the following information:
 * <br>- Earliest Start Time
 * <br>- End Time
 * <br>- Interaction ID
 * <br>- Referenced Process
 * <br>- Start Time
 * <br>- State
 * 
 * @param 1StartRequestInteractionId - long start request interaction ID value
 * 
 * @return ItpamStartRequestStatus object that exposes methods to detailed status information
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public ItpamStartRequestStatus getStartRequestStatus(long lStartRequestInteractionId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Get status for start request interaction ID " + lStartRequestInteractionId);
		
	//	Initialize status object to be returned
		ItpamStartRequestStatus status = null;
		
		try {
		
		//	Initialize ID object
			Id id = new Id();
			id.setId(lStartRequestInteractionId);
			
		//	Initialize request object
			CheckStartRequestStatus request = new CheckStartRequestStatus();
			request.setAuth(this.m_auth);
			request.setInteractionId(id);
			
		//	Initiate web service request
			CheckStartRequestStatusResponse response = this.m_stubItpamService.checkStartRequestStatus(request);
				
		//	Update status object to be returned
			status = new m_ItpamStartRequestStatus(response.getStartRequestStatus());
			
		//	Record success
			String strMessage = "Successfully completed request to get status for start request interaction ID " + lStartRequestInteractionId;
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_GET_START_REQUEST_STATUS")
								  + " (" + lStartRequestInteractionId + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_GET_START_REQUEST_STATUS")
				  				  + " (" + lStartRequestInteractionId + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return status object
		return status;
	}
	
	
/**
 * Import ITPAM object defined in specified XML file.
 * 
 * @param strSourcePathFileName - String set to path and file name of file to be imported
 * @param strTargetLibraryPathName - String set to target library path and object name
 * @param bolMakeCurrent - boolean set to true if imported file should be flagged as the current version
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void importObject(String strSourcePathFileName, String strTargetLibraryPathName, boolean bolMakeAvailable, boolean bolMakeCurrent)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	   {
				
		try {

		//	Initialize file parameter object
			ImportFileParam file = new ImportFileParam();
			file.setAuth(m_auth);
			file.setImportLocation(strTargetLibraryPathName);
			file.setLocalSourceLocation(strSourcePathFileName);
			file.setIsMakeAvailable(bolMakeAvailable);
			file.setIsSetCurrent(bolMakeCurrent);

		//	Initialize request object
			ImportFileParamE request = new ImportFileParamE();
			request.setImportFileParam(file);
			
		//	Initiate web service request
			this.m_stubItpamService.importObject(request);
			
		//	Record success
			String strMessage = "Successfully completed request to import file " + strSourcePathFileName + "'";
			ItpamWebServiceClient.logger.info(strMessage);
			
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_IMPORT_OBJECT")
								  + " (" + strSourcePathFileName + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_IMPORT_OBJECT")
				  				  + " (" + strSourcePathFileName + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
	}

/**
 * Reject pending interaction request.
 * 
 * @param lInteractionId - long pending interaction ID value
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void rejectPendingInteraction(long lInteractionId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Approve pending interaction ID " + lInteractionId);

	//	Initialize boolean flag
		boolean bolApprove = false;
		
	//	Invoke process control method
		this.executePendingInteraction(lInteractionId, bolApprove, null);
	}


/**
 * Resume process specified by process ID.
 * 
 * @param lProcessId - String set to process ID
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void resumeProcess(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Suspend process ID " + lProcessId);
	
	//	Initialize action object
		Action action = Action.resume;
		
	//	Invoke process control method
		this.controlProcess(lProcessId, action);
	}

	
/**
 * Start process specified.
 * 
 * @param strProcessName - String set to fully qualified process name
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') or null. Required if
 * parameter 'strStartTime' is specified
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm') or null
 * @param strParameterNameAttachments - String set to parameter name attachments will be linked or null. Required if attachment
 * properties are specified.
 * @param propertiesAttachments - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return long set to process ID value
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public long startProcess(String strProcessName,
							   Properties propertiesParameters,
							   String strStartDate, String strStartTime,
			                   String strParameterNameAttachments, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	       ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Start process '" + strProcessName + "'");
		
	//	Set action
		String strAction = "Start";
		
	//	Invoke method to execute process
		return this.executeProcess(strProcessName, strAction, propertiesParameters, strStartDate, strStartTime, strParameterNameAttachments, propertiesAttachments);
	}
	
	
/**
 * Initiate process via start request form.
 * 
 * @param strStartRequestName - String set to start request form name
 * @param strStartRequestPath - String set to start request form library path
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') or null
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm') or null
 * @param strParameterNameAttachments - String set to parameter name attachments will be linked or null. Required if attachment
 * properties are specified.
 * @param propertiesAttachments - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return long start request interaction ID value
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public long startRequest(String strStartRequestName, String strStartRequestPath,
							   Properties propertiesParameters, String strStartDate, String strStartTime,
							   String strParameterNameAttachments, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Start request '" + strStartRequestName + "'");
		
		return this.executeStartRequest(strStartRequestName, strStartRequestPath,
		 		   						propertiesParameters, strStartDate, strStartTime,
		 		   						strParameterNameAttachments, propertiesAttachments);
	}
	
	
/**
 * Suspend process specified by process ID.
 * 
 * @param lProcessId - String set to process ID
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	public void suspendProcess(long lProcessId)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Record progress
		ItpamWebServiceClient.logger.debug("Suspend process ID " + lProcessId);
	
	//	Initialize action object
		Action action = Action.suspend;
		
	//	Invoke process control method
		this.controlProcess(lProcessId, action);
	}

	
/**
 * Abort, resume or suspend process specified.
 * 
 * @param lProcessId - String set to process ID
 * @param action - Action object (Action.abort, Action.resume, Action.suspend)
 * 
 * @return String set to resulting action status
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private void controlProcess(Long lProcessId, Action action)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
		
		try {
			
		//	Initialize request object
			ControlProcess request = new ControlProcess();
			request.setAction(action);
			request.setAuth(this.m_auth);
			request.setProcessID(lProcessId);
			
		//	Initiate web service request
			this.m_stubItpamService.controlProcess(request);
			
		//	Record success
			String strMessage = "Successfully completed request to '" + action.toString() + "' process ID " + lProcessId;
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_CONTROL_PROCESS")
								  + " (" + lProcessId + ", " + action.toString() + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_CONTROL_PROCESS")
				  				  + " (" + lProcessId + ", " + action.toString() + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
	}

	
/**
 * Execute specified process.
 * 
 * @param strProcessName - String set to fully qualified process name
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') or null. Required if
 * parameter 'strStartTime' is specified
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm') or null
 * @param strParameterNameAttachments - String set to parameter name attachments will be linked or null. Required if attachment
 * properties are specified.
 * @param propertiesAttachments - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return String set to process ID
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private long executeProcess(String strProcessName,
								  String strAction,
							      Properties propertiesParameters,
							      String strStartDate, String strStartTime,
			                      String strParameterNameAttachments, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	       ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Initialize return value
		long lProcessId;
		
		try {
			
		//	Initialize name object
			Name name = this.getInstanceOfName(strProcessName);	
			
		//	Initialize attachments object
			Attachments attachments = this.getInstanceOfAttachments(strParameterNameAttachments, propertiesAttachments);
			
		//	Initialize options object
			Options options = this.getInstanceOfOptions(strStartDate, strStartTime);
			
		//	Initialize parameters object
			Params parameters = this.getInstanceOfParameters(propertiesParameters);
			
		//	Initialize flow object
			Flow flow = new Flow();
			flow.setAction(strAction);
			flow.setAttachments(attachments);
			flow.setAuth(m_auth);
			flow.setName(name.getName());
			flow.setOptions(options);
			flow.setParams(parameters);
			
		//	Initialize request object
			ExecuteProcess request = new ExecuteProcess();
			request.setFlow(flow);
			
		//	Initiate web service request
			ExecuteC2OFlowResponse response = this.m_stubItpamService.executeProcess(request);
			
		//	Update return value
			lProcessId = Long.parseLong(response.getExecuteC2OFlowResult().getROID());
			
		//	Record success
			String strMessage = "Successfully completed request to '" + strAction + "' process '" + this._strProcessName + "' "
				              + " process ID " + lProcessId;
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_EXECUTE_PROCESS")
								  + " (" + strProcessName + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_EXECUTE_PROCESS")
				  				  + " (" + strProcessName + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
		
	//	Return value
		return lProcessId;
	}
	
	
/**
 * Approve or reject pending interaction.
 * 
 * @param lInteractionId - long user interaction ID value
 * @param bolApprove - boolean set to 'true' to approve or false to reject the request
 * @param propertiesParameters - Properties object encapsulating parameter names and values
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private void executePendingInteraction(long lInteractionId, boolean bolApprove, Properties propertiesParameters)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{			
	//	Initialize action string
		String strAction = null;
		if (bolApprove){
			strAction = "approve";
		}
		else {
			strAction = "reject";
		}
		
		try {
			
		//	Initialize user interaction ID object
			UserInteractionID id = new UserInteractionID();
			id.setUserInteractionID(lInteractionId);
			
		//	Initialize parameters object
			Params parameters = this.getInstanceOfParameters(propertiesParameters);
			
		//	Initialize request object
			ExecutePendingInteraction request = new ExecutePendingInteraction();
			request.setAuth(this.m_auth);
			request.setUserInteractionID(id);
			request.setParams(parameters);
			
		//	Determine if approved or rejected flag is supported by pending interaction
			ItpamInteraction interaction = this.getPendingInteraction(lInteractionId);
			if (interaction.getIsApprovalRequired()) {
				
				request.setIsApprove(bolApprove);
			}
			else {
				
				if (!bolApprove) {
					
					String strMessage = this._messages.get("E_METHOD_REJECT_PENDING_INTERACTION");
					String strLocalizedMessage = this._messages.getLocalized("E_METHOD_REJECT_PENDING_INTERACTION");
					throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
				}
			}
			
		//	Initiate web service request
			this.m_stubItpamService.executePendingInteraction(request);
			String strMessage = "Successfully completed request to '" + strAction + "' pending interaction ID " + lInteractionId;
			if (!this.getParametersList(parameters).equals("")) {

				strMessage = strMessage + " (";
				strMessage = strMessage + this.getParametersList(parameters);
				strMessage = strMessage + ")";
			}
			ItpamWebServiceClient.logger.info(strMessage);
		}
		catch (Exception e) {

			try {
			//	Initialize exception messages
				String strMessage = this._messages.get("E_METHOD_EXECUTE_PENDING_INTERACTION")
								  + " (" + lInteractionId + ", " + strAction + ")";
				String strLocalizedMessage = this._messages.getLocalized("E_METHOD_EXECUTE_PENDING_INTERACTION")
				  				  + " (" + lInteractionId + ", " + strAction + ")";
				
			//	Get type specific exception
				e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
			}
			catch (Exception eUnexpected){
				
				e = eUnexpected;
			}
			
		//	Record exception
			ItpamWebServiceClient.logger.error(e.getMessage());
			
		//	Throw specific exception type
			throw(e);
		}
	}


/**
 * Initiate process via start request form.
 * 
 * @param strStartRequestName - String set to start request form name
 * @param strStartRequestPath - String set to start request form library path
 * @param propertiesParameters - Properties object encapsulating parameter names and values or null
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy') or null
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm') or null
 * @param strParameterNameAttachments - String set to parameter name attachments will be linked or null. Required if attachment
 * properties are specified.
 * @param propertiesAttachments - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return String set to process ID
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private long executeStartRequest(String strStartRequestName, String strStartRequestPath,
							 		   Properties propertiesParameters, String strStartDate, String strStartTime,
							 		   String strParameterNameAttachments, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Initialize return value
		long lStartRequestInteractionId;
		
		try {
			
		//	Initialize attachments object
			Attachments attachments = this.getInstanceOfAttachments(strParameterNameAttachments, propertiesAttachments);
			
		//	Initialize location object
			ObjLocation location = this.getInstanceOfLocation(strStartRequestName, strStartRequestPath);
			
		//	Initialize options object
			Options options = this.getInstanceOfOptions(strStartDate, strStartTime);
			
		//	Initialize parameters object
			Params parameters = this.getInstanceOfParameters(propertiesParameters);
			
		//	Initialize request object
			ExecuteStartRequest request = new ExecuteStartRequest();
			request.setAttachments(attachments);
			request.setAuth(this.m_auth);
			request.setObjLocation(location);
			request.setOptions(options);
			request.setParams(parameters);
			
		//	Initiate web service request
			ExecuteStartRequestResponse response = this.m_stubItpamService.executeStartRequest(request);
			
		//	Update return value
			lStartRequestInteractionId = response.getStartRequestStatus().getInteractionId();
			
		//	Record progress
			String strMessage = "Successfully completed request to 'Start' start request " + location.getPath() + location.getName() + " start request interaction ID " + lStartRequestInteractionId;
			if (!this.getParametersList(parameters).equals("")) {

				strMessage = strMessage + " (";
				strMessage = strMessage + this.getParametersList(parameters);
				strMessage = strMessage + ")";
			}
			ItpamWebServiceClient.logger.info(strMessage);
			}
			catch (Exception e) {

				try {
				//	Initialize exception messages
					String strMessage = this._messages.get("E_METHOD_EXECUTE_START_REQUEST")
									  + " (" + this._strStartRequestName + ", " + this._strStartRequestPath + ")";
					String strLocalizedMessage = this._messages.getLocalized("E_METHOD_EXECUTE_START_REQUEST")
					  				  + " (" + this._strStartRequestName + ", " + this._strStartRequestPath + ")";
					
				//	Get type specific exception
					e = this.getTypeSpecificException(e, strMessage, strLocalizedMessage);
				}
				catch (Exception eUnexpected){
					
					e = eUnexpected;
				}
				
			//	Record exception
				ItpamWebServiceClient.logger.error(e.getMessage());
				
			//	Throw specific exception type
				throw(e);
			}
		
	//	Return value
		return lStartRequestInteractionId;
	}
	
	
/**
 * Convert POJO Strings set to content ID and content type to ITPAM version specific AttachmentsFilter object
 * 
 * @param strContentId - String set to content ID or null
 * @param strContentType - String set to content type or null
 * 
 * @return version specific ITPAM AttachmentFilter object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private AttachmentFilter getInstanceOfAttachmentFilter(String strContentId, String strContentType)
	throws ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
		
	//	Instantiate attachment filter object
		AttachmentFilter filter = null;
		
	//	Throw exception if either parameter is not null or empty
		if ((!this.isNullEmpty(strContentId)) || (!this.isNullEmpty(strContentType))){

		//	Initialize attachment filter object
			filter = new AttachmentFilter();
			if (!this.isNullEmpty(strContentId)) filter.setContentId(strContentId);
			if (!this.isNullEmpty(strContentType)) filter.setContentType(strContentType);
		}
		
	//	Return object
		return filter;
	}
	
	
/**
 * Convert POJO array of long attachment IDs to ITPAM version specific AttachmentIDs object
 * 
 * @param aryAttachmentIds - array of long attachment IDs
 * 
 * @return version specific ITPAM AttachmentIDs object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private AttachmentIds getInstanceOfAttachmentIds(long[] aryAttachmentIds)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
		
	//	Instantiate attachments object
		AttachmentIds attachmentIds = null;
		
	//	Throw exception if either parameter is not null or empty
		if (aryAttachmentIds == null){
			
		//	Set exception messages
			String strMessage = this._messages.get("E_MISSING_OR_INVALID_PARAMETER")
							  + " (attachmentIds)";
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_PARAMETER")
			  				  + " (attachmentIds)";
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);	
		}
		else if (aryAttachmentIds.length == 0){
			
		//	Set exception messages
			String strMessage = this._messages.get("E_MISSING_OR_INVALID_PARAMETER")
							  + " (attachmentIds)";
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_PARAMETER")
			  				  + " (attachmentIds)";
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);	
		}
		else {
			
		//	Initialize attachments object
			attachmentIds = new AttachmentIds();
			attachmentIds.setAttachmentId(aryAttachmentIds);
		}
		
	//	Return object
		return attachmentIds;
	}


/**
 * Convert POJO String and Properties objects to ITPAM version specific Attachments object
 * 
 * @param strParameterName - String set to parameter name attachments will be linked to
 * @param propertiesAttachemsnts - Properties object encapsulating attachment name and URI name/value pairs
 * 
 * @return version specific ITPAM Attachments object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private Attachments getInstanceOfAttachments(String strParameterName, Properties propertiesAttachments)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	
	//	Instantiate attachments object
		Attachments attachments = null;
			
	//	Determine if valid parameters specified
		if ((this.isNullEmpty(strParameterName)) && (propertiesAttachments != null)){
			
		//	Set exception messages
			String strMessage = this._messages.get("E_MISSING_OR_INVALID_PARAMETER")
							  + " (attachmentParameterName)";
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_PARAMETER")
			  				  + " (attachmentParameterName)";
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);	
		}
		else if ((!this.isNullEmpty(strParameterName)) && (propertiesAttachments == null)){
			
		//	Set exception messages
			String strMessage = this._messages.get("E_MISSING_OR_INVALID_PARAMETER")
							  + " (Attachment Properties)";
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_PARAMETER")
			  				  + " (Attachment Properties)";
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);				
		}
		else if (  (!this.isNullEmpty(strParameterName))
				&& (propertiesAttachments != null)
				&& (propertiesAttachments.size() > 0))
		{
			
			try {
				
			//	Initialize attachments object	
				attachments = new Attachments();
				attachments.setAttachmentsParamName(strParameterName);
				
			//	Iterate and add attachments
				for (Iterator<Object> iAttachments = propertiesAttachments.keySet().iterator(); iAttachments.hasNext();){
					
				//	Initialize attachment and SOAP with attachments reference objects					
					Attachment attachment = new Attachment();					
					SwaRef swaref = new SwaRef();					
					
				//	Get/set attachment ID, name and source location
					String strAttachmentName = (String) iAttachments.next();
					String strAttachmentSourceLocation = propertiesAttachments.getProperty(strAttachmentName);
					swaref.setSwaRef(new URI("cid:" + strAttachmentName));
						
				//	Set attachment properties
					attachment.setAttachmentID(swaref);
					attachment.setName(strAttachmentName);
					attachment.setLocalSourceLocation(strAttachmentSourceLocation);
					
				//	Add attachment to attachments object
					attachments.addAttachment(attachment);
				}
			}
			catch (MalformedURIException e) {
				
			//	Set exception messages
				String strMessage = this._messages.get("E_MISSING_OR_INVALID_URI")
								  + " (Attachment Properties)";
				String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_URI")
				  				  + " (Attachment Properties)";
				
			//	Record exception
				ItpamWebServiceClient.logger.error(strMessage);
				
			//	Throw exception
				throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
			}
		}
		
	//	Return attachment object
		return attachments;
	}
	
	
/**
 * Convert POJO Strings set to start and end dates to DateRange object
 * 
 * @param strStartDate - String set to date range start date
 * @param strEndDate - String set to date range end date
 * 
 * <br>
 * <br>Supported date formats 'yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy'
 * <br>Supported time formats 'hh:mm:ss', 'hh:mm'
 * <br>
 * 
 * @return version specific ITPAM DateRange object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private DateRange getInstanceOfDateRange(String strStartDate, String strEndDate)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Instantiate return object
		DateRange dateRange = null;			
			
	//	Initialize start date
		Date dateStart = this.getDateFromString(strStartDate);
		
	//	Initialize end date
		Date dateEnd = this.getDateFromString(strEndDate);

	//	Initialize object
		dateRange = new DateRange();
		dateRange.setFromDate(dateStart);
		dateRange.setToDate(dateEnd);
		
	//	Return date range object
		return dateRange;
	}
	
	
/**
 * Convert POJO Strings set to target path and file name to IT PAM download file name object
 * 
 * @param strStartDate - String set to date range start date
 * @param strEndDate - String set to date range end date
 * 
 * <br>
 * <br>Supported date formats 'yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy'
 * <br>Supported time formats 'hh:mm:ss', 'hh:mm'
 * <br>
 * 
 * @return version specific ITPAM DownloadFileName object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private DownloadFileName getInstanceOfDownloadFileName(String strTargetPath, String strTargetFileName, boolean bolOverwriteTargetFile)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Instantiate return object
		DownloadFileName downloadFileName = null;
		
	//	Validate required arguments
		if ((this.isNullEmpty(strTargetPath)) || (this.isNullEmpty(strTargetFileName))){
			
		//	Set exception messages
			String strMessage = this._messages.get("E_MISSING_OR_INVALID_TARGET_PATH");
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_TARGET_PATH");
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else {
			
		//	Initialize object
			downloadFileName = new DownloadFileName();
			downloadFileName.setDownloadLocation(strTargetPath);
			downloadFileName.setFileName(strTargetFileName);
			downloadFileName.setOverwriteFile(bolOverwriteTargetFile);
		}
		
	//	Return date range object
		return downloadFileName;
	}
	
	
/**
 * Convert POJO array of Strings set to object types of objects in the source path to ExportLibFilter object
 * 
 * @param aryObjectTypeFilterStrings - Array of Strings set to object types of objects in the source path to
 * 		be exported
 * <br><br>	
 * 		Supported object type filter strings:
 * <br>	- Agenda
 * <br>	- Calendar
 * <br>	- Commander
 * <br>	- CustomIcon
 * <br>	- CustomOperator
 * <br>	- CustomSensor
 * <br>	- Dataset
 * <br>	- InteractionRequestForm
 * <br>	- LogViewer
 * <br>	- Package
 * <br>	- Process
 * <br>	- ProcessWatch
 * <br>	- Resources
 * <br>	- StatePolicy
 * <br>	- System
 * <br>
 * 
 * @return version specific ITPAM ExportLibFilter object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private ExportLibFilter getInstanceOfExportLibFilter(String[] aryObjectTypeFilterStrings)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Initialize return object
		ExportLibFilter filter = null;
		
	//	Initialize export object types object
		if (aryObjectTypeFilterStrings != null){
			
		//	Initialize supported object types string
			String strObjectTypesSupported = "Agenda, "
										   + "Calendar, "
										   + "Commander, "
										   + "CustomIcon, "
										   + "CustomOperator, "
										   + "CustomSensor, "
										   + "Dataset, "
										   + "InteractionRequestForm, "
										   + "LogViewer, "
										   + "Package, "
										   + "Process, "
										   + "ProcessWatch, "
										   + "Resources, "
										   + "StatePolicy, "
										   + "System";
			
		//	Instantiate export object types object
			ExportObjectTypes exportObjectTypes = new ExportObjectTypes();
			
		//	Iterate object type strings
			for (int intObjectType = 0; intObjectType < aryObjectTypeFilterStrings.length; intObjectType++){
				
			//	Validate object type string
				if (this.isNullEmpty(aryObjectTypeFilterStrings[intObjectType])){
					
				//	Set exception messages
					String strMessage = this._messages.get("E_MISSING_OR_INVALID_OBJECT_TYPE")
	  				  				  + " (" + strObjectTypesSupported + ")";
					String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_OBJECT_TYPE")
					  				  + " (" + strObjectTypesSupported + ")";
					
				//	Record exception
					ItpamWebServiceClient.logger.error(strMessage);
					
				//	Throw exception
					throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
				}
				else {
					
					if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("Agenda")){
						
						exportObjectTypes.addObjectType(ObjectTypes.Agenda);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("Calendar")){
						
						exportObjectTypes.addObjectType(ObjectTypes.Calendar);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("Commander")){
						
						exportObjectTypes.addObjectType(ObjectTypes.Commander);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("CustomIcon")){
						
						exportObjectTypes.addObjectType(ObjectTypes.CustomIcon);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("CustomSensor")){
						
						exportObjectTypes.addObjectType(ObjectTypes.CustomSensor);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("Dataset")){
						
						exportObjectTypes.addObjectType(ObjectTypes.Dataset);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("InteractionRequestForm")){
						
						exportObjectTypes.addObjectType(ObjectTypes.InteractionRequestForm);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("LogViewer")){
						
						exportObjectTypes.addObjectType(ObjectTypes.LogViewer);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("Package")){
						
						exportObjectTypes.addObjectType(ObjectTypes.Package);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("Process")){
						
						exportObjectTypes.addObjectType(ObjectTypes.Process);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("ProcessWatch")){
						
						exportObjectTypes.addObjectType(ObjectTypes.ProcessWatch);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("Resources")){
						
						exportObjectTypes.addObjectType(ObjectTypes.Resources);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("StatePolicy")){
						
						exportObjectTypes.addObjectType(ObjectTypes.StatePolicy);						
					}
					else if (aryObjectTypeFilterStrings[intObjectType].equalsIgnoreCase("System")){
						
						exportObjectTypes.addObjectType(ObjectTypes.System);						
					}
					else {
					//	Set exception messages
						String strMessage = this._messages.get("E_MISSING_OR_INVALID_OBJECT_TYPE")
		  				  				 + " (" + strObjectTypesSupported + ")";
						String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_OBJECT_TYPE")
		  				  				  + " (" + strObjectTypesSupported + ")";
						
					//	Record exception
						ItpamWebServiceClient.logger.error(strMessage);
						
					//	Throw exception
						throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
					}
				}
			}

			filter = new ExportLibFilter();
			filter.setExportObjectTypes(exportObjectTypes);
		}
		
	//	Return object
		return filter;
	}
	
	
/**
 * Convert POJO array of Strings set to start request forms keywords to Keyword object
 * 
 * @param aryKeywords - Array of Strings set to start request form keyword(s)
 * 
 * @return version specific ITPAM Keyword object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private Keywords getInstanceOfKeywords(String[] aryKeywords)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
	   	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{

	//	Initialize return object
		Keywords keywords = null;
		
	//	Initialize keywords object
		if (aryKeywords != null){
			
		//	Iterate keywords
			for (int intKeyword = 0; intKeyword < aryKeywords.length; intKeyword++){
				
			//	Validate keyword
				if (this.isNullEmpty(aryKeywords[intKeyword])){
					
				//	Set exception messages
					String strMessage = this._messages.get("E_MISSING_OR_INVALID_KEYWORD")
									  + " (Keywords)";
					String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_KEYWORD")
					  				  + " (Keywords)";
					
				//	Record exception
					ItpamWebServiceClient.logger.error(strMessage);
					
				//	Throw exception
					throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
				}
			}

			keywords = new Keywords();
			keywords.setKeyword(aryKeywords);
		}
		
	//	Return object
		return keywords;
	}
	
	
/**
 * Convert POJO Strings set to start request name and path to Location object
 * 
 * @param strStartRequestName - String set to start request name
 * @param strStartRequestPathPath - String set to library path
 * 
 * @return version specific ITPAM Location object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private ObjLocation getInstanceOfLocation(String strStartRequestName, String strStartRequestPath)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Initialize return object
		ObjLocation location = new ObjLocation();
		
	//	Update start request name and path fields
		this._strStartRequestName = strStartRequestName;
		this._strStartRequestPath = strStartRequestPath;
	
	//	Throw exception if missing or invalid detected otherwise update return object
		if ((this.isNullEmpty(strStartRequestName)) || (this.isNullEmpty(strStartRequestPath))){
			
		//	Set exception message
			String strMissingParms = "";
			if ((this.isNullEmpty(strStartRequestName))){
				
				this._strStartRequestName = "?";
				strMissingParms = "startRequestName";
			}
			if ((this.isNullEmpty(strStartRequestPath))){

				this._strStartRequestPath = "?";
				if (!this.isNullEmpty(strMissingParms)) strMissingParms = strMissingParms + ", ";
				strMissingParms = strMissingParms + "startRequestPath";
			}
			
		//	Set exception messages
			String strMessage = this._messages.get("E_MISSING_OR_INVALID_PARAMETER")
							  + " (" + strMissingParms + ")";
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_PARAMETER")
			  				  + " (" + strMissingParms + ")";;
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else {
					
		//	Append required trailing slash to path if needed
			if (!strStartRequestPath.endsWith("/")) {strStartRequestPath = strStartRequestPath + "/";}
			
		//	Set start request name and path
			this._strStartRequestName = strStartRequestName;
			this._strStartRequestPath = strStartRequestPath;
			
		//	Update object
			location.setName(strStartRequestName);
			location.setPath(strStartRequestPath);
		}
		
	//	Return location object
		return location;
	}
	
	
/**
 * Convert POJO Strings set to start date and time to ITPAM version specific Options object
 * 
 * @param strProcessName - String set to fully qualified process name
 * 
 * @return version specific ITPAM Name object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private Name getInstanceOfName(String strProcessName)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{
	//	Initialize return object
		Name name = new Name();
	
	//	Throw exception if missing or invalid detected otherwise update return object
		if (this.isNullEmpty(strProcessName)){
			
		//	Set process name
			this._strProcessName = "?";
			
		//	Set exception messages
			String strMessage = this._messages.get("E_MISSING_OR_INVALID_PARAMETER")
							  + " (processName)";
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_PARAMETER")
			  				  + " (processName)";;
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else {
			
		//	Set process name
			this._strProcessName = strProcessName;
			
		//	Update object
			name.setName(strProcessName);
		}
		
	//	Return location object
		return name;
	}


/**
 * Convert POJO Strings set to start date and time to ITPAM version specific Options object
 * 
 * @param strStartDate - String set to start date ('yyyy-MM-dd', 'yyyy/MM/dd', 'MM-dd-yyyy', 'MM/dd/yyyy')
 * @param strStartTime - String set to time ('hh:mm:ss', 'hh:mm')
 * 
 * @return version specific ITPAM Options object with start date and/or time set to values specified
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private Options getInstanceOfOptions(String strStartDate, String strStartTime)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
	{

	//	Initialize options object
		Options options = null;
		
	//	Throw exception if invalid or missing parameters detected otherwise attempt to create object
		if ((this.isNullEmpty(strStartDate)) && (!this.isNullEmpty(strStartTime))){
			
		//	Set exception messages
			String strMessage = this._messages.get("E_MISSING_OR_INVALID_PARAMETER")
							  + " (startDate)";
			String strLocalizedMessage = this._messages.getLocalized("E_MISSING_OR_INVALID_PARAMETER")
			  				  + " (startDate)";
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		else if (!isNullEmpty(strStartDate)) {
			
		//	Initialize object and set start date
			options = new Options();
			Date date = this.getDateFromString(strStartDate);
			StartDate startDate = new StartDate();
			startDate.setStartDate(date);
			options.setStartDate(startDate);

		//	Set start time if applicable
			if (!isNullEmpty(strStartTime)) {
				
				Time time = new Time(strStartTime);
				StartTime startTime = new StartTime();
				startTime.setStartTime(time);
				options.setStartTime(startTime);
			}
		}
		
	//	Return options object
		return options;
	}
	
	
/**
 * Convert POJO Properties object to ITPAM version specific Params object
 * 
 * @param properties - Properties object encapsulating parameter names and values
 * 
 * @return version specific ITPAM Params object with parameters set to parameter names and values
 * encapsulated in the specified Properties object
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private Params getInstanceOfParameters(Properties properties)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
    {
		
	//	Initialize parameters object
		Params parameters = null;
		
	//	Iterate properties
		if (properties != null){
			
		//	Initialize and populate parameters object
			 parameters = new Params();
			
			for (Iterator<Object> iKeys = properties.keySet().iterator(); iKeys.hasNext();){
				
				String strParameterName = (String) iKeys.next();
				String strParameterValue = properties.getProperty(strParameterName);
				
			//	Initialize new parameter
				Param parameter = new Param();
				parameter.setName(strParameterName);
				parameter.setString(strParameterValue);
				
			//	Add parameter
				parameters.addParam(parameter);
			}
		}
		
	//	Return parameters object
		return parameters;
	}
	
	
/**
 * Get parameters from specified IT PAM page object.
 * 
 * @param page - IT PAM page object to be processed
 * 
 * @return Vector of ItpamParameter objects
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamUnsupportedParameterTypeException
 * @throws ItpamWebServiceClientException
 */
	private Vector<ItpamParameter> getParametersFromPages(Pages pages) 
	throws Exception, ItpamUnsupportedParameterTypeException
	{

	//	Initialize return vector
		Vector<ItpamParameter> vParameters = new Vector<ItpamParameter>();
		
	//	Process pages
		if (pages != null){

	//	Get array of pages
		Page[] aryPages= pages.getPage();
		
	//	Iterate pages
		if (aryPages != null){
				
			HashMap<String, ItpamParameter> hmParametersBySequenceNo = new HashMap<String, ItpamParameter>();
					
			for (int intPage = 0; intPage < aryPages.length; intPage++){
					
				Page page = aryPages[intPage];
				String strParentPageName = null;
				if (page.getName() != null){strParentPageName = page.getName().getName();}
				
				PageChoice[] aryPageChoices = page.getPageChoice();
				
			//	Iterate page choices
				if (aryPageChoices != null){
						
						for (int intPageChoice = 0; intPageChoice < aryPageChoices.length; intPageChoice++){
								
							PageChoice pageChoice = aryPageChoices[intPageChoice];
							ValGroup valGroup = pageChoice.getValGroup();
							
							if (valGroup.getItpamBoolean() != null){
								
								ItpamBoolean[] aryParameters = valGroup.getItpamBoolean();
								if (aryParameters != null){
									
								}
									for (int intParameter = 0; intParameter < aryParameters.length; intParameter++) {
									
										ItpamParameter parameter = new m_ItpamParameter(aryParameters[intParameter], strParentPageName);										
										if (parameter != null) {hmParametersBySequenceNo.put(strParentPageName + " Parameter No. " + Integer.toString(parameter.getSequenceNo()), parameter);}
									}
							}
							else if (valGroup.getItpamDate() != null){
								
								ItpamDate[] aryParameters = valGroup.getItpamDate();
								if (aryParameters != null){
									
								}
									for (int intParameter = 0; intParameter < aryParameters.length; intParameter++) {
									
										ItpamParameter parameter = new m_ItpamParameter(aryParameters[intParameter], strParentPageName);										
										if (parameter != null) {hmParametersBySequenceNo.put(strParentPageName + " Parameter No. " + Integer.toString(parameter.getSequenceNo()), parameter);}
									}
							}
							else if (valGroup.getItpamDouble() != null){
								
								ItpamDouble[] aryParameters = valGroup.getItpamDouble();
								if (aryParameters != null){
									
								}
									for (int intParameter = 0; intParameter < aryParameters.length; intParameter++) {
									
										ItpamParameter parameter = new m_ItpamParameter(aryParameters[intParameter], strParentPageName);										
										if (parameter != null) {hmParametersBySequenceNo.put(strParentPageName + " Parameter No. " + Integer.toString(parameter.getSequenceNo()), parameter);}
									}
							}
							else if (valGroup.getItpamInt() != null){
								
								ItpamInt[] aryParameters = valGroup.getItpamInt();
								if (aryParameters != null){
									
								}
									for (int intParameter = 0; intParameter < aryParameters.length; intParameter++) {
									
										ItpamParameter parameter = new m_ItpamParameter(aryParameters[intParameter], strParentPageName);										
										if (parameter != null) {hmParametersBySequenceNo.put(strParentPageName + " Parameter No. " + Integer.toString(parameter.getSequenceNo()), parameter);}
									}
							}
							else if (valGroup.getItpamLong() != null){
								
								ItpamLong[] aryParameters = valGroup.getItpamLong();
								if (aryParameters != null){
									
								}
									for (int intParameter = 0; intParameter < aryParameters.length; intParameter++) {
									
										ItpamParameter parameter = new m_ItpamParameter(aryParameters[intParameter], strParentPageName);										
										if (parameter != null) {hmParametersBySequenceNo.put(strParentPageName + " Parameter No. " + Integer.toString(parameter.getSequenceNo()), parameter);}
									}
							}
							else if (valGroup.getItpamPassword() != null){
								
								ItpamValue[] aryParameters = valGroup.getItpamPassword();
								if (aryParameters != null){
									
								}
									for (int intParameter = 0; intParameter < aryParameters.length; intParameter++) {
									
										ItpamParameter parameter = new m_ItpamParameter(aryParameters[intParameter], strParentPageName);										
										if (parameter != null) {hmParametersBySequenceNo.put(strParentPageName + " Parameter No. " + Integer.toString(parameter.getSequenceNo()), parameter);}
									}
							}
							else if (valGroup.getItpamString() != null){
								
								ItpamString[] aryParameters = valGroup.getItpamString();
								if (aryParameters != null){
									
								}
									for (int intParameter = 0; intParameter < aryParameters.length; intParameter++) {
									
										ItpamParameter parameter = new m_ItpamParameter(aryParameters[intParameter], strParentPageName);										
										if (parameter != null) {hmParametersBySequenceNo.put(strParentPageName + " Parameter No. " + Integer.toString(parameter.getSequenceNo()), parameter);}
									}
							}
							else {
	
							//	Initialize exception messages
								String strMessage = this._messages.get("E_UNSUPPORTED_ITPAM_PARAMETER_TYPE");
								String strLocalizedMessage = this._messages.getLocalized("E_UNSUPPORTED_ITPAM_PARAMETER_TYPE");
								
							//	Throw exception
								throw new ItpamUnsupportedParameterTypeException(strMessage, strLocalizedMessage);
							}
						}
				}
			}
				
		//	Initialize list of parameter hash map keys
			List<String> listParameterKeys = new ArrayList<String>(hmParametersBySequenceNo.keySet());
			
		//	Sort list of parameter hash map keys
			Collections.sort(listParameterKeys);
			
		//	Add parameters to return vector in sequence number order
			for (Iterator<String> iParameterKeys = listParameterKeys.iterator(); iParameterKeys.hasNext();){
				
				String strParameterKey = iParameterKeys.next();
				vParameters.add(hmParametersBySequenceNo.get(strParameterKey));
		}
	}
	}
		
	//	Return vector
		return vParameters;
	}
	
	
/**	Get string set to single line list of parameter name/value pairs encapsulated in parameters object specified.
 * 
 * @param parameters - Params object encapsulating parameter name/value pairs
 * 
 * @return String set to single line list of parameter name/value pairs encapsulated in parameters object specified
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private String getParametersList(Params parameters)
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
    {
		
	//	Initialize return string
		String strParametersList = "";
		
	//	Iterate parameters and build list
		if (parameters != null){
			
			if (parameters.getParam() != null) {
				
				for (int intParameter = 0; intParameter < parameters.getParam().length; intParameter++){
					
					if (!strParametersList.equals("")) strParametersList = strParametersList + " ";
					strParametersList = strParametersList + parameters.getParam()[intParameter].getName();
					strParametersList = strParametersList + "=";
					strParametersList = strParametersList + parameters.getParam()[intParameter].getString();
				}
			}
		}
		
	//	Return string
		return strParametersList;
	}


/**
 * Initialize common class level objects.
 * 
 * @throws ItpamMethodNotSupportedException
 * @throws ItpamParameterMissingOrInvalidException
 * @throws ItpamStringToDateException
 * @throws ItpamEntityNotFoundException
 * @throws ItpamUserPermissionsException
 * @throws ItpamUnknownHostException
 * @throws ItpamWebServiceClientException
 */
	private void initCommonObjects()
	throws Exception, ItpamMethodNotSupportedException, ItpamParameterMissingOrInvalidException, ItpamStringToDateException,
    	   ItpamEntityNotFoundException, ItpamUserPermissionsException, ItpamUnknownHostException, ItpamWebServiceClientException
    {		
	//	Record progress
		ItpamWebServiceClient.logger.debug("Initializing web service client common objects");
		
		try {
			
		//	Initialize web service stub
			this.m_stubItpamService = new ItpamServiceStub(this._strTargetEndpointURI);
			
		//	Initialize user authentication object
			UserAuth userAuth = new UserAuth();
			userAuth.setPassword(this._strPassword);
			userAuth.setUser(this._strUser);
			
		//	Initialize authentication object
			this.m_auth = new Auth();
			this.m_auth.setUserAuth(userAuth);	
		}
		catch (Exception e){
				
		//	Set exception messages
			String strMessage = this._messages.get("E_METHOD_INIT_COMMON_OBJECTS");
			String strLocalizedMessage = this._messages.getLocalized("E_METHOD_INIT_COMMON_OBJECTS");
			
		//	Record exception
			ItpamWebServiceClient.logger.error(strMessage);
			
		//	Throw exception
			throw new ItpamParameterMissingOrInvalidException(strMessage, strLocalizedMessage);
		}
		
	//	Record progress
		ItpamWebServiceClient.logger.debug("Initialized web service client common objects");
	}
}
