package com.ca.dso.itpam.webservice.sample;

import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClientFactory;

/**
 * CA IT PAM Web Service Client Sample
 * 
 * @author George Curran, CA
 * 
 * An (intentionally) simplistic class to demonstrate the basic usage of the CA IT PAM Web Servic Client
 * wrapper class.
 * 																													*/
public class Sample {
	
/**
 * The "main" method executed when the "Sample" launch configuration is run (from the menu select "Run" then
 * "Run Configuration", expand the "Java Applications" node in the tree in the left had pane, select "Sample"
 * and finally click the "Run" button).  Before launching, be sure to replace the tokens for <Server>, <Port>,
 * <Version>, <User Name> and <User's Password> with values specific to your test environment.  The method will
 * instantiate an instance of the web service client and invoke the client's "getServerStatus" method.  Status
 * message returned will be output to the console.
 * 
 * @throws Exception if one or more of the environment specific values required (IT PAM web service URI, version
 * user name or password) or the URI specified is not resolvable or unreachable.
 * 																													*/
	public static void main(String[] args) throws Exception {

	//	Replace the <Server>, <Port>, <Version>, <User Name> and <User's Password>
	//	tokens below with values that apply for your test environment.
		String strItpamUri = "http://<Server>:<Port>/itpam/soap";
		String strItpamVersion = "<Version>";
		String strItpamUser = "<User Name>";
		String strItpamPassword = "<User's Password>";	

	//	Because the client is designed to support multiple versions of the
	//	CA IT PAM web service it is implemented by an abstract class that is
	//	extended by a classes specific to each version.  Since abstract classes
	//	cannot be instantiated directly a "factory" object is instantiated and
	//	used.
		ItpamWebServiceClientFactory itpamWebServiceClientFactory
			= new ItpamWebServiceClientFactory();
		
	//	An instance of the web service client by invoking the "getInstanceOf"
	//	methods exposed by the "factory".
		ItpamWebServiceClient itpamWebServiceClient 
			= itpamWebServiceClientFactory.getInstanceOf(strItpamVersion,
														 strItpamUri,
														 strItpamUser,
														 strItpamPassword);
		
	//	To verify the web service client can actually communicate with the
	//	web service, invoke the "getServerStatus" method.  The method will
	//	test the connection with the server hosting the web service and
	//	return a indicating server status if successful.  Should an exception
	//	result, check the details of the error message and make the necessary
	//	correction (typically an invalid URI, version, user name or password).
		String strServerStatus = itpamWebServiceClient.getServerStatus();
		
	//	Output the server status message to the console.
		System.out.println(strServerStatus);
	}
}

