package com.ca.dso.itpam.webservice.sample;

import java.util.Random;

import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient.ItpamProcessStatus;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient.ItpamStartRequestStatus;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClientFactory;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient_3_0;
import com.ca.dso.itpam.webservice.client.ItpamWebServiceClient_4_0;

/**
 * CA IT PAM Web Service Client Sample
 * 
 * @author ainmo01, CA
 * 
 * A class to test all the enhancements made to the CLI utility as part of the PAM 4.2 release.
 * 																													*/
public class Tests {
	
	private static Boolean failed = false;
	
/**
 * main method
 * 
 * @throws Exception if one or more of the environment specific values required (IT PAM web service URI, version
 * user name or password) or the URI specified is not resolvable or unreachable.
 * 																													*/
	@SuppressWarnings("unused")
	public static void main(String[] args) throws Exception {

		//	Replace the <Server>, <Port>, <Version>, <User Name> and <User's Password>
		//	tokens below with values that apply for your test environment.
		String strItpamUri = args[0];
		String strItpamVersion = "";
		String strItpamUser = args[1];
		String strItpamPassword = args[2];	
		String importProcessPath = args[3];
		String importSrfPath = args[4];
		
		if(args == null || args.length != 5) {
			System.err.println("Please pass the following input values: PAM_Web_Service_URI Username Password ProcessPath_To_Import SRFPath_To_Import");
		}
		
		
		//	Because the client is designed to support multiple versions of the
		//	CA IT PAM web service it is implemented by an abstract class that is
		//	extended by a classes specific to each version.  Since abstract classes
		//	cannot be instantiated directly a "factory" object is instantiated and
		//	used.
		ItpamWebServiceClientFactory itpamWebServiceClientFactory = new ItpamWebServiceClientFactory();
		
		// Test - version=2.1
		try {
			System.out.println("Test: version=2.1 is specified by the user. This version is not supported.");
			strItpamVersion = "2.1";
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			System.out.println(" --- FAILED.");
			failed = true;
		} catch (Exception ex) {
			System.out.println(" --- PASSED.");
		}
		
		System.out.println();
		
		// Test - version=2.2
		try {
			System.out.println("Test: version=2.2 is specified by the user. This version is not supported.");
			strItpamVersion = "2.2";
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			System.out.println(" --- FAILED.");
			failed = true;
		} catch (Exception ex) {
			System.out.println(" --- PASSED.");
		}
		
		System.out.println();
		
		// Test - version=3.1
		try {
			System.out.println("Test: version=3.x is specified by the user. This version is supported.");
			strItpamVersion = "3.1";
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			if(itpamWebServiceClient instanceof ItpamWebServiceClient_3_0) {
				System.out.println(" --- PASSED.");
			} else {
				System.out.println(" --- FAILED.");
				failed = true;
			}
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		System.out.println();
		
		// Test - version 4.2
		try {
			System.out.println("Test: version=4.x is specified by the user. This version is supported.");
			strItpamVersion = "4.0";
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			if(itpamWebServiceClient instanceof ItpamWebServiceClient_4_0) {
				System.out.println(" --- PASSED.");
			} else {
				System.out.println(" --- FAILED.");
				failed = true;
			}
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		System.out.println();
		
		// Test - version is not passed
		try {
			System.out.println("Test: version number is not specified by the user. The utility uses the version retrieved from the PAM Web Service.");
			strItpamVersion = null;
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			if(itpamWebServiceClient instanceof ItpamWebServiceClient_4_0 || itpamWebServiceClient instanceof ItpamWebServiceClient_3_0) {
				System.out.println(" --- PASSED.");
			} else {
				System.out.println(" --- FAILED.");
				failed = true;
			}
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		System.out.println();
		
		// Test - version specified by user is > version retrieved by utility
		try {
			System.out.println("Test: version number specified by the user (9.0) is greater than the version number retrieved by the utility. This is not supported.");
			strItpamVersion = "9.0";
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			System.out.println(" --- FAILED.");
			failed = true;
		} catch (Exception ex) {
			System.out.println(" --- PASSED.");
		}
		
		System.out.println();
		
		// Test - importObject
		try {
			System.out.println("Test: import a process with an OverwriteAction.");

			strItpamVersion = "3.1";
			importObject(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, "replaceObject", importProcessPath);
			
			strItpamVersion = "4.0";
			importObject(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, "replaceObject", importProcessPath);
			importObject(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, "incrementObjectVersion", importProcessPath);
			importObject(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, "skipImport", importProcessPath);
			
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		System.out.println();
		
		// Test - Start Request
		String srfRandomTagId_30 = "PAM31000-" + String.valueOf(randomNumberInRange()) + "-srf-8cce-8781c18647c8";
		String srfRandomTagId_40 = "PAM40000-" + String.valueOf(randomNumberInRange()) + "-srf-8cce-8781c18647c8";
			
		try {
			System.out.println("Test: execute a start request form with tagId and isAutoArchive.");
			
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf("3.0", strItpamUri, strItpamUser, strItpamPassword);
			itpamWebServiceClient.importObject(importSrfPath, "/WS_CLI_Tests/", true, true);
			
			
			strItpamVersion = "3.1";
			startRequest(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, 
					"newDataset_SRF", "/WS_CLI_Tests/", srfRandomTagId_30 , false);
			
			strItpamVersion = "4.0";
			startRequest(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, 
					"newDataset_SRF", "/WS_CLI_Tests/", srfRandomTagId_40 , false);
			
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		System.out.println();
		
		// Test Get Start Request Status
		try {
			System.out.println("Test: get start request form status with tagId.");
				
			strItpamVersion = "3.1";
			getStartRequestStatus(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, srfRandomTagId_30);
			
			strItpamVersion = "4.0";
			getStartRequestStatus(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, srfRandomTagId_40);
			
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		System.out.println();
		
		// Test - Start Process
		String processRandomTagId_30 = "PAM31000-" + String.valueOf(randomNumberInRange()) + "-proc-8cce-8781c18647c8";
		String processRandomTagId_40 = "PAM40000-" + String.valueOf(randomNumberInRange()) + "-proc-8cce-8781c18647c8";
			
		long procId = 0;
		try {
			System.out.println("Test: start a process with tagId and isAutoArchive.");
				
			strItpamVersion = "3.1";
			startProcess(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, 
					"/WS_CLI_Tests/newDataset", processRandomTagId_30 , false);
			
			strItpamVersion = "4.0";
			procId = startProcess(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, 
					"/WS_CLI_Tests/newDataset", processRandomTagId_40 , false);
			
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		System.out.println();
		
		// Test Get Process status
		try {
			System.out.println("Test: get process status with tagId.");
				
			strItpamVersion = "3.1";
			getProcessStatus(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, processRandomTagId_30);
			
			strItpamVersion = "4.0";
			getProcessStatus(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, processRandomTagId_40);
			
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		System.out.println();
		
		// Test control instance
		try {
			System.out.println("Test: control instance with rootUUID, instanceId, and tagId.");
			String rootUuid = "";
			long instanceId = procId;
			String tagId = "";
			
			strItpamVersion = "3.1";
			controlInstance(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, rootUuid, instanceId, tagId);
			
			// instance Id
			strItpamVersion = "4.0";
			controlInstance(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, rootUuid, instanceId, tagId);
			
			// tag Id
			instanceId = 0;
			tagId = processRandomTagId_40;
			controlInstance(itpamWebServiceClientFactory, strItpamVersion, strItpamUri, strItpamUser, strItpamPassword, rootUuid, instanceId, tagId);
			
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		if(failed) {
			System.err.println("\n ===> Some tests FAILED.");
			System.exit(1);
		} else {
			System.out.println("\n ===> All tests PASSED.");
			System.exit(0);
		}
		
	}
	
	private static void importObject (ItpamWebServiceClientFactory itpamWebServiceClientFactory, String strItpamVersion, String strItpamUri, String strItpamUser,
			String strItpamPassword, String strOverwriteAction, String importProcessPath) {
		try {
			String output = "";
			if(strItpamVersion.startsWith("3")) {
				output = " --- (PAM: 3.x) OverwriteAction is ignored";
			} else {
				output = " --- (PAM: 4.x) OverwriteAction=" + strOverwriteAction;
			}

			System.out.print(output);
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			itpamWebServiceClient.importObject(importProcessPath, "/WS_CLI_Tests/", true, true, strOverwriteAction);
			System.out.println(" --- PASSED.");
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
	}
	
	private static long startRequest (ItpamWebServiceClientFactory itpamWebServiceClientFactory, String strItpamVersion, String strItpamUri, String strItpamUser,
			String strItpamPassword, String importSrfFileName, String importSrfDirPath, String tagId, boolean isAutoArchive) {
		long res = 0;
		try {
			String output = "";
			if(strItpamVersion.startsWith("3")) {
				output = " --- (PAM: 3.x) tagId=" + tagId + " and isAutoArchive=" + isAutoArchive + " are ignored";
			} else {
				output = " --- (PAM: 4.x) tagId=" + tagId + " and isAutoArchive=" + isAutoArchive;
			}

			System.out.print(output);
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			res = itpamWebServiceClient.startRequest(importSrfFileName, importSrfDirPath, null, null, null, tagId, isAutoArchive, null, null, null);
			System.out.println(" --- PASSED (Id:" + res + ")");
			
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		return res;
	}
	
	private static void getStartRequestStatus (ItpamWebServiceClientFactory itpamWebServiceClientFactory, String strItpamVersion, String strItpamUri, String strItpamUser,
			String strItpamPassword,String tagId) {
		try {
			String output = "";
			if(strItpamVersion.startsWith("3")) {
				output = " --- (PAM: 3.x) tagId=" + tagId + " was never set";
			} else {
				output = " --- (PAM: 4.x) tagId=" + tagId + " was set";
			}

			System.out.print(output);
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			ItpamStartRequestStatus res = itpamWebServiceClient.getStartRequestStatus(0, tagId);
			System.out.println(" --- PASSED. (Id:" + res.getInteractionId() + " / Status:" + res.getState() + ")");
			
		} catch (Exception ex) {
			if(strItpamVersion.startsWith("3")) {
				System.out.println(" --- PASSED. (Unable to retrieve status using TagId)");
			} else {
				System.out.println(" --- FAILED.");
				failed = true;
			}
			
		}
		
	}
	
	private static int randomNumberInRange() {
	    Random random = new Random();
	    int randomNum = random.nextInt(1000) + 1000;
	    return randomNum;
	}
	
	
	private static long startProcess (ItpamWebServiceClientFactory itpamWebServiceClientFactory, String strItpamVersion, String strItpamUri, String strItpamUser,
			String strItpamPassword, String importSrfFileName, String tagId, boolean isAutoArchive) {
		long res = 0;
		try {
			String output = "";
			if(strItpamVersion.startsWith("3")) {
				output = " --- (PAM: 3.x) tagId=" + tagId + " and isAutoArchive=" + isAutoArchive + " are ignored";
			} else {
				output = " --- (PAM: 4.x) tagId=" + tagId + " and isAutoArchive=" + isAutoArchive;
			}

			System.out.print(output);
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			res = itpamWebServiceClient.startProcess(importSrfFileName, null, null, null, tagId, isAutoArchive, null, null, null);
			System.out.println(" --- PASSED (Id:" + res + ")");
			
		} catch (Exception ex) {
			System.out.println(" --- FAILED.");
			failed = true;
		}
		
		return res;
	}
	
	private static void getProcessStatus (ItpamWebServiceClientFactory itpamWebServiceClientFactory, String strItpamVersion, String strItpamUri, String strItpamUser,
			String strItpamPassword,String tagId) {
		try {
			String output = "";
			if(strItpamVersion.startsWith("3")) {
				output = " --- (PAM: 3.x) tagId=" + tagId + " was never set";
			} else {
				output = " --- (PAM: 4.x) tagId=" + tagId + " was set";
			}

			System.out.print(output);
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			ItpamProcessStatus res = itpamWebServiceClient.getProcessStatus(0, tagId);
			System.out.println(" --- PASSED. (Id:" + res.getProcessId() + " / Status:" + res.getProcessState() + ")");
			
		} catch (Exception ex) {
			if(strItpamVersion.startsWith("3")) {
				System.out.println(" --- PASSED. (Unable to retrieve status using TagId)");
			} else {
				System.out.println(" --- FAILED.");
				failed = true;
			}
			
		}
		
	}
	
	private static void controlInstance (ItpamWebServiceClientFactory itpamWebServiceClientFactory, String strItpamVersion, String strItpamUri, String strItpamUser,
			String strItpamPassword,String rootUuid, long instanceId, String tagId) {
		try {
			String output = "";
			if(strItpamVersion.startsWith("3")) {
				output = " --- (PAM: 3.x)";
			} else {
				output = " --- (PAM: 4.x) rootUuid=" + rootUuid + ", instanceId=" + instanceId + ", tagId=" + tagId;
			}

			System.out.print(output);
			ItpamWebServiceClient itpamWebServiceClient = itpamWebServiceClientFactory.getInstanceOf(strItpamVersion, strItpamUri, strItpamUser, strItpamPassword);
			itpamWebServiceClient.controlInstance(rootUuid, instanceId, tagId);
			System.out.println(" --- PASSED.");
			
		} catch (Exception ex) {
			if(strItpamVersion.startsWith("3")) {
				System.out.println(" --- PASSED. (SOAP method is not supported in PAM 3.x)");
			} else {
				System.out.println(" --- FAILED.");
				failed = true;
			}
			
		}
		
	}
}

